@@ -3,7 +3,9 @@
  *                                                                             *
  * libretroshare: retroshare core library                                      *
  *                                                                             *
- * Copyright 2018 by Mr.Alice <mralice@users.sourceforge.net>                  *
+ * Copyright (C) 2018  Mr.Alice <mralice@users.sourceforge.net>                *
+ * Copyright (C) 2021  Gioacchino Mazzurco <gio@eigenlab.org>                  *
+ * Copyright (C) 2021  Asociación Civil Altermundi <info@altermundi.net>       *
  *                                                                             *
  * This program is free software: you can redistribute it and/or modify        *
  * it under the terms of the GNU Lesser General Public License as              *
@@ -30,7 +32,7 @@
 #include "retroshare/rsids.h"
 #include "retroshare/rspeers.h"
 #include "retroshare/rsinit.h"
-
+#include "util/cxx17retrocompat.h"
 #include "rsserver/p3face.h"
 
 #define P3FILELISTS_DEBUG() std::cerr << time(NULL)    << " : FILE_LISTS : " << __FUNCTION__ << " : "
@@ -65,6 +67,7 @@ p3FileDatabase::p3FileDatabase(p3ServiceControl *mpeers)
     mRemoteDirectories.clear() ;	// we should load them!
     mOwnId = mpeers->getOwnId() ;
 
+    mBannedFileListNeedsUpdate = false;
     mLocalSharedDirs = new LocalDirectoryStorage(mFileSharingDir + "/" + LOCAL_SHARED_DIRS_FILE_NAME,mOwnId);
     mHashCache = new HashStorage(mFileSharingDir + "/" + HASH_CACHE_FILE_NAME) ;
 
@@ -88,7 +91,7 @@ void p3FileDatabase::setIgnoreLists(const std::list<std::string>& ignored_prefix
     RS_STACK_MUTEX(mFLSMtx) ;
     mLocalDirWatcher->setIgnoreLists(ignored_prefixes,ignored_suffixes,ignore_flags) ;
 
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 }
 bool p3FileDatabase::getIgnoreLists(std::list<std::string>& ignored_prefixes,std::list<std::string>& ignored_suffixes, uint32_t& ignore_flags)
 {
@@ -113,11 +116,19 @@ void p3FileDatabase::setSharedDirectories(const std::list<SharedDirInfo>& shared
         RS_STACK_MUTEX(mFLSMtx) ;
 
         mLocalSharedDirs->setSharedDirectoryList(shared_dirs) ;
-        mLocalDirWatcher->forceUpdate();
+        mLocalDirWatcher->forceUpdate(false);
 
     }
 
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
+
+    if(rsEvents)
+    {
+        auto ev = std::make_shared<RsSharedDirectoriesEvent>();
+        ev->mEventCode = RsSharedDirectoriesEventCode::SHARED_DIRS_LIST_CHANGED;
+        rsEvents->postEvent(ev);
+    }
+
 }
 void p3FileDatabase::getSharedDirectories(std::list<SharedDirInfo>& shared_dirs)
 {
@@ -132,7 +143,14 @@ void p3FileDatabase::updateShareFlags(const SharedDirInfo& info)
     }
 
     RsServer::notify()->notifyListChange(NOTIFY_LIST_DIRLIST_LOCAL, 0);
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
+
+    if(rsEvents)
+    {
+        auto ev = std::make_shared<RsSharedDirectoriesEvent>();
+        ev->mEventCode = RsSharedDirectoriesEventCode::SHARED_DIRS_LIST_CHANGED;
+        rsEvents->postEvent(ev);
+    }
 }
 
 p3FileDatabase::~p3FileDatabase()
@@ -204,7 +222,7 @@ int p3FileDatabase::tick()
 
     if(mUpdateFlags)
     {
-        IndicateConfigChanged();
+        IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 
         if(mUpdateFlags & P3FILELISTS_UPDATE_FLAG_LOCAL_DIRS_CHANGED)
             RsServer::notify()->notifyListChange(NOTIFY_LIST_DIRLIST_LOCAL, 0);
@@ -316,10 +334,25 @@ cleanup = true;
     {
         RS_STACK_MUTEX(mFLSMtx) ;
 
-        RsFileListsBannedHashesConfigItem *item = new RsFileListsBannedHashesConfigItem ;
+        // We need to be careful not to exceed the max size limit for a RsItem.
 
-        item->primary_banned_files_list = mPrimaryBanList;
-        sList.push_back(item) ;
+        RsFileListsBannedHashesConfigItem *item = nullptr;
+
+        for(auto it(mPrimaryBanList.begin());it!=mPrimaryBanList.end();++it)
+        {
+            if(item == nullptr)
+                item = new RsFileListsBannedHashesConfigItem ;
+
+            item->primary_banned_files_list.insert(*it);
+
+            if(item->primary_banned_files_list.size() > 1000)	// safe bet for size
+            {
+                sList.push_back(item) ;
+                item = nullptr;
+            }
+        }
+        if(item != nullptr)
+            sList.push_back(item) ;
     }
 
     RsConfigKeyValueSet *rskv = new RsConfigKeyValueSet();
@@ -452,6 +485,7 @@ bool p3FileDatabase::loadList(std::list<RsItem *>& load)
 	ignored_suffixes.push_back( "~" );
 	ignored_suffixes.push_back( ".part" );
 #endif
+    mPrimaryBanList.clear();
 
     for(std::list<RsItem *>::iterator it = load.begin(); it != load.end(); ++it)
     {
@@ -567,7 +601,7 @@ bool p3FileDatabase::loadList(std::list<RsItem *>& load)
 
         if(fb)
         {
-            mPrimaryBanList = fb->primary_banned_files_list ;
+            mPrimaryBanList.insert(fb->primary_banned_files_list.begin(),fb->primary_banned_files_list.end()) ;
             mBannedFileListNeedsUpdate = true;
             mLastPrimaryBanListChangeTimeStamp = time(NULL);
         }
@@ -701,7 +735,7 @@ void p3FileDatabase::cleanup()
 			std::cerr << "(WW) Initialising directory watcher salt to some random value " << std::endl;
 			mLocalDirWatcher->setHashSalt(RsFileHash::random()) ;
 
-            IndicateConfigChanged();
+            IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_NOW);
 		}
     }
 }
@@ -752,7 +786,7 @@ uint32_t p3FileDatabase::locked_getFriendIndex(const RsPeerId& pid)
 
         mFriendIndexMap[pid] = found;
 
-        IndicateConfigChanged();
+        IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 
         return found ;
     }
@@ -968,65 +1002,58 @@ int p3FileDatabase::getSharedDirStatistics(const RsPeerId& pid,SharedDirStats& s
     }
 }
 
-void p3FileDatabase::removeExtraFile(const RsFileHash& hash)
+bool p3FileDatabase::removeExtraFile(const RsFileHash& hash)
 {
+	bool ret = false;
     {
     RS_STACK_MUTEX(mFLSMtx) ;
 
-    mExtraFiles->removeExtraFile(hash);
+	ret = mExtraFiles->removeExtraFile(hash);
     mLastExtraFilesCacheUpdate = 0 ; // forced cache reload
     }
 
     RsServer::notify()->notifyListChange(NOTIFY_LIST_DIRLIST_LOCAL, 0);
+	return ret;
 }
 
-void p3FileDatabase::getExtraFilesDirDetails(void *ref,DirectoryStorage::EntryIndex e,DirDetails& d) const
+void p3FileDatabase::getExtraFilesDirDetails_locked(void *ref,DirectoryStorage::EntryIndex e,DirDetails& d) const
 {
-    // update the cache of extra files if last requested too long ago
+    std::vector<FileInfo> tmp_extrafiles;	// cache for extra files, to avoid requesting them too often.
+    mExtraFiles->getExtraFileList(tmp_extrafiles);
 
-    rstime_t now = time(NULL);
-
-    if(mLastExtraFilesCacheUpdate + DELAY_BETWEEN_EXTRA_FILES_CACHE_UPDATES <= now)
-    {
-        mExtraFiles->getExtraFileList(mExtraFilesCache);
-        mLastExtraFilesCacheUpdate = now;
-    }
-
-    //
-
-	if(e == 0)	// "virtual extra files directory" => create a dir with as many child as they are extra files
+    if(e == 0)	// "virtual extra files directory" => create a dir with as many child as they are extra files
 	{
 		d.parent = NULL ;
 
 		d.prow = 0;//fi-1 ;
 		d.type = DIR_TYPE_PERSON;
 		d.hash.clear() ;
-		d.count   = mExtraFilesCache.size();
+        d.size   = tmp_extrafiles.size();
 		d.max_mtime = time(NULL);
 		d.mtime     = time(NULL);
 		d.name = "[Extra List]";
 		d.path    = "/";
 		d.ref     = ref ;
 
-        for(uint32_t i=0;i<mExtraFilesCache.size();++i)
+        for(uint32_t i=0;i<tmp_extrafiles.size();++i)
 		{
 			DirStub stub;
 			stub.type = DIR_TYPE_FILE;
-			stub.name = mExtraFilesCache[i].fname;
+            stub.name = tmp_extrafiles[i].fname;
 			convertEntryIndexToPointer<sizeof(void*)>(i+1,1,stub.ref);	// local shared files from extra list
 
 			d.children.push_back(stub);
 		}
 	}
-	else	// extra file. Just query the corresponding data from ftExtra
+    else if(e < tmp_extrafiles.size() + 1)	// extra file. Just query the corresponding data from ftExtra
 	{
 		d.prow = 1;//fi-1 ;
 		d.type = DIR_TYPE_EXTRA_FILE;
 
-        FileInfo& f(mExtraFilesCache[(int)e-1]) ;
+        FileInfo& f(tmp_extrafiles[(int)e-1]) ;
 
 		d.hash      = f.hash;
-		d.count     = f.size;
+        d.size     = f.size;
 		d.max_mtime = 0;			// this is irrelevant
 		d.mtime     = 0;			// this is irrelevant
 		d.name      = f.path;		// so that the UI shows the complete path, since the parent directory is not really a directory.
@@ -1035,37 +1062,52 @@ void p3FileDatabase::getExtraFilesDirDetails(void *ref,DirectoryStorage::EntryIn
 
 		convertEntryIndexToPointer<sizeof(void*)>(0,1,d.parent) ;
 	}
+    else
+    {
+        d.ref = nullptr;
+        d.parent = nullptr ;
+
+        d.prow = 0;//fi-1 ;
+        d.type = DIR_TYPE_UNKNOWN;
+        d.hash.clear() ;
+        d.size   = 0;
+    }
 
     d.flags = DIR_FLAGS_ANONYMOUS_DOWNLOAD ;
 	d.id = RsPeerId();
 }
 
 // This function converts a pointer into directory details, to be used by the AbstractItemModel for browsing the files.
-int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags flags) const
+int p3FileDatabase::RequestDirDetails(
+        void* ref, DirDetails& d, FileSearchFlags flags ) const
 {
-    RS_STACK_MUTEX(mFLSMtx) ;
+	RS_STACK_MUTEX(mFLSMtx);
 
-    d.children.clear();
+	d.children.clear();
 
-    // Case where the pointer is NULL, which means we're at the top of the list of shared directories for all friends (including us)
-    // or at the top of our own list of shared directories, depending on the flags.
+	/* Case where the pointer is NULL, which means we're at the top of the list
+	 * of shared directories for all friends (including us) or at the top of our
+	 * own list of shared directories, depending on the flags.
+	 *
+	 * Friend index is used as follows:
+	 *  0		: own id
+	 *  1...n	: other friends
+	 *
+	 *  entry_index: starts at 0.
+	 *
+	 * The point is: we cannot use (0,0) because it encodes to NULL. No existing
+	 * combination should encode to NULL.
+	 * So we need to properly convert the friend index into 0 or into a friend
+	 * tab index in mRemoteDirectories.
+	 *
+	 * We should also check the consistency between flags and the content of ref.
+	 */
 
-    // Friend index is used as follows:
-    //	0		: own id
-    //  1...n	: other friends
-    //
-    // entry_index: starts at 0.
-    //
-    // The point is: we cannot use (0,0) because it encodes to NULL. No existing combination should encode to NULL.
-    // So we need to properly convert the friend index into 0 or into a friend tab index in mRemoteDirectories.
-    //
-    // We should also check the consistency between flags and the content of ref.
-
-    if (ref == NULL)
-    {
-        d.ref = NULL ;
-        d.type = DIR_TYPE_ROOT;
-        d.parent = NULL;
+	if (ref == nullptr)
+	{
+		d.ref = nullptr;
+		d.type = DIR_TYPE_ROOT;
+		d.parent = nullptr;
         d.prow = -1;
         d.name = "root";
         d.hash.clear() ;
@@ -1075,12 +1117,13 @@ int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags
         d.max_mtime = 0 ;
 
         if(flags & RS_FILE_HINTS_LOCAL)
-        {
-            void *p;
+		{
+			void *p = nullptr;
 
-            {
-			convertEntryIndexToPointer<sizeof(void*)>(0,0,p);	// root of own directories
-            DirStub stub;
+			{
+				// root of own directories
+				convertEntryIndexToPointer<sizeof(void*)>(0, 0, p);
+				DirStub stub;
             stub.type = DIR_TYPE_PERSON;
             stub.name = mServCtrl->getOwnId().toStdString();
             stub.ref  = p;
@@ -1089,9 +1132,11 @@ int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags
 
             if(mExtraFiles->size() > 0)
             {
-            convertEntryIndexToPointer<sizeof(void*)>(0,1,p);	// local shared files from extra list
-            DirStub stub;
-            stub.type = DIR_TYPE_PERSON;						// not totally exact, but used as a trick.
+				// local shared files from extra list
+				convertEntryIndexToPointer<sizeof(void*)>(0, 1, p);
+				DirStub stub;
+				// not totally exact, but used as a trick.
+				stub.type = DIR_TYPE_PERSON;
             stub.name = "[Extra List]";
             stub.ref  = p;
 
@@ -1113,7 +1158,7 @@ int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags
                d.children.push_back(stub);
             }
 
-        d.count = d.children.size();
+        d.size = 0; // non documented
 
 #ifdef DEBUG_FILE_HIERARCHY
         P3FILELISTS_DEBUG() << "ExtractData: ref=" << ref << ", flags=" << flags << " : returning this: " << std::endl;
@@ -1124,20 +1169,24 @@ int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags
     }
 
     uint32_t fi;
-    DirectoryStorage::EntryIndex e ;
+	DirectoryStorage::EntryIndex e;
 
     convertPointerToEntryIndex<sizeof(void*)>(ref,e,fi);
 
-    // check consistency
-    if( (fi == 0 && !(flags & RS_FILE_HINTS_LOCAL)) ||  (fi > 1 && (flags & RS_FILE_HINTS_LOCAL)))
-    {
-        P3FILELISTS_ERROR() << "(EE) remote request on local index or local request on remote index. This should not happen." << std::endl;
-        return false ;
-    }
+	// check consistency
+	if( (fi == 0 && !(flags & RS_FILE_HINTS_LOCAL)) ||
+	        (fi > 1 && (flags & RS_FILE_HINTS_LOCAL)))
+	{
+		RS_ERR("Remote request on local index or local request on remote index");
+		return false;
+	}
 
-    if((flags & RS_FILE_HINTS_LOCAL) && fi == 1) // extra list
+	if((flags & RS_FILE_HINTS_LOCAL) && fi == 1) // extra list
     {
-        getExtraFilesDirDetails(ref,e,d);
+        getExtraFilesDirDetails_locked(ref,e,d);
+
+        if(d.ref == nullptr)
+            return false;
 
 #ifdef DEBUG_FILE_HIERARCHY
 		P3FILELISTS_DEBUG() << "ExtractData: ref=" << ref << ", flags=" << flags << " : returning this: " << std::endl;
@@ -1147,25 +1196,28 @@ int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags
     }
 
 
-    DirectoryStorage *storage = (flags & RS_FILE_HINTS_LOCAL)? ((DirectoryStorage*)mLocalSharedDirs) : ((DirectoryStorage*)mRemoteDirectories[fi-1]);
+	DirectoryStorage* storage =
+	        (flags & RS_FILE_HINTS_LOCAL) ?
+	            ((DirectoryStorage*) mLocalSharedDirs) :
+	            ((DirectoryStorage*) mRemoteDirectories[fi-1]);
 
-    // Case where the index is the top of a single person. Can be us, or a friend.
+	/* Case where the index is the top of a single person.
+	 * Can be us, or a friend. */
+	if(!storage || !storage->extractData(e,d))
+	{
+		RS_WARN( "request on index; ", e, ", for directory ID:",
+		         ( (!storage)? ("[NULL]") : (storage->peerId().toStdString()) ),
+		         " failed" );
+		return false;
+	}
 
-    if(storage==NULL || !storage->extractData(e,d))
-    {
-#ifdef DEBUG_FILE_HIERARCHY
-        P3FILELISTS_DEBUG() << "(WW) request on index " << e << ", for directory ID=" << ((storage==NULL)?("[NULL]"):(storage->peerId().toStdString())) << " failed. This should not happen." << std::endl;
-#endif
-        return false ;
-    }
+	/* update indexes. This is a bit hacky, but does the job. The cast to
+	 * intptr_t is the proper way to convert a pointer into an int. */
+	convertEntryIndexToPointer<sizeof(void*)>((intptr_t)d.ref,fi,d.ref);
 
-    // update indexes. This is a bit hacky, but does the job. The cast to intptr_t is the proper way to convert
-    // a pointer into an int.
-
-    convertEntryIndexToPointer<sizeof(void*)>((intptr_t)d.ref,fi,d.ref) ;
-
-    for(uint32_t i=0;i<d.children.size();++i)
-        convertEntryIndexToPointer<sizeof(void*)>((intptr_t)d.children[i].ref,fi,d.children[i].ref);
+	for(uint32_t i=0; i<d.children.size(); ++i)
+		convertEntryIndexToPointer<sizeof(void*)>(
+		            (intptr_t) d.children[i].ref, fi, d.children[i].ref );
 
     if(e == 0)	// root
     {
@@ -1175,9 +1227,9 @@ int p3FileDatabase::RequestDirDetails(void *ref, DirDetails& d, FileSearchFlags
     else
     {
         if(d.parent == 0)	 // child of root node
-            d.prow = (flags & RS_FILE_HINTS_LOCAL)?0:(fi-1);
+			d.prow = (flags & RS_FILE_HINTS_LOCAL) ? 0 : (fi-1);
         else
-            d.prow = storage->parentRow(e) ;
+			d.prow = storage->parentRow(e);
 
         convertEntryIndexToPointer<sizeof(void*)>((intptr_t)d.parent,fi,d.parent) ;
     }
@@ -1228,9 +1280,9 @@ uint32_t p3FileDatabase::getType(void *ref,FileSearchFlags flags) const
     }
 }
 
-void p3FileDatabase::forceDirectoryCheck()              // Force re-sweep the directories and see what's changed
+void p3FileDatabase::forceDirectoryCheck(bool add_safe_delay)              // Force re-sweep the directories and see what's changed
 {
-    mLocalDirWatcher->forceUpdate();
+    mLocalDirWatcher->forceUpdate(add_safe_delay);
 }
 void p3FileDatabase::togglePauseHashingProcess()
 {
@@ -1251,7 +1303,7 @@ void p3FileDatabase::setFollowSymLinks(bool b)
 {
     RS_STACK_MUTEX(mFLSMtx) ;
     mLocalDirWatcher->setFollowSymLinks(b) ;
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 }
 bool p3FileDatabase::followSymLinks() const
 {
@@ -1262,7 +1314,7 @@ void p3FileDatabase::setIgnoreDuplicates(bool i)
 {
     RS_STACK_MUTEX(mFLSMtx) ;
     mLocalDirWatcher->setIgnoreDuplicates(i) ;
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 }
 bool p3FileDatabase::ignoreDuplicates() const
 {
@@ -1273,7 +1325,7 @@ void p3FileDatabase::setMaxShareDepth(int i)
 {
     RS_STACK_MUTEX(mFLSMtx) ;
     mLocalDirWatcher->setMaxShareDepth(i) ;
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 }
 int  p3FileDatabase::maxShareDepth() const
 {
@@ -1284,7 +1336,7 @@ void p3FileDatabase::setWatchEnabled(bool b)
 {
     RS_STACK_MUTEX(mFLSMtx) ;
     mLocalDirWatcher->setEnabled(b) ;
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 }
 bool p3FileDatabase::watchEnabled()
 {
@@ -1296,7 +1348,7 @@ void p3FileDatabase::setWatchPeriod(uint32_t seconds)
     RS_STACK_MUTEX(mFLSMtx) ;
 
     mLocalDirWatcher->setFileWatchPeriod(seconds);
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
 }
 uint32_t p3FileDatabase::watchPeriod()
 {
@@ -1304,7 +1356,9 @@ uint32_t p3FileDatabase::watchPeriod()
     return mLocalDirWatcher->fileWatchPeriod();
 }
 
-int p3FileDatabase::SearchKeywords(const std::list<std::string>& keywords, std::list<DirDetails>& results,FileSearchFlags flags,const RsPeerId& client_peer_id)
+int p3FileDatabase::SearchKeywords(
+        const std::list<std::string>& keywords, std::list<DirDetails>& results,
+        FileSearchFlags flags, const RsPeerId& client_peer_id )
 {
     if(flags & RS_FILE_HINTS_LOCAL)
     {
@@ -1316,15 +1370,15 @@ int p3FileDatabase::SearchKeywords(const std::list<std::string>& keywords, std::
 
             mLocalSharedDirs->searchTerms(keywords,firesults) ;
 
-            for(std::list<EntryIndex>::iterator it(firesults.begin());it!=firesults.end();++it)
-            {
-                void *p=NULL;
-                convertEntryIndexToPointer<sizeof(void*)>(*it,0,p);
-				pointers.push_back(p) ;
-            }
+			for(auto& it: std::as_const(firesults))
+			{
+				void *p = nullptr;
+				convertEntryIndexToPointer<sizeof(void*)>(it, 0, p);
+				pointers.push_back(p);
+			}
         }
 
-        filterResults(pointers,results,flags,client_peer_id) ;
+		filterResults(pointers, results, flags, client_peer_id);
     }
 
     if(flags & RS_FILE_HINTS_REMOTE)
@@ -1421,9 +1475,15 @@ int p3FileDatabase::SearchBoolExp(RsRegularExpression::Expression *exp, std::lis
     return !results.empty() ;
 
 }
-bool p3FileDatabase::search(const RsFileHash &hash, FileSearchFlags hintflags, FileInfo &info) const
+
+bool p3FileDatabase::search(
+        const RsFileHash &hash, FileSearchFlags hintflags, FileInfo &info) const
 {
-    RS_STACK_MUTEX(mFLSMtx) ;
+	RS_STACK_MUTEX(mFLSMtx);
+
+	if( (hintflags & RS_FILE_HINTS_EXTRA) &&
+	        mExtraFiles->search(hash, hintflags, info) )
+		return true;
 
     if(hintflags & RS_FILE_HINTS_LOCAL)
     {
@@ -1467,7 +1527,9 @@ bool p3FileDatabase::search(const RsFileHash &hash, FileSearchFlags hintflags, F
     return false;
 }
 
-int p3FileDatabase::filterResults(const std::list<void*>& firesults,std::list<DirDetails>& results,FileSearchFlags flags,const RsPeerId& peer_id) const
+int p3FileDatabase::filterResults(
+        const std::list<void*>& firesults, std::list<DirDetails>& results,
+        FileSearchFlags flags, const RsPeerId& peer_id ) const
 {
     results.clear();
 
@@ -1475,29 +1537,33 @@ int p3FileDatabase::filterResults(const std::list<void*>& firesults,std::list<Di
 
     /* translate/filter results */
 
-    for(std::list<void*>::const_iterator rit(firesults.begin()); rit != firesults.end(); ++rit)
-    {
-        DirDetails cdetails ;
+	for(void* rit: std::as_const(firesults))
+	{
+		DirDetails cdetails;
 
-        if(!RequestDirDetails (*rit,cdetails,RS_FILE_HINTS_LOCAL))
-        {
-            P3FILELISTS_ERROR() << "(EE) Cannot get dir details for entry " << *rit << std::endl;
-            continue ;
-        }
+		if(!RequestDirDetails(rit, cdetails, RS_FILE_HINTS_LOCAL))
+		{
+			RS_ERR("Cannot retrieve dir details for entry: ", rit);
+			print_stacktrace();
+			continue ;
+		}
+
+		RS_DBG( "Filtering candidate: ", rit,
+		        ", name: ", cdetails.name, ", path: ", cdetails.path,
+		        ", flags: ", cdetails.flags, ", peer: ", peer_id );
+
+		if(!peer_id.isNull())
+		{
+			FileSearchFlags permission_flags =
+			        rsPeers->computePeerPermissionFlags(
+			            peer_id, cdetails.flags, cdetails.parent_groups );
+
+			if (cdetails.type == DIR_TYPE_FILE && ( permission_flags & flags ))
+			{
+				cdetails.id.clear();
+				results.push_back(cdetails);
 #ifdef DEBUG_P3FILELISTS
-        P3FILELISTS_DEBUG() << "Filtering candidate " << *rit << ", flags=" << cdetails.flags << ", peer=" << peer_id ;
-#endif
-
-        if(!peer_id.isNull())
-        {
-            FileSearchFlags permission_flags = rsPeers->computePeerPermissionFlags(peer_id,cdetails.flags,cdetails.parent_groups) ;
-
-            if (cdetails.type == DIR_TYPE_FILE && ( permission_flags & flags ))
-            {
-                cdetails.id.clear() ;
-                results.push_back(cdetails);
-#ifdef DEBUG_P3FILELISTS
-                std::cerr << ": kept" << std::endl ;
+				std::cerr << ": kept" << std::endl ;
 #endif
             }
 #ifdef DEBUG_P3FILELISTS
@@ -1509,7 +1575,19 @@ int p3FileDatabase::filterResults(const std::list<void*>& firesults,std::list<Di
             results.push_back(cdetails);
     }
 
-    return !results.empty() ;
+	for(auto& res: results)
+	{
+		/* Most file will just have file name stored, but single file shared
+		 * without a shared dir will contain full path instead of just the
+		 * name, so purify it to perform the search */
+		if(res.name.find("/") != std::string::npos )
+		{
+			std::string _tDirPath;
+			RsDirUtil::splitDirFromFile(res.name, _tDirPath, res.name);
+		}
+	}
+
+	return !results.empty();
 }
 
 bool p3FileDatabase::convertSharedFilePath(const std::string& path,std::string& fullpath)
@@ -1583,78 +1661,90 @@ void p3FileDatabase::tickSend()
 
 void p3FileDatabase::handleDirSyncRequest(RsFileListsSyncRequestItem *item)
 {
-    RsFileListsSyncResponseItem *ritem = new RsFileListsSyncResponseItem;
+	RsFileListsSyncResponseItem* ritem = new RsFileListsSyncResponseItem;
 
-    // look at item TS. If local is newer, send the full directory content.
-    {
-        RS_STACK_MUTEX(mFLSMtx) ;
+	// look at item TS. If local is newer, send the full directory content.
+	{
+		RS_STACK_MUTEX(mFLSMtx);
 
-#ifdef DEBUG_P3FILELISTS
-        P3FILELISTS_DEBUG() << "Received directory sync request from peer " << item->PeerId() << ". hash=" << item->entry_hash << ", flags=" << (void*)(intptr_t)item->flags << ", request id: " << std::hex << item->request_id << std::dec << ", last known TS: " << item->last_known_recurs_modf_TS << std::endl;
-#endif
+		RS_DBG( "Received directory sync request from peer ", item->PeerId(),
+		        ". hash=", item->entry_hash, ", flags=", item->flags,
+		        ", request id: ", item->request_id, ", last known TS: ",
+		        item->last_known_recurs_modf_TS );
 
-        EntryIndex entry_index = DirectoryStorage::NO_INDEX;
+		EntryIndex entry_index = DirectoryStorage::NO_INDEX;
+		if(!mLocalSharedDirs->getIndexFromDirHash(item->entry_hash,entry_index))
+		{
+			RS_DBG("Cannot find entry index for hash ", item->entry_hash,
+			       " cannot respond to sync request." );
+			return;
+		}
 
-        if(!mLocalSharedDirs->getIndexFromDirHash(item->entry_hash,entry_index))
-        {
-#ifdef DEBUG_P3FILELISTS
-            P3FILELISTS_DEBUG() << "  (EE) Cannot find entry index for hash " << item->entry_hash << ": cannot respond to sync request." << std::endl;
-#endif
-            return;
-        }
+		uint32_t entry_type = mLocalSharedDirs->getEntryType(entry_index);
+		ritem->PeerId(item->PeerId());
+		ritem->request_id = item->request_id;
+		ritem->entry_hash = item->entry_hash;
 
-        uint32_t entry_type = mLocalSharedDirs->getEntryType(entry_index) ;
-        ritem->PeerId(item->PeerId()) ;
-        ritem->request_id = item->request_id;
-        ritem->entry_hash = item->entry_hash ;
+		std::list<RsNodeGroupId> node_groups;
+		FileStorageFlags node_flags;
 
-        std::list<RsNodeGroupId> node_groups;
-        FileStorageFlags node_flags;
+		if(entry_type != DIR_TYPE_DIR)
+		{
+			RS_DBG( "Directory does not exist anymore, or is not a directory, "
+			        "or permission denied. Answering with proper flags." );
+			ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE |
+			        RsFileListsItem::FLAGS_ENTRY_WAS_REMOVED;
+		}
+		else if( entry_index != 0 &&
+		         (!mLocalSharedDirs->getFileSharingPermissions(
+		              entry_index, node_flags,node_groups ) ||
+		          !(rsPeers->computePeerPermissionFlags(
+		                item->PeerId(), node_flags, node_groups ) &
+		            RS_FILE_HINTS_BROWSABLE)) )
+		{
+			RS_ERR("cannot get file permissions for entry index: ", entry_index,
+			       ", or permission denied." );
+			ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE |
+			        RsFileListsItem::FLAGS_ENTRY_WAS_REMOVED;
+		}
+		else
+		{
+			rstime_t local_recurs_max_time;
+			mLocalSharedDirs->getDirectoryRecursModTime(
+			            entry_index, local_recurs_max_time );
 
-        if(entry_type != DIR_TYPE_DIR)
-        {
-#ifdef DEBUG_P3FILELISTS
-            P3FILELISTS_DEBUG() << "  Directory does not exist anymore, or is not a directory, or permission denied. Answering with proper flags." << std::endl;
-#endif
-            ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE | RsFileListsItem::FLAGS_ENTRY_WAS_REMOVED ;
-        }
-        else if(entry_index != 0 && (!mLocalSharedDirs->getFileSharingPermissions(entry_index,node_flags,node_groups) || !(rsPeers->computePeerPermissionFlags(item->PeerId(),node_flags,node_groups) & RS_FILE_HINTS_BROWSABLE)))
-        {
-            P3FILELISTS_ERROR() << "(EE) cannot get file permissions for entry index " << (void*)(intptr_t)entry_index << ", or permission denied." << std::endl;
-            ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE | RsFileListsItem::FLAGS_ENTRY_WAS_REMOVED ;
-        }
-        else
-        {
-            rstime_t local_recurs_max_time ;
-            mLocalSharedDirs->getDirectoryRecursModTime(entry_index,local_recurs_max_time) ;
+			/* normally, should be "<", but since we provided the TS it should
+			 * be equal, so != is more robust. */
+			if(item->last_known_recurs_modf_TS != local_recurs_max_time)
+			{
+				RS_DBG( "Directory is more recent than what the friend knows. "
+				        "Sending full dir content as response.");
 
-            if(item->last_known_recurs_modf_TS != local_recurs_max_time)	// normally, should be "<", but since we provided the TS it should be equal, so != is more robust.
-            {
-#ifdef DEBUG_P3FILELISTS
-                P3FILELISTS_DEBUG() << "  Directory is more recent than what the friend knows. Sending full dir content as response." << std::endl;
-#endif
+				ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE |
+				        RsFileListsItem::FLAGS_SYNC_DIR_CONTENT;
+				ritem->last_known_recurs_modf_TS = local_recurs_max_time;
 
-                ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE | RsFileListsItem::FLAGS_SYNC_DIR_CONTENT;
-                ritem->last_known_recurs_modf_TS = local_recurs_max_time;
+				/* We supply the peer id, in order to possibly remove some
+				 * subdirs, if entries are not allowed to be seen by this peer.
+				 */
+				mLocalSharedDirs->serialiseDirEntry(
+				            entry_index, ritem->directory_content_data,
+				            item->PeerId() );
+			}
+			else
+			{
+				RS_DBG3( "Directory is up to date w.r.t. what the friend knows."
+				         " Sending ACK." );
 
-                // We supply the peer id, in order to possibly remove some subdirs, if entries are not allowed to be seen by this peer.
-                mLocalSharedDirs->serialiseDirEntry(entry_index,ritem->directory_content_data,item->PeerId()) ;
-            }
-            else
-            {
-#ifdef DEBUG_P3FILELISTS
-                P3FILELISTS_DEBUG() << "  Directory is up to date w.r.t. what the friend knows. Sending ACK." << std::endl;
-#endif
+				ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE |
+				        RsFileListsItem::FLAGS_ENTRY_UP_TO_DATE;
+				ritem->last_known_recurs_modf_TS = local_recurs_max_time;
+			}
+		}
+	}
 
-                ritem->flags = RsFileListsItem::FLAGS_SYNC_RESPONSE | RsFileListsItem::FLAGS_ENTRY_UP_TO_DATE ;
-                ritem->last_known_recurs_modf_TS = local_recurs_max_time ;
-            }
-        }
-    }
-
-    // sends the response.
-
-    splitAndSendItem(ritem) ;
+	// sends the response.
+	splitAndSendItem(ritem);
 }
 
 void p3FileDatabase::splitAndSendItem(RsFileListsSyncResponseItem *ritem)
@@ -1945,7 +2035,10 @@ p3FileDatabase::DirSyncRequestId p3FileDatabase::makeDirSyncReqId(const RsPeerId
     // This is kind of arbitrary. The important thing is that the same ID needs to be generated every time for a given (peer_id,entry index) pair, in a way
     // that cannot be brute-forced or reverse-engineered, which explains the random bias and the usage of the hash, that is itself random.
 
-    uint64_t r = random_bias ^ *((uint64_t*)tmp.toByteArray()) ;
+    uint64_t tmp2 ;
+    memcpy(&tmp2,tmp.toByteArray(),sizeof(uint64_t));
+
+    uint64_t r = random_bias ^ tmp2;
 
 #ifdef DEBUG_P3FILELISTS
     std::cerr << "Creating ID " << std::hex << r << std::dec << " from peer id " << peer_id << " and hash " << hash << std::endl;
@@ -2042,7 +2135,7 @@ bool p3FileDatabase::banFile(const RsFileHash& real_file_hash, const std::string
 		}
 	}
 
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_NOW);
 	return true;
 }
 bool p3FileDatabase::unbanFile(const RsFileHash& real_file_hash)
@@ -2057,7 +2150,7 @@ bool p3FileDatabase::unbanFile(const RsFileHash& real_file_hash)
         mBannedFileListNeedsUpdate = true ;
     }
 
-    IndicateConfigChanged();
+    IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_NOW);
     return true;
 }
 
@@ -2098,7 +2191,7 @@ void p3FileDatabase::setTrustFriendNodesForBannedFiles(bool b)
 {
 	if(b != mTrustFriendNodesForBannedFiles)
     {
-		IndicateConfigChanged();
+        IndicateConfigChanged(RsConfigMgr::CheckPriority::SAVE_OFTEN);
         mBannedFileListNeedsUpdate = true;
     }
 
