@@ -35,7 +35,7 @@
 #include "pqi/pqissllistener.h"
 
 #include "pqi/p3linkmgr.h"
-#include <retroshare/rspeers.h>
+#include "retroshare/rspeers.h"
 #include <retroshare/rsdht.h>
 #include <retroshare/rsbanlist.h>
 
@@ -372,9 +372,11 @@ int 	pqissl::status()
 	// tick......
 int	pqissl::tick()
 {
-	RsStackMutex stack(mSslMtx); /**** LOCKED MUTEX ****/
+	// there is no reason to lock pqissl mutex now 
+	// we will lock the mutex later if we actually need to call to ConnectAttempt
+	// RsStackMutex stack(mSslMtx); /**** LOCKED MUTEX ****/
 
-	//pqistreamer::tick();
+	// pqistreamer::tick();
 
 	// continue existing connection attempt.
 	if (!active)
@@ -385,7 +387,8 @@ int	pqissl::tick()
 #ifdef PQISSL_LOG_DEBUG 
 			rslog(RSL_DEBUG_BASIC, pqisslzone, "pqissl::tick() Continuing Connection Attempt!");
 #endif
-
+			// now lock pqissl mutex, that will take up to 10 ms
+			RsStackMutex stack(mSslMtx); /**** LOCKED MUTEX ****/
 			ConnectAttempt();
 			return 1;
 		}
@@ -713,6 +716,7 @@ int pqissl::Initiate_Connection()
 		switch (errno)
 		{
 		case EINPROGRESS:
+		case EWOULDBLOCK:
 			waiting = WAITING_SOCK_CONNECT;
 			sockfd = osock;
 			return 0;
@@ -860,10 +864,10 @@ int 	pqissl::Basic_Connection_Complete()
 	{
 		// error - reset socket.
 		// this is a definite bad socket!.
-		
+#ifdef PQISSL_LOG_DEBUG2
   		rslog(RSL_WARNING, pqisslzone, 
 	  	  "pqissl::Basic_Connection_Complete() Select ERROR(2)");
-		
+#endif
 		net_internal_close(sockfd);
 		sockfd=-1;
 		//reset();
@@ -1078,7 +1082,7 @@ int pqissl::SSL_Connection_Complete()
 	/* if we are passive - then accept! */
 	int err;
 
-	if (sslmode)
+	if (sslmode == PQISSL_ACTIVE)
 	{
 #ifdef PQISSL_LOG_DEBUG 
         rslog(RSL_DEBUG_BASIC, pqisslzone, "--------> Active Connect! Client side.");
@@ -1096,8 +1100,7 @@ int pqissl::SSL_Connection_Complete()
 	if (err != 1)
 	{
 		int serr = SSL_get_error(ssl_connection, err);
-		if ((serr == SSL_ERROR_WANT_READ) 
-				|| (serr == SSL_ERROR_WANT_WRITE))
+		if ((serr == SSL_ERROR_WANT_READ)  || (serr == SSL_ERROR_WANT_WRITE))
 		{
 #ifdef PQISSL_LOG_DEBUG 
   			rslog(RSL_DEBUG_BASIC, pqisslzone, 
@@ -1108,6 +1111,20 @@ int pqissl::SSL_Connection_Complete()
 			return 0;
 		}
 
+		if(rsEvents)
+		{
+			X509 *x509 = SSL_get_peer_certificate(ssl_connection);
+
+            if(x509)
+			{
+				auto ev = std::make_shared<RsAuthSslConnectionAutenticationEvent>();
+				ev->mSslId = RsX509Cert::getCertSslId(*x509);
+				ev->mErrorCode = RsAuthSslError::PEER_REFUSED_CONNECTION;
+
+				if(!ev->mSslId.isNull())
+					rsEvents->postEvent(ev);
+			}
+		}
 
 		std::string out;
 		rs_sprintf(out, "pqissl::SSL_Connection_Complete()\nIssues with SSL Connect(%d)!\n", err);
@@ -1115,9 +1132,6 @@ int pqissl::SSL_Connection_Complete()
 
 		rslog(RSL_WARNING, pqisslzone, out);
 
-		// attempt real error.
-		Extract_Failed_SSL_Certificate();
-
 		rslog(RSL_ALERT, pqisslzone, "pqissl::SSL_Connection_Complete() -> calling reset()");
 		reset_locked();
 		waiting = WAITING_FAIL_INTERFACE;
@@ -1132,159 +1146,94 @@ int pqissl::SSL_Connection_Complete()
 	return 1;
 }
 
-int 	pqissl::Extract_Failed_SSL_Certificate()
-{
-	std::cerr << "pqissl::Extract_Failed_SSL_Certificate() FAILED Connection due to Security Issues";
-	std::cerr << std::endl;
-
-#ifdef PQISSL_LOG_DEBUG 
-  	rslog(RSL_DEBUG_BASIC, pqisslzone, 
-	  "pqissl::Extract_Failed_SSL_Certificate()");
-#endif
-
-	// Get the Peer Certificate....
-	X509 *peercert = SSL_get_peer_certificate(ssl_connection);
-
-	if (peercert == NULL)
-	{
-  		rslog(RSL_WARNING, pqisslzone, 
-		  "pqissl::Extract_Failed_SSL_Certificate() Peer Didnt Give Cert");
-
-		std::cerr << "pqissl::Extract_Failed_SSL_Certificate() ERROR Peer Didn't Give Us Certificate";
-		std::cerr << std::endl;
-
-		return -1;
-	}
-
-#ifdef PQISSL_LOG_DEBUG 
-  	rslog(RSL_DEBUG_BASIC, pqisslzone, 
-	  "pqissl::Extract_Failed_SSL_Certificate() Have Peer Cert - Registering");
-#endif
-
-	std::cerr << "pqissl::Extract_Failed_SSL_Certificate() Passing FAILED Cert to AuthSSL for analysis";
-	std::cerr << std::endl;
-
-	// save certificate... (and ip locations)
-	// false for outgoing....
-	// we actually connected to remote_addr, 
-	// 	which could be 
-	//      (pqissl's case) sslcert->serveraddr or sslcert->localaddr.
-
-	RsPeerId sslid ;
-	getX509id(peercert, sslid) ;
-
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-	RsPgpId gpgid(getX509CNString(peercert->cert_info->issuer));
-	std::string sslcn = getX509CNString(peercert->cert_info->subject);
-#else
-	RsPgpId gpgid(getX509CNString(X509_get_issuer_name(peercert)));
-	std::string sslcn = getX509CNString(X509_get_subject_name(peercert));
-#endif
-
-	AuthSSL::getAuthSSL()->FailedCertificate(peercert, gpgid,sslid,sslcn,remote_addr, false);
-	mLinkMgr->notifyDeniedConnection(gpgid, sslid, sslcn, remote_addr, false);
-
-	return 1;
-}
-
-
-
-
 int pqissl::Authorise_SSL_Connection()
 {
-#ifdef PQISSL_DEBUG
-	std::cerr << __PRETTY_FUNCTION__ << std::endl;
-#endif
+	Dbg3() << __PRETTY_FUNCTION__ << std::endl;
 
-	if (time(NULL) > ssl_connect_timeout)
+	constexpr int failure = -1;
+
+	if (time(nullptr) > ssl_connect_timeout)
 	{
-		std::cerr << __PRETTY_FUNCTION__ << " Connection timed out reset!"
-		          << std::endl;
+		RsInfo() << __PRETTY_FUNCTION__ << " Connection timed out reset!"
+		         << std::endl;
 		reset_locked();
 	}
 
 	int err;
 	if (0 >= (err = SSL_Connection_Complete())) return err;
 
-#ifdef PQISSL_LOG_DEBUG 
-  	rslog(RSL_DEBUG_BASIC, pqisslzone, 
-	  "pqissl::Authorise_SSL_Connection() SSL_Connection_Complete");
-#endif
+	Dbg3() << __PRETTY_FUNCTION__ << "SSL_Connection_Complete success."
+	       << std::endl;
 
 	// reset switch.
 	waiting = WAITING_NOT;
 
-	X509 *peercert = SSL_get_peer_certificate(ssl_connection);
-
-	if (peercert == NULL)
+#ifdef RS_PQISSL_AUTH_DOUBLE_CHECK
+	X509* peercert = SSL_get_peer_certificate(ssl_connection);
+	if (!peercert)
 	{
-  		rslog(RSL_WARNING, pqisslzone, 
-		  "pqissl::Authorise_SSL_Connection() Peer Didnt Give Cert");
-
-		rslog(RSL_ALERT, pqisslzone, "pqissl::Authorise_Connection_Complete() -> calling reset()");
-		// Failed completely
-		reset_locked();
-		return -1;
+		RsFatal() << __PRETTY_FUNCTION__ << " failed to retrieve peer "
+		          << "certificate at this point this should never happen!"
+		          << std::endl;
+		print_stacktrace();
+		exit(failure);
 	}
 
-        RsPeerId certPeerId;
-        getX509id(peercert, certPeerId);
-        if (RsPeerId(certPeerId) != PeerId()) {
-                rslog(RSL_WARNING, pqisslzone,
-                  "pqissl::Authorise_SSL_Connection() the cert Id doesn't match the Peer id we're trying to connect to.");
-
-		rslog(RSL_ALERT, pqisslzone, "pqissl::Authorise_Connection_Complete() -> calling reset()");
-                // Failed completely
-                reset_locked();
-                return -1;
-        }
-
-#ifdef PQISSL_LOG_DEBUG 
-  	rslog(RSL_DEBUG_BASIC, pqisslzone, 
-	  "pqissl::Authorise_SSL_Connection() Have Peer Cert");
-#endif
-
-	// save certificate... (and ip locations)
-	// false for outgoing....
-	// we actually connected to remote_addr, 
-	// 	which could be 
-	//      (pqissl's case) sslcert->serveraddr or sslcert->localaddr.
-
-	AuthSSL::getAuthSSL()->CheckCertificate(PeerId(), peercert);
-	bool certCorrect = true; /* WE know it okay already! */
-
-    uint32_t check_result ;
-    uint32_t checking_flags = RSBANLIST_CHECKING_FLAGS_BLACKLIST;
-    if (rsPeers->servicePermissionFlags(PeerId()) & RS_NODE_PERM_REQUIRE_WL)
-        checking_flags |= RSBANLIST_CHECKING_FLAGS_WHITELIST;
-
-    if(rsBanList!=NULL && !rsBanList->isAddressAccepted(remote_addr,checking_flags,&check_result))
-    {
-		std::cerr << "(SS) refusing connection attempt from IP address " << sockaddr_storage_iptostring(remote_addr) << ". Reason: " <<
-        ((check_result == RSBANLIST_CHECK_RESULT_NOT_WHITELISTED)?"not whitelisted (peer requires whitelist)":"blacklisted") << std::endl;
-            
-        RsServer::notify()->AddFeedItem(RS_FEED_ITEM_SEC_IP_BLACKLISTED, PeerId().toStdString(), sockaddr_storage_iptostring(remote_addr), "", "", check_result);
-		reset_locked();
-		return 0 ;
-    }
-	// check it's the right one.
-	if (certCorrect)
+	RsPeerId certPeerId = RsX509Cert::getCertSslId(*peercert);
+	if (RsPeerId(certPeerId) != PeerId())
 	{
-		// then okay...
-		rslog(RSL_WARNING, pqisslzone, "pqissl::Authorise_SSL_Connection() Accepting Conn. Peer: " + PeerId().toStdString());
+		RsErr() << __PRETTY_FUNCTION__ << " the cert Id doesn't match the peer "
+		        << "id we're trying to connect to." << std::endl;
 
-        //std::cerr << "pqissl::Authorise_SSL_Connection(): accepting connection from " << sockaddr_storage_iptostring(remote_addr) << std::endl;
-		accept_locked(ssl_connection, sockfd, remote_addr);
-		return 1;
+		/* TODO: Considering how difficult is managing to get a connection to a
+		 * friend nowadays on the Internet because of evil NAT everywhere.
+		 * If the cert is from a friend anyway we should find a way to make good
+		 * use of this connection instead of throwing it away... */
+
+		X509_free(peercert);
+		reset_locked();
+		return failure;
 	}
 
-	rslog(RSL_WARNING, pqisslzone, "pqissl::Authorise_SSL_Connection() Something Wrong ... Shutdown. Peer: " + PeerId().toStdString());
+	/* At this point the actual connection authentication has already been
+	 * performed in AuthSSL::VerifyX509Callback, any furter authentication check
+	 * like the followings are redundant. */
 
-	// else shutdown ssl connection.
-	rslog(RSL_ALERT, pqisslzone, "pqissl::Authorise_Connection_Complete() -> calling reset()");
+	bool isSslOnlyFriend = rsPeers->isSslOnlyFriend(certPeerId);
 
-	reset_locked();
-	return 0;
+	uint32_t authErrCode = 0;
+	if( !isSslOnlyFriend && !AuthSSL::instance().AuthX509WithGPG(peercert,false, authErrCode) )
+	{
+		RsFatal() << __PRETTY_FUNCTION__ << " failure verifying peer "
+		          << "certificate signature. This should never happen at this "
+		          << "point!" << std::endl;
+		print_stacktrace();
+
+		X509_free(peercert); // not needed but just in case we change to return
+		exit(failure);
+	}
+
+	RsPgpId pgpId = RsX509Cert::getCertIssuer(*peercert);
+    if( !isSslOnlyFriend && pgpId != AuthPGP::getPgpOwnId() && !AuthPGP::isPGPAccepted(pgpId) )
+	{
+		RsFatal() << __PRETTY_FUNCTION__ << " pgpId: " << pgpId
+		          << " is not friend. It is very unlikely to happen at this "
+		          << "point! Either the user must have been so fast to deny "
+		          << "friendship just after VerifyX509Callback have returned "
+		          << "success and just before this code being executed, or "
+		          << "something really fishy is happening! Share the full log "
+		          << "with developers." << std::endl;
+		print_stacktrace();
+
+		X509_free(peercert); // not needed but just in case we change to return
+		exit(failure);
+	}
+#endif // def RS_PQISSL_AUTH_REDUNDANT_CHECK
+
+	Dbg2() << __PRETTY_FUNCTION__ << " Accepting connection to peer: "
+	       << PeerId() << " with address: " << remote_addr << std::endl;
+
+	return accept_locked(ssl_connection, sockfd, remote_addr);
 }
 
 
@@ -1300,43 +1249,59 @@ int pqissl::accept( SSL *ssl, int fd,
 	return accept_locked(ssl, fd, foreign_addr);
 }
 
-int	pqissl::accept_locked( SSL *ssl, int fd,
+int pqissl::accept_locked( SSL *ssl, int fd,
                            const sockaddr_storage &foreign_addr )
 {
-#ifdef PQISSL_DEBUG
-	std::cerr << __PRETTY_FUNCTION__ << std::endl;
-#endif
+	Dbg3() << __PRETTY_FUNCTION__ << std::endl;
 
+	constexpr int failure = -1;
+	constexpr int success = 1;
+
+#ifdef RS_PQISSL_BANLIST_DOUBLE_CHECK
+	/* At this point, as we are actively attempting the connection, we decide
+	 * the address to which to connect to, banned addresses should never get
+	 * here as the filtering for banned addresses happens much before, this
+	 * check is therefore redundant, and if it trigger something really fishy
+	 * must be happening (a bug somewhere else in the code). */
 	uint32_t check_result;
 	uint32_t checking_flags = RSBANLIST_CHECKING_FLAGS_BLACKLIST;
 
 	if (rsPeers->servicePermissionFlags(PeerId()) & RS_NODE_PERM_REQUIRE_WL)
 		checking_flags |= RSBANLIST_CHECKING_FLAGS_WHITELIST;
 
-	if( rsBanList && !rsBanList->isAddressAccepted( foreign_addr,
-	                                                checking_flags,
-	                                                &check_result ) )
+    if( RsX509Cert::getCertSslId(*SSL_get_peer_certificate(ssl)) != PeerId())
+        std::cerr << "(EE) pqissl::accept_locked(): PeerId() is " << PeerId() << " but certificate ID is " << RsX509Cert::getCertSslId(*SSL_get_peer_certificate(ssl)) << std::endl;
+
+	if(rsBanList && !rsBanList->isAddressAccepted( foreign_addr, checking_flags, check_result ))
 	{
-		std::cerr << __PRETTY_FUNCTION__
-		          << " (SS) refusing incoming SSL connection from blacklisted "
-		          << "foreign address "
-		          << sockaddr_storage_iptostring(foreign_addr)
-		          << ". Reason: " << check_result << "." << std::endl;
+		RsInfo() << __PRETTY_FUNCTION__
+		        << " Refusing incoming SSL connection from blacklisted "
+		        << "foreign address " << foreign_addr
+		        << ". Reason: " << check_result << ". This should never happen "
+		        << "at this point! Please report full log to developers!"
+		        << std::endl;
+		print_stacktrace();
+
+		if(rsEvents)
+		{
+			X509 *x509 = SSL_get_peer_certificate(ssl);
+			auto ev = std::make_shared<RsAuthSslConnectionAutenticationEvent>();
+			ev->mSslId = RsX509Cert::getCertSslId(*x509);
+			ev->mLocator = RsUrl(foreign_addr);
+			ev->mErrorCode = RsAuthSslError::IP_IS_BLACKLISTED;
+			rsEvents->postEvent(ev);
+		}
 
-		RsServer::notify()->AddFeedItem(
-		            RS_FEED_ITEM_SEC_IP_BLACKLISTED,
-		            PeerId().toStdString(),
-		            sockaddr_storage_iptostring(foreign_addr), "", "",
-		            check_result);
 		reset_locked();
-		return -1;
+		return failure;
 	}
+#endif //def RS_BANLIST_REDUNDANT_CHECK
 
 	if (waiting != WAITING_NOT)
 	{
-		std::cerr << __PRETTY_FUNCTION__ << " Peer: " << PeerId().toStdString()
-		          << " - Two connections in progress - Shut 1 down!"
-		          << std::endl;
+		RsInfo() << __PRETTY_FUNCTION__ << " Peer: " << PeerId()
+		         << " - Two connections in progress - Shut 1 down!"
+		         << std::endl;
 
 		// outgoing connection in progress.
 		// shut this baby down.
@@ -1382,7 +1347,7 @@ int	pqissl::accept_locked( SSL *ssl, int fd,
 	/* shutdown existing - in all cases use the new one */
 	if ((ssl_connection) && (ssl_connection != ssl))
 	{
-		std::cerr << __PRETTY_FUNCTION__
+		RsInfo() << __PRETTY_FUNCTION__
 		          << " closing Previous/Existing ssl_connection" << std::endl;
 		SSL_shutdown(ssl_connection);
 		SSL_free (ssl_connection);
@@ -1390,7 +1355,7 @@ int	pqissl::accept_locked( SSL *ssl, int fd,
 
 	if ((sockfd > -1) && (sockfd != fd))
 	{
-		std::cerr << __PRETTY_FUNCTION__ << " closing Previous/Existing sockfd"
+		RsInfo() << __PRETTY_FUNCTION__ << " closing Previous/Existing sockfd"
 		          << std::endl;
 		net_internal_close(sockfd);
 	}
@@ -1406,7 +1371,7 @@ int	pqissl::accept_locked( SSL *ssl, int fd,
 	 */
 	sockaddr_storage_copy(foreign_addr, remote_addr);
 
-	std::cerr << __PRETTY_FUNCTION__ << " SUCCESSFUL connection to: "
+	RsInfo() << __PRETTY_FUNCTION__ << " SUCCESSFUL connection to: "
 	          << PeerId().toStdString() << " remoteaddr: "
 	          << sockaddr_storage_iptostring(remote_addr) << std::endl;
 
@@ -1431,7 +1396,7 @@ int	pqissl::accept_locked( SSL *ssl, int fd,
 		waiting = WAITING_FAIL_INTERFACE; // failed completely.
 
 		reset_locked();
-		return -1;
+		return failure;
 	}
 #ifdef PQISSL_DEBUG
 	else std::cerr << __PRETTY_FUNCTION__ << " Socket made non-nlocking!"
@@ -1456,7 +1421,7 @@ int	pqissl::accept_locked( SSL *ssl, int fd,
 		sockaddr_storage addr; sockaddr_storage_copy(remote_addr, addr);
 		parent()->notifyEvent(this, NET_CONNECT_SUCCESS, addr);
 	}
-	return 1;
+	return success;
 }
 
 /********** Implementation of BinInterface **************************
@@ -1633,7 +1598,9 @@ int pqissl::readdata(void *data, int len)
 					reset_locked();
 				}
 
+#ifdef PQISSL_LOG_DEBUG2
 				rslog(RSL_ALERT, pqisslzone, out);
+#endif
 				//std::cerr << out << std::endl ;
 				return -1;
 			}
@@ -1775,6 +1742,13 @@ bool 	pqissl::moretoread(uint32_t usec)
 	{
 		rslog(RSL_ALERT, pqisslzone, 
 			"pqissl::moretoread() Select ERROR!");
+		RS_WARN(strerror(errno));
+
+		if (errno == EBADF) {
+			// happens when SAM is shut down
+			rslog(RSL_ALERT, pqisslzone, "pqissl::moretoread() -> calling reset()");
+			reset_locked();
+		}
 		return 0;
 	}
 
