@@ -3,7 +3,9 @@
  *                                                                             *
  * libretroshare: retroshare core library                                      *
  *                                                                             *
- * Copyright 2004-2006 by Robert Fernie <retroshare@lunamutt.com>              *
+ * Copyright (C) 2004-2006  Robert Fernie <retroshare@lunamutt.com>            *
+ * Copyright (C) 2018-2021  Gioacchino Mazzurco <gio@eigenlab.org>             *
+ * Copyright (C) 2020-2021  Asociación Civil Altermundi <info@altermundi.net>  *
  *                                                                             *
  * This program is free software: you can redistribute it and/or modify        *
  * it under the terms of the GNU Lesser General Public License as              *
@@ -37,21 +39,11 @@
 
 #define USE_NEW_CHUNK_CHECKING_CODE
 
-// This adds a level of indirection to types, so we can easily change them if needed
-//
-//typedef std::string   RsCertId;	// unused
-//typedef std::string   RsChanId;
-//typedef std::string   RsMsgId;
-//typedef std::string   RsAuthId;
-
-typedef SSLIdType     RsPeerId ;
-typedef PGPIdType     RsPgpId ;
 typedef Sha1CheckSum  RsFileHash ;
-typedef Sha1CheckSum  RsMessageId ;
 
 const uint32_t FT_STATE_FAILED			= 0x0000 ;
-const uint32_t FT_STATE_OKAY				= 0x0001 ;
-const uint32_t FT_STATE_WAITING 			= 0x0002 ;
+const uint32_t FT_STATE_OKAY			= 0x0001 ;
+const uint32_t FT_STATE_WAITING 		= 0x0002 ;
 const uint32_t FT_STATE_DOWNLOADING		= 0x0003 ;
 const uint32_t FT_STATE_COMPLETE 		= 0x0004 ;
 const uint32_t FT_STATE_QUEUED   		= 0x0005 ;
@@ -145,17 +137,10 @@ struct PeerBandwidthLimits : RsSerializable
 	}
 };
 
-//class SearchRequest // unused stuff.
-//{
-//	public:
-//	int searchId;
-//	RsCertId toId;  /* all zeros for everyone! */
-//	std::list<Condition> tests;
-//};
-
-
 /********************** For FileCache Interface *****************/
 
+/* G10h4ck: Having this kind of stuff on public headers is pretty dangerous for
+ * name space pollution, a C++11 enum class should be used instad ASAP */
 #define DIR_TYPE_UNKNOWN	    0x00
 #define DIR_TYPE_ROOT		0x01
 #define DIR_TYPE_PERSON  	0x02
@@ -207,10 +192,11 @@ struct FileInfo : RsSerializable
 	/// 0 if none
 	int searchId;
 
-	std::string path;
+    std::string path;		// full path of the file (*Not just the directory*)
 	std::string fname;
 	RsFileHash hash;
-	std::string ext;
+
+	RS_DEPRECATED std::string ext; /// @deprecated unused
 
 	uint64_t size;
 	uint64_t avail; /// how much we have
@@ -257,8 +243,6 @@ struct FileInfo : RsSerializable
 	}
 };
 
-std::ostream &operator<<(std::ostream &out, const FileInfo& info);
-
 /**
  * Pointers in this class have no real meaning as pointers, they are used as
  * indexes, internally by retroshare.
@@ -269,7 +253,10 @@ struct DirStub : RsSerializable
 
 	uint8_t type;
 	std::string name;
-	void *ref;
+
+	/* G10h4ck do we still need to keep this as void* instead of uint64_t for
+	 * retroshare-gui sake? */
+	void* ref;
 
 	/// @see RsSerializable
 	void serial_process(RsGenericSerializer::SerializeJob j,
@@ -278,29 +265,12 @@ struct DirStub : RsSerializable
 		RS_SERIAL_PROCESS(type);
 		RS_SERIAL_PROCESS(name);
 
-#if defined(__GNUC__) && !defined(__clang__)
-#	pragma GCC diagnostic ignored "-Wstrict-aliasing"
-#endif // defined(__GNUC__) && !defined(__clang__)
-		// (Cyril) We have to do this because on some systems (MacOS) uintptr_t is unsigned long which is not well defined. It is always
-        // preferable to force type serialization to the correct size rather than letting the compiler choose for us.
-        // /!\ This structure cannot be sent over the network. The serialization would be inconsistent.
-
-		if(sizeof(ref) == 4)
-		{
-			std::uint32_t& handle(reinterpret_cast<std::uint32_t&>(ref));
-			RS_SERIAL_PROCESS(handle);
-		}
-		else if(sizeof(ref) == 8)
-		{
-			std::uint64_t& handle(reinterpret_cast<std::uint64_t&>(ref));
-			RS_SERIAL_PROCESS(handle);
-		}
-		else
-			std::cerr << __PRETTY_FUNCTION__ << ": cannot serialize raw pointer of size " << sizeof(ref) << std::endl;
-
-#if defined(__GNUC__) && !defined(__clang__)
-#	pragma GCC diagnostic pop
-#endif // defined(__GNUC__) && !defined(__clang__)
+		/* Enforce serialization as uint64_t because void* changes size (usually
+		 * 4 bytes on 32bit arch and 8 bytes on 64bit archs) depending on
+		 * architectures and make JSON API behave inconsistenly. */
+		uint64_t handle = reinterpret_cast<uint64_t>(ref);
+		RS_SERIAL_PROCESS(handle);
+		ref = reinterpret_cast<void*>(handle);
 	}
 };
 
@@ -311,19 +281,25 @@ struct DirStub : RsSerializable
 struct DirDetails : RsSerializable
 {
 	DirDetails() : parent(nullptr), prow(0), ref(nullptr),
-	    type(DIR_TYPE_UNKNOWN), count(0), mtime(0), max_mtime(0) {}
+        type(DIR_TYPE_UNKNOWN), size(0), mtime(0), max_mtime(0) {}
 
 
+	/* G10h4ck do we still need to keep this as void* instead of uint64_t for
+	 * retroshare-gui sake? */
 	void* parent;
+
 	int prow; /* parent row */
 
+	/* G10h4ck do we still need to keep this as void* instead of uint64_t for
+	 * retroshare-gui sake? */
 	void* ref;
+
     uint8_t type;
     RsPeerId id;
     std::string name;
     RsFileHash hash;
     std::string path;		// full path of the parent directory, when it is a file; full path of the dir otherwise.
-	uint64_t count;
+    uint64_t size;          // total size of directory, or size of the file.
 	uint32_t mtime;			// file/directory modification time, according to what the system reports
 	FileStorageFlags flags;
 	uint32_t max_mtime ;	// maximum modification time of the whole hierarchy below.
@@ -333,36 +309,18 @@ struct DirDetails : RsSerializable
 
 	/// @see RsSerializable
 	void serial_process(RsGenericSerializer::SerializeJob j,
-	                    RsGenericSerializer::SerializeContext& ctx)
+	                    RsGenericSerializer::SerializeContext& ctx) override
 	{
-#if defined(__GNUC__) && !defined(__clang__)
-#	pragma GCC diagnostic ignored "-Wstrict-aliasing"
-#endif // defined(__GNUC__) && !defined(__clang__)
+		/* Enforce serialization as uint64_t because void* changes size (usually
+		 * 4 bytes on 32bit arch and 8 bytes on 64bit archs) depending on
+		 * architectures and make JSON API behave inconsistenly. */
+		uint64_t handle = reinterpret_cast<uint64_t>(ref);
+		RS_SERIAL_PROCESS(handle);
+		ref = reinterpret_cast<void*>(handle);
 
-        // (Cyril) We have to do this because on some systems (MacOS) uintptr_t is unsigned long which is not well defined. It is always
-        // preferable to force type serialization to the correct size rather than letting the compiler choose for us.
-        // /!\ This structure cannot be sent over the network. The serialization would be inconsistent.
-
-		if(sizeof(ref) == 4)
-		{
-			std::uint32_t& handle(reinterpret_cast<std::uint32_t&>(ref));
-			RS_SERIAL_PROCESS(handle);
-			std::uint32_t& parentHandle(reinterpret_cast<std::uint32_t&>(parent));
-			RS_SERIAL_PROCESS(parentHandle);
-		}
-		else if(sizeof(ref) == 8)
-		{
-			std::uint64_t& handle(reinterpret_cast<std::uint64_t&>(ref));
-			RS_SERIAL_PROCESS(handle);
-			std::uint64_t& parentHandle(reinterpret_cast<std::uint64_t&>(parent));
-			RS_SERIAL_PROCESS(parentHandle);
-		}
-		else
-			std::cerr << __PRETTY_FUNCTION__ << ": cannot serialize raw pointer of size " << sizeof(ref) << std::endl;
-
-#if defined(__GNUC__) && !defined(__clang__)
-#	pragma GCC diagnostic pop
-#endif // defined(__GNUC__) && !defined(__clang__)
+		uint64_t parentHandle = reinterpret_cast<uint64_t>(parent);
+		RS_SERIAL_PROCESS(parentHandle);
+		parent = reinterpret_cast<void*>(parentHandle);
 
 		RS_SERIAL_PROCESS(prow);
 		RS_SERIAL_PROCESS(type);
@@ -370,27 +328,15 @@ struct DirDetails : RsSerializable
 		RS_SERIAL_PROCESS(name);
 		RS_SERIAL_PROCESS(hash);
 		RS_SERIAL_PROCESS(path);
-		RS_SERIAL_PROCESS(count);
+		RS_SERIAL_PROCESS(size);
 		RS_SERIAL_PROCESS(mtime);
 		RS_SERIAL_PROCESS(flags);
 		RS_SERIAL_PROCESS(max_mtime);
 		RS_SERIAL_PROCESS(children);
 		RS_SERIAL_PROCESS(parent_groups);
 	}
-};
 
-std::ostream &operator<<(std::ostream &out, const DirDetails& details);
-
-class FileDetail
-{
-	public:
-        RsPeerId id;
-		std::string name;
-        RsFileHash hash;
-		std::string path;
-		uint64_t size;
-		uint32_t age;
-		uint32_t rank;
+	~DirDetails() override = default;
 };
 
 class CompressedChunkMap ;
@@ -412,11 +358,20 @@ struct FileChunksInfo : RsSerializable
 		CHUNK_STRATEGY_PROGRESSIVE
 	};
 
-	struct SliceInfo
+	struct SliceInfo : RsSerializable
 	{
 		uint32_t start;
 		uint32_t size;
 		RsPeerId peer_id;
+
+		/// @see RsSerializable
+		void serial_process(RsGenericSerializer::SerializeJob j,
+		                    RsGenericSerializer::SerializeContext& ctx) override
+		{
+			RS_SERIAL_PROCESS(start);
+			RS_SERIAL_PROCESS(size);
+			RS_SERIAL_PROCESS(peer_id);
+		}
 	};
 
 	uint64_t file_size; /// real size of the file
@@ -445,7 +400,7 @@ struct FileChunksInfo : RsSerializable
 		RS_SERIAL_PROCESS(chunks);
 		RS_SERIAL_PROCESS(compressed_peer_availability_maps);
 		RS_SERIAL_PROCESS(active_chunks);
-		//RS_SERIAL_PROCESS(pending_slices);
+		RS_SERIAL_PROCESS(pending_slices);
 	}
 };
 
