@@ -3,8 +3,9 @@
  *                                                                             *
  * libretroshare: retroshare core library                                      *
  *                                                                             *
- * Copyright 2004-2008 by Robert Fernie <retroshare@lunamutt.com>              *
- * Copyright (C) 2015-2018  Gioacchino Mazzurco <gio@eigenlab.org>             *
+ * Copyright (C) 2004-2008  Robert Fernie <retroshare@lunamutt.com>            *
+ * Copyright (C) 2015-2020  Gioacchino Mazzurco <gio@eigenlab.org>             *
+ * Copyright (C) 2020       Asociación Civil Altermundi <info@altermundi.net>  *
  *                                                                             *
  * This program is free software: you can redistribute it and/or modify        *
  * it under the terms of the GNU Lesser General Public License as              *
@@ -20,7 +21,7 @@
  * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
  *                                                                             *
  *******************************************************************************/
-#include "util/radix64.h"
+
 #include "pgp/pgpkeyutil.h"
 
 #include "rsserver/p3peers.h"
@@ -33,25 +34,25 @@
 #include "pqi/authssl.h"
 #include "pqi/authgpg.h"
 #include "retroshare/rsinit.h"
+#include "retroshare/rsnotify.h"
 #include "retroshare/rsfiles.h"
 #include "util/rsurl.h"
-
+#include "util/radix64.h"
+#include "util/rsbase64.h"
 #include "pgp/rscertificate.h"
 
 #include <iostream>
 #include <fstream>
 
-const std::string CERT_SSL_ID = "--SSLID--";
-const std::string CERT_LOCATION = "--LOCATION--";
-const std::string CERT_LOCAL_IP = "--LOCAL--";
-const std::string CERT_EXT_IP = "--EXT--";
-const std::string CERT_DYNDNS = "--DYNDNS--";
-
 //static const int MAX_TIME_KEEP_LOCATION_WITHOUT_CONTACT = 30*24*3600 ; // 30 days.
 
-
 #include "pqi/authssl.h"
 
+const RetroshareInviteFlags  RsPeers::defaultCertificateFlags = RetroshareInviteFlags::DNS
+                                                  | RetroshareInviteFlags::CURRENT_LOCAL_IP
+                                                  | RetroshareInviteFlags::CURRENT_EXTERNAL_IP;
+
+typedef RsCertificate::RsShortInviteFieldType RsShortInviteFieldType;	// locally in this file to avoid renaming everything.
 
 RsPeers *rsPeers = NULL;
 
@@ -122,29 +123,23 @@ std::string RsPeerNetModeString(uint32_t netModel)
 
 
 p3Peers::p3Peers(p3LinkMgr *lm, p3PeerMgr *pm, p3NetMgr *nm)
-        :mLinkMgr(lm), mPeerMgr(pm), mNetMgr(nm)
-{
-	return;
-}
-
-bool p3Peers::hasExportMinimal()
-{
-#ifdef GPGME_EXPORT_MODE_MINIMAL
-	return true ;
-#else
-	return false ;
-#endif
-}
+        :mLinkMgr(lm), mPeerMgr(pm), mNetMgr(nm) {}
 
 	/* Updates ... */
-bool p3Peers::FriendsChanged()
+bool p3Peers::FriendsChanged(const RsPeerId& pid,bool add)
 {
 #ifdef P3PEERS_DEBUG
-        std::cerr << "p3Peers::FriendsChanged()" << std::endl;
+    std::cerr << "p3Peers::FriendsChanged()" << std::endl;
 #endif
+    if(rsEvents)
+    {
+        auto ev = std::make_shared<RsPeerStateChangedEvent>(pid);
+        rsEvents->postEvent(ev);
+    }
+    RsServer::notify()->notifyListChange(NOTIFY_LIST_FRIENDS, add? NOTIFY_TYPE_ADD : NOTIFY_TYPE_DEL); // this is meant to disappear
 
-	/* TODO */
-	return false;
+    /* TODO */
+    return false;
 }
 
 bool p3Peers::OthersChanged()
@@ -158,7 +153,7 @@ bool p3Peers::OthersChanged()
 }
 
 	/* Peer Details (Net & Auth) */
-const RsPeerId& p3Peers::getOwnId()
+RsPeerId p3Peers::getOwnId()
 {
 #ifdef P3PEERS_DEBUG
         std::cerr << "p3Peers::getOwnId()" << std::endl;
@@ -264,7 +259,7 @@ bool p3Peers::setPeerMaximumRates(const RsPgpId& pid,uint32_t maxUploadRate,uint
 
 bool p3Peers::haveSecretKey(const RsPgpId& id)
 {
-	return AuthGPG::getAuthGPG()->haveSecretKey(id);
+    return AuthPGP::haveSecretKey(id);
 }
 
 /* There are too many dependancies of this function
@@ -283,7 +278,7 @@ bool p3Peers::getPeerDetails(const RsPeerId& id, RsPeerDetails &d)
 	if (id == sOwnId)
 	{
 		mPeerMgr->getOwnNetStatus(ps);
-		ps.gpg_id = AuthGPG::getAuthGPG()->getGPGOwnId();
+        ps.gpg_id = AuthPGP::getPgpOwnId();
 	}
 	else if (!mPeerMgr->getFriendNetStatus(id, ps))
 	{
@@ -295,7 +290,18 @@ bool p3Peers::getPeerDetails(const RsPeerId& id, RsPeerDetails &d)
 
 	/* get from gpg (first), to fill in the sign and trust details */
 	/* don't return now, we've got fill in the ssl and connection info */
-	getGPGDetails(ps.gpg_id, d);
+
+	if(!getGPGDetails(ps.gpg_id, d))
+	{
+		if(!ps.skip_pgp_signature_validation)
+			return false;
+
+		d.gpg_id = ps.gpg_id ;
+		d.skip_pgp_signature_validation = true;
+	}
+	else
+		d.skip_pgp_signature_validation = false;
+
 	d.isOnlyGPGdetail = false;
 
 	//get the ssl details
@@ -372,7 +378,7 @@ bool p3Peers::getPeerDetails(const RsPeerId& id, RsPeerDetails &d)
 			sockaddr_storage_ipv6_to_ipv4(it->mAddr);
 			std::string toto;
 			toto += sockaddr_storage_tostring(it->mAddr);
-			rs_sprintf_append(toto, "    %ld sec", time(NULL) - it->mSeenTime);
+			rs_sprintf_append(toto, "    %ld sec loc", time(NULL) - it->mSeenTime);
 			d.ipAddressList.push_back(toto);
 		}
 		for(it = ps.ipAddrs.mExt.mAddrs.begin(); it != ps.ipAddrs.mExt.mAddrs.end(); ++it)
@@ -380,7 +386,7 @@ bool p3Peers::getPeerDetails(const RsPeerId& id, RsPeerDetails &d)
 			sockaddr_storage_ipv6_to_ipv4(it->mAddr);
 			std::string toto;
 			toto += sockaddr_storage_tostring(it->mAddr);
-			rs_sprintf_append(toto, "    %ld sec", time(NULL) - it->mSeenTime);
+			rs_sprintf_append(toto, "    %ld sec ext", time(NULL) - it->mSeenTime);
 			d.ipAddressList.push_back(toto);
 		}
 	}
@@ -558,17 +564,33 @@ bool p3Peers::isProxyAddress(const uint32_t type, const sockaddr_storage& addr)
 
 bool p3Peers::isKeySupported(const RsPgpId& id)
 {
-	return AuthGPG::getAuthGPG()->isKeySupported(id);
+    return AuthPGP::isKeySupported(id);
 }
 
 std::string p3Peers::getGPGName(const RsPgpId &gpg_id)
 {
 	/* get from mAuthMgr as it should have more peers? */
-        return AuthGPG::getAuthGPG()->getGPGName(gpg_id);
+        return AuthPGP::getPgpName(gpg_id);
 }
 
 bool p3Peers::isPgpFriend(const RsPgpId& pgpId)
-{ return AuthGPG::getAuthGPG()->isGPGAccepted(pgpId); }
+{ return AuthPGP::isPGPAccepted(pgpId); }
+
+bool p3Peers::isSslOnlyFriend(const RsPeerId& sslId)
+{
+    bool has_ssl_only_flag = mPeerMgr->isSslOnlyFriend(sslId) ;
+
+    if(has_ssl_only_flag)
+    {
+        if(isPgpFriend(getGPGId(sslId)))
+        {
+            RsErr() << __PRETTY_FUNCTION__ << ": Peer " << sslId << " has SSL-friend-only flag but his PGP id is in the list of friends. This is inconsistent (Bug in the code). Returning false for security reasons." << std::endl;
+			return false;
+        }
+        return true;
+	}
+    return false;
+}
 
 bool p3Peers::isGPGAccepted(const RsPgpId &gpg_id_is_friend)
 { return isPgpFriend(gpg_id_is_friend); }
@@ -580,7 +602,7 @@ std::string p3Peers::getPeerName(const RsPeerId& ssl)
 #endif
 	std::string name;
 	if (ssl == AuthSSL::getAuthSSL()->OwnId()) 
-		return AuthGPG::getAuthGPG()->getGPGOwnName();
+        return AuthPGP::getPgpOwnName();
 	
 	if (mPeerMgr->getPeerName(ssl, name)) 
 	{
@@ -600,7 +622,7 @@ bool	p3Peers::getGPGAllList(std::list<RsPgpId> &ids)
 #endif
 
         /* get from mAuthMgr */
-        AuthGPG::getAuthGPG()->getGPGAllList(ids);
+        AuthPGP::getPgpAllList(ids);
         return true;
 }
 
@@ -611,7 +633,7 @@ bool	p3Peers::getGPGValidList(std::list<RsPgpId> &ids)
 #endif
 
         /* get from mAuthMgr */
-        AuthGPG::getAuthGPG()->getGPGValidList(ids);
+        AuthPGP::getPgpValidList(ids);
         return true;
 }
 
@@ -622,16 +644,28 @@ bool	p3Peers::getGPGSignedList(std::list<RsPgpId> &ids)
 #endif
 
         /* get from mAuthMgr */
-        AuthGPG::getAuthGPG()->getGPGSignedList(ids);
+        AuthPGP::getPgpSignedList(ids);
         return true;
 }
 
+bool p3Peers::getPgpFriendList(std::vector<RsPgpId>& pgpIds)
+{
+	std::list<RsPgpId> ids;
+    if(AuthPGP::getPgpAcceptedList(ids))
+	{
+		pgpIds.clear();
+		std::copy(ids.begin(), ids.end(), std::back_inserter(pgpIds));
+		return true;
+	}
+	return false;
+}
+
 bool	p3Peers::getGPGAcceptedList(std::list<RsPgpId> &ids)
 {
 #ifdef P3PEERS_DEBUG
         std::cerr << "p3Peers::getGPGAcceptedList()" << std::endl;
 #endif
-        AuthGPG::getAuthGPG()->getGPGAcceptedList(ids);
+        AuthPGP::getPgpAcceptedList(ids);
         return true;
 }
 
@@ -647,7 +681,12 @@ bool	p3Peers::getAssociatedSSLIds(const RsPgpId &gpg_id, std::list<RsPeerId> &id
 
 bool    p3Peers::gpgSignData(const void *data, const uint32_t len, unsigned char *sign, unsigned int *signlen, std::string reason /* = "" */)
 {
-	return AuthGPG::getAuthGPG()->SignDataBin(data,len,sign,signlen, reason);
+    return AuthPGP::SignDataBin(data,len,sign,signlen, reason);
+}
+
+RsPgpId p3Peers::pgpIdFromFingerprint(const RsPgpFingerprint& fpr)
+{
+	return PGPHandler::pgpIdFromFingerprint(fpr);
 }
 
 bool	p3Peers::getGPGDetails(const RsPgpId &pgp_id, RsPeerDetails &d)
@@ -657,7 +696,7 @@ bool	p3Peers::getGPGDetails(const RsPgpId &pgp_id, RsPeerDetails &d)
 #endif
 
 	/* get from mAuthMgr */
-	bool res = AuthGPG::getAuthGPG()->getGPGDetails(pgp_id, d);
+    bool res = AuthPGP::getPgpDetails(pgp_id, d);
 
 	d.isOnlyGPGdetail = true ;
 	d.service_perm_flags = mPeerMgr->servicePermissionFlags(pgp_id) ;
@@ -672,7 +711,7 @@ const RsPgpId& p3Peers::getGPGOwnId()
 #endif
 
 	/* get from mAuthMgr */
-        return AuthGPG::getAuthGPG()->getGPGOwnId();
+        return AuthPGP::getPgpOwnId();
 }
 
 RsPgpId p3Peers::getGPGId(const RsPeerId& sslid)
@@ -684,12 +723,11 @@ RsPgpId p3Peers::getGPGId(const RsPeerId& sslid)
 	/* get from mAuthMgr */
 	if (sslid == AuthSSL::getAuthSSL()->OwnId()) 
 	{
-		return AuthGPG::getAuthGPG()->getGPGOwnId();
+        return AuthPGP::getPgpOwnId();
 	}
 	peerState pcs;
-	if (mPeerMgr->getFriendNetStatus(sslid, pcs) || mPeerMgr->getOthersNetStatus(sslid, pcs)) {
+	if (mPeerMgr->getFriendNetStatus(sslid, pcs))
 		return pcs.gpg_id;
-	}
 
 	return RsPgpId();
 }
@@ -706,12 +744,12 @@ bool 	p3Peers::addFriend(const RsPeerId &ssl_id, const RsPgpId &gpg_id,ServicePe
 #ifdef P3PEERS_DEBUG
 	std::cerr << "p3Peers::addFriend() with : id : " << id << "; gpg_id : " << gpg_id << std::endl;
 #endif
-	if(AuthGPG::getAuthGPG()->isGPGId(gpg_id)) 
+    if(AuthPGP::isPGPId(gpg_id))
 	{
 #ifdef P3PEERS_DEBUG
 		std::cerr << "p3Peers::addFriend() Authorising GPG Id: " << gpg_id << std::endl;
 #endif
-		if (AuthGPG::getAuthGPG()->AllowConnection(gpg_id, true))
+        if (AuthPGP::AllowConnection(gpg_id, true))
 		{
 #ifdef P3PEERS_DEBUG
 			std::cerr << "p3Peers::addFriend() Authorization OK." << std::endl;
@@ -741,7 +779,9 @@ bool 	p3Peers::addFriend(const RsPeerId &ssl_id, const RsPgpId &gpg_id,ServicePe
 		return true;
 	} 
 
-	/* otherwise - we install as ssl_id..... 
+    FriendsChanged(ssl_id,true);
+
+	/* otherwise - we install as ssl_id.....
 	 * If we are adding an SSL certificate. we flag lastcontact as now. 
 	 * This will cause the SSL certificate to be retained for 30 days... and give the person a chance to connect!
 	 *  */
@@ -749,9 +789,20 @@ bool 	p3Peers::addFriend(const RsPeerId &ssl_id, const RsPgpId &gpg_id,ServicePe
 	return mPeerMgr->addFriend(ssl_id, gpg_id, RS_NET_MODE_UDP, RS_VS_DISC_FULL, RS_VS_DHT_FULL, now, perm_flags);
 }
 
+bool p3Peers::addSslOnlyFriend( const RsPeerId& sslId, const RsPgpId& pgp_id,const RsPeerDetails& details )
+{
+    if( mPeerMgr->addSslOnlyFriend(sslId, pgp_id,details))
+    {
+        FriendsChanged(sslId,true);
+        return true;
+    }
+    else
+		return false;
+}
+
 bool 	p3Peers::removeKeysFromPGPKeyring(const std::set<RsPgpId>& pgp_ids,std::string& backup_file,uint32_t& error_code)
 {
-	return AuthGPG::getAuthGPG()->removeKeysFromPGPKeyring(pgp_ids,backup_file,error_code) ;
+    return AuthPGP::removeKeysFromPGPKeyring(pgp_ids,backup_file,error_code) ;
 }
 
 bool 	p3Peers::removeFriendLocation(const RsPeerId &sslId)
@@ -761,8 +812,9 @@ bool 	p3Peers::removeFriendLocation(const RsPeerId &sslId)
 #endif
 		//will remove if it's a ssl id
         mPeerMgr->removeFriend(sslId, false);
-        return true;
 
+        FriendsChanged(sslId,false);
+        return true;
 }
 
 bool 	p3Peers::removeFriend(const RsPgpId& gpgId)
@@ -770,7 +822,7 @@ bool 	p3Peers::removeFriend(const RsPgpId& gpgId)
 #ifdef P3PEERS_DEBUG
 	std::cerr << "p3Peers::removeFriend() " << gpgId << std::endl;
 #endif
-	if (gpgId == AuthGPG::getAuthGPG()->getGPGOwnId()) {
+    if (gpgId == AuthPGP::getPgpOwnId()) {
         std::cerr << "p3Peers::removeFriend() ERROR  we're not going to remove our own GPG id."  << std::endl;
 		return false;
 	}
@@ -778,7 +830,7 @@ bool 	p3Peers::removeFriend(const RsPgpId& gpgId)
 #ifdef P3PEERS_DEBUG
 	std::cerr << "p3Peers::removeFriend() Removing GPG Id: " << gpgId << std::endl;
 #endif
-	if (AuthGPG::getAuthGPG()->AllowConnection(gpgId, false))
+    if (AuthPGP::AllowConnection(gpgId, false))
 	{
 #ifdef P3PEERS_DEBUG
 		std::cerr << "p3Peers::removeFriend() OK." << std::endl;
@@ -812,9 +864,15 @@ void p3Peers::getIPServersList(std::list<std::string>& ip_servers)
 {
 	mNetMgr->getIPServersList(ip_servers) ;
 }
+void p3Peers::getCurrentExtIPList(std::list<std::string>& ip_list)
+{
+	mNetMgr->getCurrentExtIPList(ip_list) ;
+}
 bool p3Peers::resetOwnExternalAddressList()
 {
-    return mPeerMgr->resetOwnExternalAddressList();
+	//TODO Phenom 2021-10-30: Need to call something like mNetMgr->netReset();
+	// to update this addresslist.
+	return mPeerMgr->resetOwnExternalAddressList();
 }
 void p3Peers::allowServerIPDetermination(bool b)
 {
@@ -1021,7 +1079,7 @@ bool p3Peers::setProxyServer(const uint32_t type, const std::string &addr_str, c
             return false ;
         }
 
-        std::cerr << "Settign proxy server address to " << addr_str << ":" << port << std::endl;
+        std::cerr << "Setting proxy server address to " << addr_str << ":" << port << std::endl;
 
 	struct sockaddr_storage addr;
 	struct sockaddr_in *addrv4p = (struct sockaddr_in *) &addr;
@@ -1057,51 +1115,301 @@ bool p3Peers::setProxyServer(const uint32_t type, const std::string &addr_str, c
 
 std::string p3Peers::getPGPKey(const RsPgpId& pgp_id,bool include_signatures)
 {
-	unsigned char *mem_block = NULL;
+	rs_owner_ptr<unsigned char> mem_block = nullptr;
 	size_t mem_block_size = 0;
 
-	if( !AuthGPG::getAuthGPG()->exportPublicKey(
-	             RsPgpId(pgp_id), mem_block, mem_block_size,
-	             false, include_signatures ) )
+    if( !AuthPGP::exportPublicKey( RsPgpId(pgp_id), mem_block, mem_block_size, false, include_signatures ) )
 	{
-		std::cerr << "Cannot output certificate for id \"" << pgp_id
-		          << "\". Sorry." << std::endl;
-		return "" ;
+		RsErr() << __PRETTY_FUNCTION__
+		        << " Failure retriving certificate for id " << pgp_id
+		        << std::endl;
+		return "";
 	}
 
-	RsPeerDetails Detail;
-	if(!getGPGDetails(pgp_id,Detail)) return "";
+	RsPeerDetails details;
+	if(!getGPGDetails(pgp_id, details)) return "";
 
-	RsCertificate cert( Detail,mem_block,mem_block_size );
-	delete[] mem_block ;
+	auto certPtr =
+	        RsCertificate::fromMemoryBlock(details, mem_block, mem_block_size);
 
-	return cert.armouredPGPKey();
+	free(mem_block);
+
+	if(certPtr) return certPtr->armouredPGPKey();
+
+	return "";
 }
 
 
-bool p3Peers::GetPGPBase64StringAndCheckSum(	const RsPgpId& gpg_id,
-															std::string& gpg_base64_string,
-															std::string& gpg_base64_checksum) 
+bool p3Peers::GetPGPBase64StringAndCheckSum(
+        const RsPgpId& gpg_id,
+        std::string& gpg_base64_string, std::string& gpg_base64_checksum )
 {
 	gpg_base64_string = "" ;
 	gpg_base64_checksum = "" ;
 
-	unsigned char *mem_block ;
-	size_t mem_block_size ;
+	rs_owner_ptr<unsigned char> mem_block = nullptr;
+	size_t mem_block_size = 0;
+    if(!AuthPGP::exportPublicKey( gpg_id,mem_block,mem_block_size,false,false ))
+		return false;
 
-	if(!AuthGPG::getAuthGPG()->exportPublicKey(gpg_id,mem_block,mem_block_size,false,false))
-		return false ;
+	RsBase64::encode(mem_block, mem_block_size, gpg_base64_string, true, false);
 
-	Radix64::encode(mem_block,mem_block_size,gpg_base64_string) ;
+	uint32_t crc = PGPKeyManagement::compute24bitsCRC(mem_block,mem_block_size);
 
-	uint32_t crc = PGPKeyManagement::compute24bitsCRC((unsigned char *)mem_block,mem_block_size) ;
+	free(mem_block);
 
-	unsigned char tmp[3] = { uint8_t((crc >> 16) & 0xff), uint8_t((crc >> 8) & 0xff), uint8_t(crc & 0xff) } ;
-	Radix64::encode(tmp,3,gpg_base64_checksum) ;
+	unsigned char tmp[3] = {
+	    uint8_t((crc >> 16) & 0xff),
+	    uint8_t((crc >> 8) & 0xff),
+	    uint8_t(crc & 0xff) } ;
+	RsBase64::encode(tmp, 3, gpg_base64_checksum, true, false);
 
-	delete[] mem_block ;
+	return true;
+}
 
-	return true ;
+static void addPacketHeader(RsShortInviteFieldType ptag, size_t size, unsigned char *& buf, uint32_t& offset, uint32_t& buf_size)
+{
+	// Check that the buffer has sufficient size. If not, increase it.
+
+	while(offset + size + 6 >= buf_size)
+	{
+        unsigned char *newbuf = (unsigned char*)malloc(2*buf_size) ;
+
+		memcpy(newbuf, buf, buf_size) ;
+		buf_size *= 2 ;
+        free(buf) ;
+		buf = newbuf ;
+	}
+
+	// Write ptag and size
+
+	buf[offset] = static_cast<uint8_t>(ptag) ;
+	offset += 1 ;
+
+	offset += PGPKeyParser::write_125Size(&buf[offset],size) ;
+}
+
+bool p3Peers::getShortInvite(std::string& invite, const RsPeerId& _sslId, RetroshareInviteFlags invite_flags, const std::string& baseUrl )
+{
+	RsPeerId sslId = _sslId;
+	if(sslId.isNull()) sslId = getOwnId();
+
+	RsPeerDetails tDetails;
+	if(!getPeerDetails(sslId, tDetails)) return false;
+
+    uint32_t buf_size = 100;
+    uint32_t offset = 0;
+    unsigned char *buf = (unsigned char*)malloc(buf_size);
+
+    addPacketHeader(RsShortInviteFieldType::SSL_ID,RsPeerId::SIZE_IN_BYTES,buf,offset,buf_size);
+	sslId.serialise(buf,buf_size,offset);
+
+    addPacketHeader(RsShortInviteFieldType::PGP_FINGERPRINT,RsPgpFingerprint::SIZE_IN_BYTES,buf,offset,buf_size);
+	tDetails.fpr.serialise(buf,buf_size,offset);
+
+    addPacketHeader(RsShortInviteFieldType::PEER_NAME,tDetails.name.size(),buf,offset,buf_size);
+	memcpy(&buf[offset],tDetails.name.c_str(),tDetails.name.size());
+    offset += tDetails.name.size();
+
+    /* If it is a hidden node, always use hidden address and port as locator */
+
+    if(tDetails.isHiddenNode)
+    {
+        addPacketHeader(RsShortInviteFieldType::HIDDEN_LOCATOR,4 + 2 + tDetails.hiddenNodeAddress.size(),buf,offset,buf_size);
+
+        buf[offset+0] = (uint8_t)((tDetails.hiddenType >> 24) & 0xff);
+        buf[offset+1] = (uint8_t)((tDetails.hiddenType >> 16) & 0xff);
+        buf[offset+2] = (uint8_t)((tDetails.hiddenType >>  8) & 0xff);
+        buf[offset+3] = (uint8_t)((tDetails.hiddenType      ) & 0xff);
+
+        buf[offset+4] = (uint8_t)((tDetails.hiddenNodePort >> 8) & 0xff);
+        buf[offset+5] = (uint8_t)((tDetails.hiddenNodePort     ) & 0xff);
+
+        memcpy(&buf[offset+6],tDetails.hiddenNodeAddress.c_str(),tDetails.hiddenNodeAddress.size());
+        offset += 4 + 2 + tDetails.hiddenNodeAddress.size();
+    }
+
+    if( !!(invite_flags & RetroshareInviteFlags::DNS) && !tDetails.dyndns.empty() && (tDetails.extPort || tDetails.localPort))
+    {
+        uint16_t tPort = tDetails.extPort ? tDetails.extPort : tDetails.localPort;
+
+        addPacketHeader(RsShortInviteFieldType::DNS_LOCATOR, 2 + tDetails.dyndns.size(),buf,offset,buf_size);
+
+        buf[offset+0] = (uint8_t)((tPort >> 8) & 0xff);
+        buf[offset+1] = (uint8_t)((tPort     ) & 0xff);
+
+        memcpy(&buf[offset+2],tDetails.dyndns.c_str(),tDetails.dyndns.size());
+        offset += 2 + tDetails.dyndns.size();
+    }
+
+    if( !!(invite_flags & RetroshareInviteFlags::FULL_IP_HISTORY) && (!tDetails.ipAddressList.empty()))
+        for(auto& s: tDetails.ipAddressList)
+        {
+            const std::string& tLc = s;
+            std::string tLocator = tLc.substr(0, tLc.find_first_of(" ")-1);
+
+            addPacketHeader(RsShortInviteFieldType::LOCATOR, tLocator.size(),buf,offset,buf_size);
+            memcpy(&buf[offset],tLocator.c_str(),tLocator.size());
+
+            offset += tLocator.size();
+        }
+    else if( !!(invite_flags & (RetroshareInviteFlags::CURRENT_LOCAL_IP | RetroshareInviteFlags::CURRENT_EXTERNAL_IP) ))	// only add at least the local and external IPs
+    {
+#ifdef USE_NEW_LOCATOR_SYSTEM
+        // This new locator system as some advantages, but here it also has major drawbacks: (1) it cannot differentiate local and external addresses,
+        // and (2) it's quite larger than the old system, which tends to make certificates more than 1 line long.
+
+        sockaddr_storage tLocal;
+
+        if(sockaddr_storage_inet_pton(tLocal, tDetails.localAddr) && sockaddr_storage_isValidNet(tLocal) && tDetails.localPort )
+        {
+            addPacketHeader(RsShortInviteFieldType::LOCATOR, tDetails.localAddr.size(),buf,offset,buf_size);
+            memcpy(&buf[offset],tDetails.localAddr.c_str(),tDetails.localAddr.size());
+
+            offset += tDetails.localAddr.size();
+        }
+        sockaddr_storage tExt;
+
+        if(sockaddr_storage_inet_pton(tExt, tDetails.extAddr) && sockaddr_storage_isValidNet(tExt) && tDetails.extPort )
+        {
+            addPacketHeader(RsShortInviteFieldType::LOCATOR, tDetails.extAddr.size(),buf,offset,buf_size);
+            memcpy(&buf[offset],tDetails.extAddr.c_str(),tDetails.extAddr.size());
+
+            offset += tDetails.extAddr.size();
+        }
+#else
+
+        if(!!(invite_flags & RetroshareInviteFlags::CURRENT_LOCAL_IP ))
+        {
+        sockaddr_storage tLocal;
+        bool validLoc =   sockaddr_storage_inet_pton(tLocal, tDetails.localAddr)
+                        && sockaddr_storage_isValidNet(tLocal)
+                        && tDetails.localPort;
+        bool isLocIpv4 = sockaddr_storage_ipv6_to_ipv4(tLocal);
+        if(validLoc && isLocIpv4)
+        {
+            uint32_t t4Addr = reinterpret_cast<sockaddr_in&>(tLocal).sin_addr.s_addr;
+
+            addPacketHeader(RsShortInviteFieldType::LOC4_LOCATOR, 4 + 2,buf,offset,buf_size);
+
+            buf[offset+0] = (uint8_t)((t4Addr >> 24) & 0xff);
+            buf[offset+1] = (uint8_t)((t4Addr >> 16) & 0xff);
+            buf[offset+2] = (uint8_t)((t4Addr >>  8) & 0xff);
+            buf[offset+3] = (uint8_t)((t4Addr      ) & 0xff);
+
+            buf[offset+4] = (uint8_t)((tDetails.localPort >> 8) & 0xff);
+            buf[offset+5] = (uint8_t)((tDetails.localPort     ) & 0xff);
+
+            offset += 4+2;
+        }
+    }
+
+        if(!!(invite_flags & RetroshareInviteFlags::CURRENT_EXTERNAL_IP ))
+        {
+        sockaddr_storage tExt;
+        bool validExt =   sockaddr_storage_inet_pton(tExt, tDetails.extAddr)
+                       && sockaddr_storage_isValidNet(tExt)
+                       && tDetails.extPort;
+        bool isExtIpv4 = sockaddr_storage_ipv6_to_ipv4(tExt);
+        if(validExt && isExtIpv4)
+        {
+            uint32_t t4Addr = reinterpret_cast<sockaddr_in&>(tExt).sin_addr.s_addr;
+
+            addPacketHeader(RsShortInviteFieldType::EXT4_LOCATOR, 4 + 2,buf,offset,buf_size);
+
+            buf[offset+0] = (uint8_t)((t4Addr >> 24) & 0xff);
+            buf[offset+1] = (uint8_t)((t4Addr >> 16) & 0xff);
+            buf[offset+2] = (uint8_t)((t4Addr >>  8) & 0xff);
+            buf[offset+3] = (uint8_t)((t4Addr      ) & 0xff);
+
+            buf[offset+4] = (uint8_t)((tDetails.extPort >> 8) & 0xff);
+            buf[offset+5] = (uint8_t)((tDetails.extPort     ) & 0xff);
+
+            offset += 4+2;
+        }
+        else if(validExt && !isExtIpv4)
+        {
+            // External address is IPv6, save it on LOCATOR
+            sockaddr_storage_setport(tExt,tDetails.extPort);
+            std::string tLocator = sockaddr_storage_tostring(tExt);
+
+            addPacketHeader(RsShortInviteFieldType::LOCATOR, tLocator.size(),buf,offset,buf_size);
+            memcpy(&buf[offset],tLocator.c_str(),tLocator.size());
+
+            offset += tLocator.size();
+        }
+        }
+#endif
+
+    }
+
+    uint32_t computed_crc = PGPKeyManagement::compute24bitsCRC(buf,offset) ;
+
+	// handle endian issues.
+	unsigned char mem[3] ;
+	mem[0] =  computed_crc        & 0xff ;
+	mem[1] = (computed_crc >> 8 ) & 0xff ;
+	mem[2] = (computed_crc >> 16) & 0xff ;
+
+	addPacketHeader( RsShortInviteFieldType::CHECKSUM,3,buf,offset,buf_size);
+	memcpy(&buf[offset],mem,3);
+    offset += 3;
+
+	Radix64::encode(buf, static_cast<int>(offset), invite);
+
+    if(!(invite_flags & RetroshareInviteFlags::RADIX_FORMAT))
+	{
+		RsUrl inviteUrl(baseUrl);
+		inviteUrl.setQueryKV("rsInvite", invite);
+		invite = inviteUrl.toString();
+	}
+    free(buf);
+
+	return true;
+}
+
+bool p3Peers::parseShortInvite(const std::string& inviteStrUrl, RsPeerDetails& details, uint32_t &err_code )
+{
+    if(inviteStrUrl.empty())
+    {
+        RsErr() << __PRETTY_FUNCTION__ << " can't parse empty invite"
+                << std::endl;
+        return false;
+    }
+    std::string rsInvite = inviteStrUrl;
+
+    RsUrl inviteUrl(inviteStrUrl);
+
+    if(inviteUrl.hasQueryK("rsInvite"))
+        rsInvite = *inviteUrl.getQueryV("rsInvite");
+
+    if(!RsCertificate::decodeRadix64ShortInvite(rsInvite, details, err_code))
+        return false;
+
+    // Also check if the PGP key is available. If so, add it in the PeerDetails:
+
+    RsPeerDetails pgp_det ;
+    if(getGPGDetails(PGPHandler::pgpIdFromFingerprint(details.fpr),pgp_det) && pgp_det.fpr == details.fpr)
+    {
+        details.issuer      = pgp_det.issuer;
+        details.gpg_id      = pgp_det.gpg_id;
+        details.gpgSigners  = pgp_det.gpgSigners;
+        details.trustLvl    = pgp_det.trustLvl;
+        details.validLvl    = pgp_det.validLvl;
+        details.ownsign     = pgp_det.ownsign;
+        details.hasSignedMe = pgp_det.hasSignedMe;
+        details.accept_connection = pgp_det.accept_connection;
+    }
+    else
+        details.skip_pgp_signature_validation = true;
+
+    if(details.gpg_id.isNull())
+    {
+		err_code = CERTIFICATE_PARSING_ERROR_MISSING_PGP_FINGERPRINT;
+        return false;
+    }
+    err_code = CERTIFICATE_PARSING_ERROR_NO_ERROR;
+    return true;
 }
 
 bool p3Peers::acceptInvite( const std::string& invite,
@@ -1170,9 +1478,7 @@ bool p3Peers::acceptInvite( const std::string& invite,
 	return true;
 }
 
-std::string p3Peers::GetRetroshareInvite(
-        const RsPeerId& sslId, bool include_signatures,
-        bool includeExtraLocators )
+std::string p3Peers::GetRetroshareInvite( const RsPeerId& sslId, RetroshareInviteFlags invite_flags)
 {
 #ifdef P3PEERS_DEBUG
 	std::cerr << __PRETTY_FUNCTION__ << std::endl;
@@ -1185,14 +1491,31 @@ std::string p3Peers::GetRetroshareInvite(
 
 	if (getPeerDetails(ssl_id, detail))
 	{
-		if(!includeExtraLocators) detail.ipAddressList.clear();
+		if(   !(invite_flags & RetroshareInviteFlags::FULL_IP_HISTORY)
+		   || detail.isHiddenNode)
+			detail.ipAddressList.clear();
+
+		//Check if external address is IPv6, then move it to ipAddressList as RsCertificate only allow 4 numbers.
+		sockaddr_storage tExt;
+		bool validExt =   sockaddr_storage_inet_pton(tExt, detail.extAddr)
+		               && sockaddr_storage_isValidNet(tExt)
+		               && detail.extPort;
+		bool isExtIpv4 = sockaddr_storage_ipv6_to_ipv4(tExt);
+
+		if(   !(invite_flags & RetroshareInviteFlags::FULL_IP_HISTORY)
+		   && !detail.isHiddenNode
+		   && validExt && !isExtIpv4)
+		{
+			sockaddr_storage_setport(tExt,detail.extPort);
+			detail.ipAddressList.push_front(sockaddr_storage_tostring(tExt) + " "); // Space needed to later parse.
+			detail.extAddr = ""; //Clear it to not trigg error.
+			detail.extPort = 0;
+		}
 
 		unsigned char *mem_block = nullptr;
 		size_t mem_block_size = 0;
 
-		if(!AuthGPG::getAuthGPG()->exportPublicKey(
-		            RsPgpId(detail.gpg_id), mem_block, mem_block_size, false,
-		            include_signatures ))
+		if(!AuthPGP::exportPublicKey( RsPgpId(detail.gpg_id), mem_block, mem_block_size, false, !!(invite_flags & RetroshareInviteFlags::PGP_SIGNATURES) ))
 		{
 			std::cerr << "Cannot output certificate for id \"" << detail.gpg_id
 			          << "\". Sorry." << std::endl;
@@ -1200,9 +1523,9 @@ std::string p3Peers::GetRetroshareInvite(
 		}
 
 		RsCertificate cert(detail, mem_block, mem_block_size);
-		delete[] mem_block;
+		free(mem_block);
 
-		return cert.toStdString();
+        return cert.toStdString(invite_flags);
 	}
 
 #ifdef P3PEERS_DEBUG
@@ -1217,9 +1540,10 @@ bool p3Peers::loadCertificateFromString(
         const std::string& cert, RsPeerId& ssl_id,
         RsPgpId& gpg_id, std::string& error_string )
 {
-	RsCertificate crt;
 	uint32_t errNum = 0;
-	if(!crt.initializeFromString(cert,errNum))
+	auto crt = RsCertificate::fromString(cert, errNum);
+
+	if(!crt)
 	{
 		error_string = "RsCertificate failed with errno: "
 		        + std::to_string(errNum) + " parsing: " + cert;
@@ -1227,11 +1551,27 @@ bool p3Peers::loadCertificateFromString(
 	}
 
 	RsPgpId gpgid;
-	bool res = AuthGPG::getAuthGPG()->
-	        LoadCertificateFromString(crt.armouredPGPKey(), gpgid,error_string);
+    bool res = AuthPGP::LoadCertificateFromString( crt->armouredPGPKey(), gpgid, error_string );
 
 	gpg_id = gpgid;
-	ssl_id = crt.sslid();
+	ssl_id = crt->sslid();
+
+    // now get all friends who declare this key ID to be the one needed to check connections, and clear their "skip_pgp_signature_validation" flag
+
+    if(res)
+    {
+		mPeerMgr->notifyPgpKeyReceived(gpgid);
+        FriendsChanged(ssl_id,true);
+    }
+
+	return res;
+}
+bool p3Peers::loadPgpKeyFromBinaryData( const unsigned char *bin_key_data,uint32_t bin_key_len, RsPgpId& gpg_id, std::string& error_string )
+{
+    bool res = AuthPGP::LoadPGPKeyFromBinaryData( bin_key_data,bin_key_len, gpg_id, error_string );
+
+    if(res)
+		mPeerMgr->notifyPgpKeyReceived(gpg_id);
 
 	return res;
 }
@@ -1240,86 +1580,67 @@ bool p3Peers::loadDetailsFromStringCert( const std::string &certstr,
                                          RsPeerDetails &pd,
                                          uint32_t& error_code )
 {
-#ifdef P3PEERS_DEBUG
-	std::cerr << "p3Peers::LoadCertificateFromString() " << std::endl;
-#endif
-	//parse the text to get ip address
-	try 
-	{
-		RsCertificate cert(certstr) ;
+	Dbg3() << __PRETTY_FUNCTION__ << std::endl;
 
-		if(!AuthGPG::getAuthGPG()->getGPGDetailsFromBinaryBlock(cert.pgp_key(),cert.pgp_key_size(), pd.gpg_id,pd.name,pd.gpgSigners))
-			return false;
+	auto certPtr = RsCertificate::fromString(certstr, error_code);
+	if(!certPtr) return false;
 
-#ifdef P3PEERS_DEBUG
-		std::cerr << "Parsing cert for sslid, location, ext and local address details. : " << certstr << std::endl;
-#endif
+	RsCertificate& cert = *certPtr;
 
-		pd.id = cert.sslid() ;
-		pd.location = cert.location_name_string();
-
-		pd.isOnlyGPGdetail = pd.id.isNull();
-        pd.service_perm_flags = RS_NODE_PERM_DEFAULT ;
-
-		if (!cert.hidden_node_string().empty())
-		{
-			pd.isHiddenNode = true;
-
-			std::string domain;
-			uint16_t port;
-			if (splitAddressString(cert.hidden_node_string(), domain, port))
-			{
-				pd.hiddenNodeAddress = domain;
-				pd.hiddenNodePort = port;
-				pd.hiddenType = mPeerMgr->hiddenDomainToHiddenType(domain);
-			}
-		}
-		else
-		{
-			pd.isHiddenNode = false;
-			pd.localAddr = cert.loc_ip_string();
-			pd.localPort = cert.loc_port_us();
-			pd.extAddr = cert.ext_ip_string();
-			pd.extPort = cert.ext_port_us();
-			pd.dyndns = cert.dns_string();
-			for(const RsUrl& locator : cert.locators())
-				pd.ipAddressList.push_back(locator.toString());
-		}
-	}
-	catch(uint32_t e) 
-	{
-		std::cerr << "ConnectFriendWizard : Parse ip address error :" << e << std::endl;
-		error_code = e;
-		return false ;
-	}
-
-	if (pd.gpg_id.isNull())
+    if(!AuthPGP::getPgpDetailsFromBinaryBlock( cert.pgp_key(), cert.pgp_key_size(), pd.gpg_id, pd.name, pd.gpgSigners ))
 		return false;
-	else 
-		return true;
+
+	Dbg4() << __PRETTY_FUNCTION__ << " Parsing cert for sslid, location, ext "
+	       << " and local address details. : " << certstr << std::endl;
+
+	pd.id = cert.sslid();
+	pd.location = cert.location_name_string();
+
+	pd.isOnlyGPGdetail = pd.id.isNull();
+	pd.service_perm_flags = RS_NODE_PERM_DEFAULT;
+
+	if (!cert.hidden_node_string().empty())
+	{
+		pd.isHiddenNode = true;
+
+		std::string domain;
+		uint16_t port;
+		if (splitAddressString(cert.hidden_node_string(), domain, port))
+		{
+			pd.hiddenNodeAddress = domain;
+			pd.hiddenNodePort = port;
+			pd.hiddenType = mPeerMgr->hiddenDomainToHiddenType(domain);
+		}
+	}
+	else
+	{
+		pd.isHiddenNode = false;
+		pd.localAddr = cert.loc_ip_string();
+		pd.localPort = cert.loc_port_us();
+		pd.extAddr = cert.ext_ip_string();
+		pd.extPort = cert.ext_port_us();
+		pd.dyndns = cert.dns_string();
+		for(const RsUrl& locator : cert.locators())
+			pd.ipAddressList.push_back(locator.toString());
+	}
+
+	return true;
 }
 
-bool p3Peers::cleanCertificate(const std::string &certstr, std::string &cleanCert,int& error_code)
+bool p3Peers::cleanCertificate(const std::string &certstr, std::string &cleanCert,bool& is_short_format,uint32_t& error_code,RsPeerDetails& details)
 {
 	RsCertificate::Format format ;
 
-	return RsCertificate::cleanCertificate(certstr,cleanCert,format,error_code,true) ;
-}
+	bool res = RsCertificate::cleanCertificate(certstr,cleanCert,format,error_code,true,details) ;
 
-bool 	p3Peers::saveCertificateToFile(const RsPeerId &id, const std::string &/*fname*/)
-{
-	/* remove unused parameter warnings */
-	(void) id;
+    if(format == RsCertificate::RS_CERTIFICATE_RADIX)
+        is_short_format = false;
+    else if(format == RsCertificate::RS_CERTIFICATE_SHORT_RADIX)
+        is_short_format = true;
+    else
+        return false ;
 
-#ifdef P3PEERS_DEBUG
-        std::cerr << "p3Peers::SaveCertificateToFile() not implemented yet " << id;
-	std::cerr << std::endl;
-#endif
-
-//	ensureExtension(fname, "pqi");
-//
-//        return AuthSSL::getAuthSSL()->SaveCertificateToFile(id, fname);
-        return false;
+    return res;
 }
 
 std::string p3Peers::saveCertificateToString(const RsPeerId &id)
@@ -1335,26 +1656,30 @@ std::string p3Peers::saveCertificateToString(const RsPeerId &id)
         }
 }
 
-bool 	p3Peers::signGPGCertificate(const RsPgpId &id)
+bool 	p3Peers::signGPGCertificate(const RsPgpId &id, const std::string &gpg_passphrase)
 {
 #ifdef P3PEERS_DEBUG
 	std::cerr << "p3Peers::SignCertificate() " << id;
 	std::cerr << std::endl;
 #endif
+        rsNotify->cachePgpPassphrase(gpg_passphrase);
+        rsNotify->setDisableAskPassword(true);
 
+        bool res = AuthPGP::SignCertificateLevel0(id);
 
-        AuthGPG::getAuthGPG()->AllowConnection(id, true);
-        return AuthGPG::getAuthGPG()->SignCertificateLevel0(id);
+        rsNotify->clearPgpPassphrase();
+        rsNotify->setDisableAskPassword(false);
+
+        return res;
 }
 
-
 bool 	p3Peers::trustGPGCertificate(const RsPgpId &id, uint32_t trustlvl)
 {
 #ifdef P3PEERS_DEBUG
 	std::cerr << "p3Peers::TrustCertificate() " << id;
 	std::cerr << std::endl;
 #endif
-	return AuthGPG::getAuthGPG()->TrustCertificate(id, trustlvl);
+    return AuthPGP::TrustCertificate(id, trustlvl);
 }
 
 	/* Group Stuff */
@@ -1484,7 +1809,7 @@ RsPeerDetails::RsPeerDetails()
         :isOnlyGPGdetail(false),
           name(""),email(""),location(""),
           org(""),authcode(""),
-          trustLvl(0), validLvl(0),ownsign(false), 
+          trustLvl(0), validLvl(0),skip_pgp_signature_validation(false),ownsign(false),
           hasSignedMe(false),accept_connection(false),
           state(0),actAsServer(false),
           connectPort(0),
@@ -1495,63 +1820,9 @@ RsPeerDetails::RsPeerDetails()
           lastConnect(0),lastUsed(0),connectState(0),connectStateString(""),
           connectPeriod(0),
           foundDHT(false), wasDeniedConnection(false), deniedTS(0),
-          linkType ( RS_NET_CONN_TRANS_TCP_UNKNOWN)
-{
-}
+          linkType ( RS_NET_CONN_TRANS_TCP_UNKNOWN) {}
 
-std::ostream &operator<<(std::ostream &out, const RsPeerDetails &detail)
-{
-	out << "RsPeerDetail: " << detail.name << "  <" << detail.id << ">";
-	out << std::endl;
-
-	out << " email:   " << detail.email;
-	out << " location:" << detail.location;
-	out << " org:     " << detail.org;
-	out << std::endl;
-
-	out << " fpr:     " << detail.fpr;
-	out << " authcode:" << detail.authcode;
-	out << std::endl;
-
-	out << " signers:";
-	out << std::endl;
-
-	std::list<RsPgpId>::const_iterator it;
-        for(it = detail.gpgSigners.begin();
-                it != detail.gpgSigners.end(); ++it)
-	{
-		out << "\t" << *it;
-		out << std::endl;
-	}
-	out << std::endl;
-
-	out << " trustLvl:    " << detail.trustLvl;
-        out << " ownSign:     " << detail.ownsign;
-	out << std::endl;
-
-	out << " state:       " << detail.state;
-	out << " netMode:     " << detail.netMode;
-	out << std::endl;
-
-	out << " localAddr:   " << detail.localAddr;
-	out << ":" << detail.localPort;
-	out << std::endl;
-	out << " extAddr:   " << detail.extAddr;
-	out << ":" << detail.extPort;
-	out << std::endl;
-
-
-	out << " lastConnect:       " << detail.lastConnect;
-	out << " connectPeriod:     " << detail.connectPeriod;
-	out << std::endl;
-
-	return out;
-}
-
-RsGroupInfo::RsGroupInfo()
-{
-	flag = 0;
-}
+RsGroupInfo::RsGroupInfo() : flag(0) {}
 
 ServicePermissionFlags p3Peers::servicePermissionFlags(const RsPeerId& ssl_id) 
 {
@@ -1566,4 +1837,9 @@ void p3Peers::setServicePermissionFlags(const RsPgpId& gpg_id,const ServicePermi
 	mPeerMgr->setServicePermissionFlags(gpg_id,flags) ;
 }
 
+RsPeerStateChangedEvent::RsPeerStateChangedEvent(RsPeerId sslId) :
+    RsEvent(RsEventType::PEER_STATE_CHANGED), mSslId(sslId) {}
 
+RsPeers::~RsPeers() = default;
+RsAuthSslConnectionAutenticationEvent::~RsAuthSslConnectionAutenticationEvent() = default;
+RsConnectionEvent::~RsConnectionEvent() = default;
