@@ -32,21 +32,28 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-
+#include <system_error>
 #include <iostream>
 
+#include <openssl/ssl.h>
+
 #include "retroshare/rsinit.h"
 #include "rsaccounts.h"
-
+#include "util/rsdebug.h"
 #include "util/rsdir.h"
 #include "util/rsstring.h"
 #include "util/folderiterator.h"
-
+#include "util/largefile_retrocompat.hpp"
 #include "pqi/authssl.h"
 #include "pqi/sslfns.h"
 #include "pqi/authgpg.h"
 
-#include <openssl/ssl.h>
+
+
+#ifdef __ANDROID__
+#	include "rs_android/rsjni.hpp"
+#	include "rs_android/retroshareserviceandroid.hpp"
+#endif
 
 // Global singleton declaration of data.
 RsAccountsDetail* RsAccounts::rsAccountsDetails = nullptr;
@@ -328,22 +335,7 @@ bool RsAccountsDetail::defaultBaseDirectory()
 {
 	std::string basedir;
 
-/******************************** WINDOWS/UNIX SPECIFIC PART ******************/
-#ifndef WINDOWS_SYS
-
-	// unix: homedir + /.retroshare
-	char *h = getenv("HOME");
-	if (h == NULL)
-	{
-		std::cerr << "defaultBaseDirectory() Error: cannot determine $HOME dir"
-		          << std::endl;
-		return false ;
-	}
-
-	basedir = h;
-	basedir += "/.retroshare";
-
-#else
+#ifdef WINDOWS_SYS
 	if (RsInit::isPortable())
 	{
 		// use directory "Data" in portable version
@@ -375,13 +367,53 @@ bool RsAccountsDetail::defaultBaseDirectory()
 		}
 		basedir += "\\RetroShare";
 	}
-#endif
-/******************************** WINDOWS/UNIX SPECIFIC PART ******************/
+#elif defined (__ANDROID__) // def WINDOWS_SYS
+
+	struct ApplicationInfo
+	{
+		static constexpr auto Name()
+		{ return "android/content/pm/ApplicationInfo"; }
+	};
+
+	auto uenv = jni::GetAttachedEnv(RsJni::getVM());
+	JNIEnv& env = *uenv;
+	auto androidContext = RetroShareServiceAndroid::getAndroidContext(env);
+	auto& contextClass =
+	        jni::Class<RetroShareServiceAndroid::Context>::Singleton(env);
+
+	auto& applicationInfoClass = jni::Class<ApplicationInfo>::Singleton(env);
+
+	auto getApplicationInfo =
+	        contextClass.GetMethod<jni::Object<ApplicationInfo> ()>(
+				env, "getApplicationInfo" );
+
+	auto applicationInfo = androidContext.Call(env, getApplicationInfo);
+
+	auto dataDirField = jni::Field<ApplicationInfo, jni::String>(
+	            env, applicationInfoClass, "dataDir" );
+
+	jni::Local<jni::String> dataDir = applicationInfo.Get<jni::String>(
+				env, dataDirField );
+
+	basedir = jni::Make<std::string>(env, dataDir) + "/.retroshare/";
+
+#else // def WINDOWS_SYS, if defined (__ANDROID__)
+	// unix: homedir + /.retroshare
+	char* h = getenv("HOME");
+	if(h == nullptr)
+	{
+		RS_ERR("cannot determine $HOME dir");
+		return false ;
+	}
+
+	basedir = h;
+	basedir += "/.retroshare";
+#endif // def WINDOWS_SYS
 
 	/* store to class variable */
 	mBaseDirectory = basedir;
-	std::cerr << "defaultBaseDirectory() = " << mBaseDirectory;
-	std::cerr << std::endl;
+
+	RS_INFO(mBaseDirectory);
 	return true;
 }
 
@@ -517,8 +549,7 @@ bool RsAccountsDetail::getAvailableAccounts(std::map<RsPeerId, AccountDetails> &
 	std::list<std::string> directories;
 	std::list<std::string>::iterator it;
 
-	std::cerr << "RsAccounts::getAvailableAccounts()";
-	std::cerr << std::endl;
+    RsInfo() << "RsAccounts: parsing existing accounts.";
 
 	/* now iterate through the directory...
 	 * directories - flags as old,
@@ -530,7 +561,7 @@ bool RsAccountsDetail::getAvailableAccounts(std::map<RsPeerId, AccountDetails> &
 
 	if (!dirIt.isValid())
 	{
-		std::cerr << "Cannot Open Base Dir - No Available Accounts" << std::endl;
+        RsErr() << "  Cannot Open Base Dir - No Available Accounts" ;
 		return false ;
 	}
 
@@ -589,7 +620,7 @@ bool RsAccountsDetail::getAvailableAccounts(std::map<RsPeerId, AccountDetails> &
 
 		if (it->length() != 32 + 6)
 		{
-			std::cerr << "getAvailableAccounts() Skipping Invalid sized dir: " << *it << std::endl;
+            RsErr() << "  getAvailableAccounts() Skipping Invalid sized dir: " << *it ;
 			continue;
 		}
 
@@ -613,7 +644,7 @@ bool RsAccountsDetail::getAvailableAccounts(std::map<RsPeerId, AccountDetails> &
 		}
 		else
 		{
-			std::cerr << "getAvailableAccounts() Skipping Invalid Prefix dir: " << *it << std::endl;
+            RsErr() << "  Skipping Invalid Prefix dir: " << *it ;
 			continue;
 		}
 
@@ -642,13 +673,9 @@ bool RsAccountsDetail::getAvailableAccounts(std::map<RsPeerId, AccountDetails> &
 				ait = accounts.find(tmpId.mSslId);
 				if (ait != accounts.end())
 				{
-					std::cerr << "getAvailableAccounts() ERROR Duplicate SSLIDs";
-					std::cerr << " - only one will be available";
-					std::cerr << std::endl;
-					std::cerr << " ID1 (overridden) : " << ait->first << " Directory: " << ait->second.mAccountDir;
-					std::cerr << std::endl;
-					std::cerr << " ID2 (available)  : " << tmpId.mSslId << " Directory: " << tmpId.mAccountDir;
-					std::cerr << std::endl;
+                    RsErr() << "  getAvailableAccounts() ERROR Duplicate SSLIDs - only one will be available";
+                    RsErr() << "    ID1 (overridden) : " << ait->first << " Directory: " << ait->second.mAccountDir;
+                    RsErr() << "    ID2 (available)  : " << tmpId.mSslId << " Directory: " << tmpId.mAccountDir;
 
 				}
 
@@ -686,7 +713,8 @@ static bool checkAccount(const std::string &accountdir, AccountDetails &account,
 	bool ret = false;
 
 	/* check against authmanagers private keys */
-	if (LoadCheckX509(cert_name.c_str(), account.mPgpId, account.mLocation, account.mSslId))
+	if(AuthSSL::instance().parseX509DetailsFromFile(
+	            cert_name, account.mSslId, account.mPgpId, account.mLocation ))
 	{
         // new locations store the name in an extra file
         if(account.mLocation == "")
@@ -700,10 +728,10 @@ static bool checkAccount(const std::string &accountdir, AccountDetails &account,
 		if(! RsAccounts::GetPGPLoginDetails(account.mPgpId, account.mPgpName, account.mPgpEmail))
 			return false ;
 
-		if(!AuthGPG::getAuthGPG()->haveSecretKey(account.mPgpId))
+        if(!AuthPGP::haveSecretKey(account.mPgpId))
 			return false ;
 
-		if(!AuthGPG::getAuthGPG()->isKeySupported(account.mPgpId))
+        if(!AuthPGP::isKeySupported(account.mPgpId))
 		{
 			std::string keystring = account.mPgpId.toStdString() + " " + account.mPgpName + "&#60;" + account.mPgpEmail ;
 			unsupported_keys[keystring].push_back("Location: " + account.mLocation + "&nbsp;&nbsp;(" + account.mSslId.toStdString() + ")") ;
@@ -810,14 +838,15 @@ static bool checkAccount(const std::string &accountdir, AccountDetails &account,
 
 	/* Use RetroShare's exe dir */
 	dataDirectory = ".";
-#elif defined(ANDROID)
+#elif defined(__ANDROID__)
+	// TODO: This is probably not really used on Android
 	dataDirectory = PathBaseDirectory()+"/usr/share/retroshare";
-#elif defined(DATA_DIR)
+#elif defined(RS_DATA_DIR)
 	// cppcheck-suppress ConfigurationNotChecked
-	dataDirectory = DATA_DIR;
+	dataDirectory = RS_DATA_DIR;
 	// For all other OS the data directory must be set in libretroshare.pro
 #else
-#	error "For your target OS automatic data dir discovery is not supported, cannot compile if DATA_DIR variable not set."
+#	error "For your target OS automatic data dir discovery is not supported, cannot compile if RS_DATA_DIR variable not set."
 #endif
 
 	if (!check)
@@ -850,9 +879,10 @@ static bool checkAccount(const std::string &accountdir, AccountDetails &account,
 
 
                 /* Generating GPGme Account */
-int      RsAccountsDetail::GetPGPLogins(std::list<RsPgpId> &pgpIds) {
-        AuthGPG::getAuthGPG()->availableGPGCertificatesWithPrivateKeys(pgpIds);
-	return 1;
+int      RsAccountsDetail::GetPGPLogins(std::list<RsPgpId>& pgpIds)
+{
+    AuthPGP::availablePgpCertificatesWithPrivateKeys(pgpIds);
+    return 1;
 }
 
 int      RsAccountsDetail::GetPGPLoginDetails(const RsPgpId& id, std::string &name, std::string &email)
@@ -862,10 +892,10 @@ int      RsAccountsDetail::GetPGPLoginDetails(const RsPgpId& id, std::string &na
         #endif
 
 		  bool ok = true ;
-        name = AuthGPG::getAuthGPG()->getGPGName(id,&ok);
+        name = AuthPGP::getPgpName(id,&ok);
 		  if(!ok)
 			  return 0 ;
-        email = AuthGPG::getAuthGPG()->getGPGEmail(id,&ok);
+        email = AuthPGP::getPgpEmail(id,&ok);
 		  if(!ok)
 			  return 0 ;
 
@@ -885,7 +915,7 @@ bool RsAccountsDetail::SelectPGPAccount(const RsPgpId& pgpId)
 {
 	bool retVal = false;
 
-	if (0 < AuthGPG::getAuthGPG() -> GPGInit(pgpId))
+    if (0 < AuthPGP::PgpInit(pgpId))
 	{
 		retVal = true;
 #ifdef DEBUG_ACCOUNTS
@@ -905,7 +935,7 @@ bool RsAccountsDetail::SelectPGPAccount(const RsPgpId& pgpId)
 
 bool     RsAccountsDetail::GeneratePGPCertificate(const std::string& name, const std::string& email, const std::string& passwd, RsPgpId &pgpId, const int keynumbits, std::string &errString)
 {
-	return AuthGPG::getAuthGPG()->GeneratePGPCertificate(name, email, passwd, pgpId, keynumbits, errString);
+    return AuthPGP::GeneratePgpCertificate(name, email, passwd, pgpId, keynumbits, errString);
 }
 
 		// PGP Support Functions.
@@ -917,29 +947,34 @@ void RsAccountsDetail::getUnsupportedKeys(std::map<std::string,std::vector<std::
 
 bool RsAccountsDetail::exportIdentity(const std::string& fname,const RsPgpId& id)
 {
-	return AuthGPG::getAuthGPG()->exportProfile(fname,id);
+    return AuthPGP::exportProfile(fname,id);
 }
 
 bool RsAccountsDetail::importIdentity(const std::string& fname,RsPgpId& id,std::string& import_error)
 {
-	return AuthGPG::getAuthGPG()->importProfile(fname,id,import_error);
+    return AuthPGP::importProfile(fname,id,import_error);
 }
 
 bool RsAccountsDetail::importIdentityFromString(const std::string &data, RsPgpId &imported_pgp_id, std::string &import_error)
 {
-    return AuthGPG::getAuthGPG()->importProfileFromString(data, imported_pgp_id, import_error);
+    return AuthPGP::importProfileFromString(data, imported_pgp_id, import_error);
 }
 
 bool RsAccountsDetail::exportIdentityToString(
         std::string& data, const RsPgpId& pgpId, bool includeSignatures,
         std::string& errorMsg )
 {
-	return AuthGPG::getAuthGPG()->exportIdentityToString(
+    return AuthPGP::exportIdentityToString(
 	            data, pgpId, includeSignatures, errorMsg );
 }
 
 bool RsAccountsDetail::copyGnuPGKeyrings()
 {
+#ifdef __ANDROID__
+	RS_ERR(std::errc::not_supported);
+	print_stacktrace();
+	return false;
+#else
 	std::string pgp_dir = PathPGPDirectory() ;
 
 	if(!RsDirUtil::checkCreateDirectory(pgp_dir))
@@ -991,6 +1026,7 @@ bool RsAccountsDetail::copyGnuPGKeyrings()
 	}
 
 	return true ;
+#endif // def __ANDROID__
 }
 
 
@@ -1019,7 +1055,7 @@ bool     RsAccountsDetail::GenerateSSLCertificate(const RsPgpId& pgp_id, const s
 
 	int nbits = 4096;
 
-	//std::string pgp_name = AuthGPG::getAuthGPG()->getGPGName(pgp_id);
+    //std::string pgp_name = AuthGPG::getGPGName(pgp_id);
 
 	// Create the filename .....
 	// Temporary Directory for creating files....
@@ -1117,8 +1153,11 @@ bool     RsAccountsDetail::GenerateSSLCertificate(const RsPgpId& pgp_id, const s
 	std::string location;
 	RsPgpId pgpid_retrieved;
 
-	if (LoadCheckX509(cert_name.c_str(), pgpid_retrieved, location, sslId) == 0) {
-		std::cerr << "RsInit::GenerateSSLCertificate() Cannot check own signature, maybe the files are corrupted." << std::endl;
+	if(!AuthSSL::instance().parseX509DetailsFromFile(
+	            cert_name, sslId, pgpid_retrieved, location ))
+	{
+		RsErr() << __PRETTY_FUNCTION__ << " Cannot check own signature, maybe "
+		        << "the files are corrupted." << std::endl;
 		return false;
 	}
 
@@ -1182,14 +1221,6 @@ bool RsAccountsDetail::setupAccount(const std::string& accountdir)
 	std::string subdir2 = accountdir + "/";
 	subdir2 += kPathConfigDirectory;
 
-	std::string subdir3 = accountdir + "/";
-	subdir3 += "cache";
-
-	std::string subdir4 = subdir3 + "/";
-	std::string subdir5 = subdir3 + "/";
-	subdir4 += "local";
-	subdir5 += "remote";
-
 	// fatal if cannot find/create.
 	std::cerr << "Checking For Directories" << std::endl;
 	if (!RsDirUtil::checkCreateDirectory(accountdir))
@@ -1207,22 +1238,6 @@ bool RsAccountsDetail::setupAccount(const std::string& accountdir)
 		std::cerr << "Cannot Create Config Directory" << std::endl;
 		return false ;
 	}
-	if (!RsDirUtil::checkCreateDirectory(subdir3))
-	{
-		std::cerr << "Cannot Create Config/Cache Dir" << std::endl;
-		return false ;
-	}
-	if (!RsDirUtil::checkCreateDirectory(subdir4))
-	{
-		std::cerr << "Cannot Create Config/Cache/local Dir" << std::endl;
-		return false ;
-	}
-	if (!RsDirUtil::checkCreateDirectory(subdir5))
-	{
-		std::cerr << "Cannot Create Config/Cache/remote Dir" << std::endl;
-		return false ;
-	}
-
 	return true;
 }
 
@@ -1291,7 +1306,7 @@ bool RsAccounts::init(const std::string& opt_base_dir,int& error_code)
 	if(!RsDirUtil::checkCreateDirectory(pgp_dir))
 		throw std::runtime_error("Cannot create pgp directory " + pgp_dir) ;
 
-	AuthGPG::init(	pgp_dir + "/retroshare_public_keyring.gpg",
+	AuthPGP::init(	pgp_dir + "/retroshare_public_keyring.gpg",
 	                pgp_dir + "/retroshare_secret_keyring.gpg",
 	                pgp_dir + "/retroshare_trustdb.gpg",
 	                pgp_dir + "/lock");
