@@ -1,9 +1,8 @@
 /*******************************************************************************
  * libretroshare/src/services: p3idservice.h                                   *
  *                                                                             *
- * libretroshare: retroshare core library                                      *
- *                                                                             *
- * Copyright 2012-2012 Robert Fernie <retroshare@lunamutt.com>                 *
+ * Copyright (C) 2012-2014  Robert Fernie <retroshare@lunamutt.com>            *
+ * Copyright (C) 2017-2019  Gioacchino Mazzurco <gio@altermundi.net>           *
  *                                                                             *
  * This program is free software: you can redistribute it and/or modify        *
  * it under the terms of the GNU Lesser General Public License as              *
@@ -19,51 +18,28 @@
  * along with this program. If not, see <https://www.gnu.org/licenses/>.       *
  *                                                                             *
  *******************************************************************************/
-#ifndef P3_IDENTITY_SERVICE_HEADER
-#define P3_IDENTITY_SERVICE_HEADER
+#pragma once
 
+/// RetroShare GXS identities service
 
-#include "retroshare/rsidentity.h"	// External Interfaces.
-#include "gxs/rsgenexchange.h"		// GXS service.
-#include "gxs/rsgixs.h"			// Internal Interfaces.
-
-#include "gxs/gxstokenqueue.h"		
-#include "rsitems/rsgxsiditems.h"
 
 #include <map>
 #include <string>
 
+#include "retroshare/rsidentity.h"	// External Interfaces.
+#include "gxs/rsgenexchange.h"		// GXS service.
+#include "gxs/rsgixs.h"			// Internal Interfaces.
+#include "util/rsdebug.h"
+#include "gxs/gxstokenqueue.h"		
+#include "rsitems/rsgxsiditems.h"
 #include "util/rsmemcache.h"
 #include "util/rstickevent.h"
 #include "util/rsrecogn.h"
-
 #include "pqi/authgpg.h"
-
 #include "rsitems/rsgxsrecognitems.h"
 
 class PgpAuxUtils;
 
-/* 
- * Identity Service
- *
- */
-
-#if 0
-class GxsReputation
-{
-	public:
-	GxsReputation();
-
-	bool updateIdScore(bool pgpLinked, bool pgpKnown);
-	bool update();    // checks ranges and calculates overall score.
-	int mOverallScore;
-	int mIdScore;      // PGP, Known, etc.
-	int mOwnOpinion;
-	int mPeerOpinion;
-};
-
-#endif
-
 class OpinionRequest
 {
 	public:
@@ -215,18 +191,17 @@ struct SerialisedIdentityStruct
     rstime_t mLastUsageTS;
 };
 
-// Not sure exactly what should be inherited here?
-// Chris - please correct as necessary.
-
+// We cache all identities, and provide alternative (instantaneous)
+// functions to extract info, rather than the horrible Token system.
 class p3IdService: public RsGxsIdExchange, public RsIdentity,  public GxsTokenQueue, public RsTickEvent, public p3Config
 {
 public:
 	p3IdService(RsGeneralDataService* gds, RsNetworkExchangeService* nes, PgpAuxUtils *pgpUtils);
 
-	virtual RsServiceInfo getServiceInfo();
+    virtual RsServiceInfo getServiceInfo()override;
 	static	uint32_t idAuthenPolicy();
 
-	virtual void service_tick(); // needed for background processing.
+    virtual void service_tick()override; // needed for background processing.
 
 
 	/*!
@@ -239,19 +214,29 @@ public:
 
 	/* Data Specific Interface */
 
+	/// @see RsIdentity
+	bool getIdentitiesInfo(const std::set<RsGxsId>& ids,
+	        std::vector<RsGxsIdGroup>& idsInfo ) override;
+
+	/// @see RsIdentity
+	bool getIdentitiesSummaries(std::list<RsGroupMetaData>& ids) override;
+
 	// These are exposed via RsIdentity.
-	virtual bool getGroupData(const uint32_t &token, std::vector<RsGxsIdGroup> &groups);
-	virtual bool getGroupSerializedData(const uint32_t &token, std::map<RsGxsId,std::string>& serialized_groups);
+    virtual bool getGroupData(const uint32_t &token, std::vector<RsGxsIdGroup> &groups) override;
+    virtual bool getGroupSerializedData(const uint32_t &token, std::map<RsGxsId,std::string>& serialized_groups) override;
 
 	//virtual bool getMsgData(const uint32_t &token, std::vector<RsGxsIdOpinion> &opinions);
 
 	// These are local - and not exposed via RsIdentity.
 	virtual bool createGroup(uint32_t& token, RsGxsIdGroup &group);
 	virtual bool updateGroup(uint32_t& token, RsGxsIdGroup &group);
-	virtual bool deleteGroup(uint32_t& token, RsGxsIdGroup &group);
+	virtual bool deleteGroup(uint32_t& token, RsGxsGroupId& group);
 	//virtual bool createMsg(uint32_t& token, RsGxsIdOpinion &opinion);
 
-	/**************** RsIdentity External Interface.
+    virtual bool receiveNewIdentity(RsNxsGrp *identity_grp) override;
+    virtual bool retrieveNxsIdentity(const RsGxsId& group_id,RsNxsGrp *& identity_grp)override;
+
+    /**************** RsIdentity External Interface.
 	 * Notes:
 	 * 
 	 * All the data is cached together for the moment - We should probably
@@ -261,51 +246,89 @@ public:
 	 * 
 	 */
 	//virtual bool  getNickname(const RsGxsId &id, std::string &nickname);
-	virtual bool  getIdDetails(const RsGxsId &id, RsIdentityDetails &details);
+    virtual bool  getIdDetails(const RsGxsId &id, RsIdentityDetails &details) override;
 
-	// 
+	RS_DEPRECATED_FOR(RsReputations)
 	virtual bool submitOpinion(uint32_t& token, const RsGxsId &id, 
-	                           bool absOpinion, int score);
-	virtual bool createIdentity(uint32_t& token, RsIdentityParameters &params);
+                               bool absOpinion, int score) override;
 
-	virtual bool updateIdentity(uint32_t& token, RsGxsIdGroup &group);
-	virtual bool deleteIdentity(uint32_t& token, RsGxsIdGroup &group);
+	/// @see RsIdentity
+	virtual bool createIdentity(
+	        RsGxsId& id,
+	        const std::string& name, const RsGxsImage& avatar = RsGxsImage(),
+	        bool pseudonimous = true, const std::string& pgpPassword = "" ) override;
 
-    virtual void setDeleteBannedNodesThreshold(uint32_t days) ;
-    virtual uint32_t deleteBannedNodesThreshold() ;
+    virtual bool createIdentity(uint32_t& token, RsIdentityParameters &params) override;
+
+	/// @see RsIdentity
+    bool updateIdentity( const RsGxsId& id, const std::string& name, const RsGxsImage& avatar, bool pseudonimous, const std::string& pgpPassword) override;
+
+	/// @see RsIdentity
+	bool deleteIdentity(RsGxsId& id) override;
+
+	RS_DEPRECATED
+    virtual bool deleteIdentity(uint32_t& token, RsGxsIdGroup &group) override;
+
+    virtual void setDeleteBannedNodesThreshold(uint32_t days)  override;
+    virtual uint32_t deleteBannedNodesThreshold()  override;
 
 	virtual bool parseRecognTag(const RsGxsId &id, const std::string &nickname,
-	                            const std::string &tag, RsRecognTagDetails &details);
+                                const std::string &tag, RsRecognTagDetails &details) override;
 	virtual bool getRecognTagRequest(const RsGxsId &id, const std::string &comment, 
-	                                 uint16_t tag_class, uint16_t tag_type, std::string &tag);
+                                     uint16_t tag_class, uint16_t tag_type, std::string &tag) override;
 
 	virtual bool setAsRegularContact(const RsGxsId& id,bool is_a_contact) override;
 	virtual bool isARegularContact(const RsGxsId& id) override;
 	virtual void setAutoAddFriendIdsAsContact(bool b) override;
 	virtual bool autoAddFriendIdsAsContact() override;
 
-	virtual uint32_t nbRegularContacts() ;
-	virtual rstime_t getLastUsageTS(const RsGxsId &id) ;
+    virtual uint32_t nbRegularContacts()  override;
+    virtual rstime_t getLastUsageTS(const RsGxsId &id) override ;
 
 	/**************** RsGixs Implementation ***************/
 
-	virtual bool getOwnIds(std::list<RsGxsId> &ownIds, bool signed_only = false);
+	/// @see RsIdentity
+	bool getOwnSignedIds(std::vector<RsGxsId>& ids) override;
 
-	//virtual bool getPublicKey(const RsGxsId &id, RsTlvSecurityKey &key) ;
-	//virtual void networkRequestPublicKey(const RsGxsId& key_id,const std::list<RsPeerId>& peer_ids) ;
+	/// @see RsIdentity
+	bool getOwnPseudonimousIds(std::vector<RsGxsId>& ids) override;
+
+	/// @see RsIdentity
+	bool getOwnIds(
+	        std::list<RsGxsId> &ownIds, bool signed_only = false ) override;
+
+	/// @see RsIdentity
+	bool isKnownId(const RsGxsId& id) override;
+
+	/// @see RsIdentity
+	bool isOwnId(const RsGxsId& key_id) override;
+
+	/// @see RsIdentity
+	bool exportIdentityLink(
+	        std::string& link, const RsGxsId& id,
+	        bool includeGxsData = true,
+	        const std::string& baseUrl = DEFAULT_IDENTITY_BASE_URL,
+	        std::string& errMsg = RS_DEFAULT_STORAGE_PARAM(std::string)
+	        ) override;
+
+	/// @see RsIdentity
+	bool importIdentityLink(
+	        const std::string& link,
+	        RsGxsId& id = RS_DEFAULT_STORAGE_PARAM(RsGxsId),
+	        std::string& errMsg = RS_DEFAULT_STORAGE_PARAM(std::string)
+	        ) override;
 
-	virtual bool isOwnId(const RsGxsId& key_id) ;
 
 	virtual bool signData( const uint8_t* data,
 	                       uint32_t data_size,
 	                       const RsGxsId& signer_id,
 	                       RsTlvKeySignature& signature,
-	                       uint32_t& signing_error);
+                           uint32_t& signing_error)override;
 
 	virtual bool validateData( const uint8_t *data, uint32_t data_size,
 	                           const RsTlvKeySignature& signature,
 	                           bool force_load, const RsIdentityUsage &info,
-	                           uint32_t& signing_error );
+                               uint32_t& signing_error )override;
 
 	virtual bool encryptData( const uint8_t* decrypted_data,
 	                          uint32_t decrypted_data_size,
@@ -313,7 +336,7 @@ public:
 	                          uint32_t& encrypted_data_size,
 	                          const RsGxsId& encryption_key_id,
 	                          uint32_t& error_status,
-	                          bool force_load = true );
+                              bool force_load = true )override;
 
 	bool encryptData( const uint8_t* decrypted_data,
 	                  uint32_t decrypted_data_size,
@@ -328,7 +351,7 @@ public:
 	                          uint32_t& decrypted_data_size,
 	                          const RsGxsId& decryption_key_id,
 	                          uint32_t& error_status,
-	                          bool force_load = true );
+                              bool force_load = true )override;
 
 	virtual bool decryptData(const uint8_t* encrypted_data,
 	                          uint32_t encrypted_data_size,
@@ -336,24 +359,30 @@ public:
 	                          uint32_t& decrypted_data_size,
 	                          const std::set<RsGxsId>& decrypt_ids,
 	                          uint32_t& error_status,
-	                          bool force_load = true );
+                              bool force_load = true );
 
 
-	virtual bool haveKey(const RsGxsId &id);
-	virtual bool havePrivateKey(const RsGxsId &id);
+    virtual bool haveKey(const RsGxsId &id)override;
+    virtual bool havePrivateKey(const RsGxsId &id)override;
 
-	virtual bool getKey(const RsGxsId &id, RsTlvPublicRSAKey &key);
-	virtual bool getPrivateKey(const RsGxsId &id, RsTlvPrivateRSAKey &key);
+    virtual bool getKey(const RsGxsId &id, RsTlvPublicRSAKey &key)override;
+    virtual bool getPrivateKey(const RsGxsId &id, RsTlvPrivateRSAKey &key)override;
 
 	virtual bool requestKey( const RsGxsId &id,
 	                         const std::list<RsPeerId> &peers,
-	                         const RsIdentityUsage &use_info );
-	virtual bool requestPrivateKey(const RsGxsId &id);
+                             const RsIdentityUsage &use_info )override;
+    virtual bool requestPrivateKey(const RsGxsId &id)override;
 
-	virtual bool serialiseIdentityToMemory(const RsGxsId& id,
-	                                       std::string& radix_string);
-	virtual bool deserialiseIdentityFromMemory(const std::string& radix_string,
-	                                           RsGxsId* id = nullptr);
+	RS_DEPRECATED_FOR(exportIdentityLink)
+    virtual bool serialiseIdentityToMemory(const RsGxsId& id, std::string& radix_string) override;
+	RS_DEPRECATED_FOR(importIdentityLink)
+    virtual bool deserialiseIdentityFromMemory(const std::string& radix_string, RsGxsId* id = nullptr) override;
+
+	/// @see RsIdentity
+	bool requestIdentity(
+	            const RsGxsId& id,
+	            const std::vector<RsPeerId>& peers = std::vector<RsPeerId>()
+	        ) override;
 
 	/**************** RsGixsReputation Implementation ****************/
 
@@ -367,19 +396,20 @@ public:
 
 protected:
 	/** Notifications **/
-	virtual void notifyChanges(std::vector<RsGxsNotify*>& changes);
+    virtual void notifyChanges(std::vector<RsGxsNotify*>& changes) override;
 
 	/** Overloaded to add PgpIdHash to Group Definition **/
-	virtual ServiceCreate_Return service_CreateGroup(RsGxsGrpItem* grpItem, RsTlvSecurityKeySet& keySet);
+    virtual ServiceCreate_Return service_CreateGroup(RsGxsGrpItem* grpItem, RsTlvSecurityKeySet& keySet) override;
 
     // Overloads RsGxsGenExchange
-    virtual bool acceptNewGroup(const RsGxsGrpMetaData *grpMeta) ;
+    virtual bool acceptNewGroup(const RsGxsGrpMetaData *grpMeta) override ;
 
-    // Overloaded from GxsTokenQueue for Request callbacks.
-	virtual void handleResponse(uint32_t token, uint32_t req_type);
+	// Overloaded from GxsTokenQueue for Request callbacks.
+	virtual void handleResponse(uint32_t token, uint32_t req_type
+	                            , RsTokenService::GxsRequestStatus status) override;
 
 	// Overloaded from RsTickEvent.
-	virtual void handle_event(uint32_t event_type, const std::string &elabel);
+    virtual void handle_event(uint32_t event_type, const std::string &elabel) override;
 
 	//===================================================//
 	//                  p3Config methods                 //
@@ -387,10 +417,10 @@ protected:
 
 	// Load/save the routing info, the pending items in transit, and the config variables.
 	//
-	virtual bool loadList(std::list<RsItem*>& items) ;
-	virtual bool saveList(bool& cleanup,std::list<RsItem*>& items) ;
+    virtual bool loadList(std::list<RsItem*>& items) override ;
+    virtual bool saveList(bool& cleanup,std::list<RsItem*>& items) override ;
 
-	virtual RsSerialiser *setupSerialiser() ;
+    virtual RsSerialiser *setupSerialiser() override ;
 
 
 private:
@@ -458,7 +488,8 @@ private:
  *
  */
 	bool pgphash_start();
-	bool pgphash_handlerequest(uint32_t token);
+    bool pgphash_load_group_data(uint32_t token);
+    bool pgphash_handlerequest(uint32_t token);
 	bool pgphash_process();
 
 	bool checkId(const RsGxsIdGroup &grp, RsPgpId &pgp_id, bool &error);
@@ -466,8 +497,8 @@ private:
 
 	/* MUTEX PROTECTED DATA (mIdMtx - maybe should use a 2nd?) */
 
-	std::map<RsPgpId, PGPFingerprintType> mPgpFingerprintMap;
-	std::list<RsGxsIdGroup> mGroupsToProcess;
+	std::map<RsPgpId, RsPgpFingerprint> mPgpFingerprintMap;
+    std::map<RsGxsGroupId,RsGxsIdGroup> mGroupsToProcess;
 
 	/************************************************************************
  * recogn processing.
@@ -529,7 +560,7 @@ private:
 	void cleanUnusedKeys() ;
 	void slowIndicateConfigChanged() ;
 
-	virtual void timeStampKey(const RsGxsId& id, const RsIdentityUsage& reason) ;
+	virtual void timeStampKey(const RsGxsId& id, const RsIdentityUsage& reason) override;
 	rstime_t locked_getLastUsageTS(const RsGxsId& gxs_id);
 
 	std::string genRandomId(int len = 20);
@@ -587,6 +618,10 @@ private:
 
 	// keep a list of regular contacts. This is useful to sort IDs, and allow some services to priviledged ids only.
 	std::set<RsGxsId> mContacts;
+
+    // temporary list of identities which authentication failed, so that we do not ask for them again in the same session.
+    std::set<RsGxsId> mRejectedIdentities;
+
 	RsNetworkExchangeService* mNes;
 
 	/**************************
@@ -595,16 +630,15 @@ private:
 	 */
 
 	PgpAuxUtils *mPgpUtils;
-
+    rstime_t mLastPGPHashProcessTime ;
 	rstime_t mLastKeyCleaningTime ;
 	rstime_t mLastConfigUpdate ;
 
-	bool mOwnIdsLoaded ;
+	bool mOwnIdsLoaded;
+	bool ownIdsAreLoaded() { RS_STACK_MUTEX(mIdMtx); return mOwnIdsLoaded; }
+
 	bool mAutoAddFriendsIdentitiesAsContacts;
-    uint32_t mMaxKeepKeysBanned ;
+	uint32_t mMaxKeepKeysBanned;
+
+	RS_SET_CONTEXT_DEBUG_LEVEL(2)
 };
-
-#endif // P3_IDENTITY_SERVICE_HEADER
-
-
-
