@@ -29,7 +29,7 @@
 #include "rsitems/rsgxscommentitems.h"
 
 #include "rsserver/p3face.h"
-#include "retroshare/rsnotify.h"
+#include "retroshare/rsposted.h"
 
 // For Dummy Msgs.
 #include "util/rsrandom.h"
@@ -47,13 +47,18 @@
 #define POSTBASE_UNPROCESSED_MSGS	0x0012
 #define POSTBASE_ALL_MSGS 		0x0013
 #define POSTBASE_BG_POST_META		0x0014
+
+#define POSTED_UNUSED_BY_FRIENDS_DELAY (2*30*86400)  // delete unused posted groups after 2 months
+
 /********************************************************************************/
 /******************* Startup / Tick    ******************************************/
 /********************************************************************************/
 
 p3PostBase::p3PostBase(RsGeneralDataService *gds, RsNetworkExchangeService *nes, RsGixs* gixs,
 	RsSerialType* serviceSerialiser, uint16_t serviceType)
-    : RsGenExchange(gds, nes, serviceSerialiser, serviceType, gixs, postBaseAuthenPolicy()), GxsTokenQueue(this), RsTickEvent(), mPostBaseMtx("PostBaseMtx")
+    : RsGenExchange(gds, nes, serviceSerialiser, serviceType, gixs, postBaseAuthenPolicy()), GxsTokenQueue(this), RsTickEvent(),
+      mPostBaseMtx("PostBaseMutex"),
+      mKnownPostedMutex("PostBaseKnownPostedMutex")
 {
 	mBgProcessing = false;
 
@@ -87,77 +92,173 @@ void p3PostBase::notifyChanges(std::vector<RsGxsNotify *> &changes)
 	std::cerr << std::endl;
 #endif
 
-	p3Notify *notify = NULL;
-	if (!changes.empty())
-	{
-		notify = RsServer::notify();
-	}
+	for(auto it = changes.begin(); it != changes.end(); ++it)
+    {
+        RsGxsMsgChange *msgChange = dynamic_cast<RsGxsMsgChange *>(*it);
 
-	std::vector<RsGxsNotify *>::iterator it;
+        if(msgChange)
+        {
+            // To start with we are just going to trigger updates on these groups.
+            // FUTURE OPTIMISATION.
+            // It could be taken a step further and directly request these msgs for an update.
+            addGroupForProcessing(msgChange->mGroupId);
 
-	for(it = changes.begin(); it != changes.end(); ++it)
-	{
-		RsGxsGroupChange *groupChange = dynamic_cast<RsGxsGroupChange *>(*it);
-		RsGxsMsgChange *msgChange = dynamic_cast<RsGxsMsgChange *>(*it);
-		if (msgChange)
-		{
+            if (rsEvents)
+            {
+                switch(msgChange->getType())
+                {
+                case RsGxsNotify::TYPE_RECEIVED_NEW:
+                case RsGxsNotify::TYPE_PUBLISHED:
+                {
+                    auto ev = std::make_shared<RsGxsPostedEvent>();
+                    ev->mPostedMsgId    = msgChange->mMsgId;
+                    ev->mPostedThreadId = msgChange->mNewMsgItem->meta.mThreadId;
+                    ev->mPostedGroupId  = msgChange->mGroupId;
+
+                    if(nullptr != dynamic_cast<RsGxsCommentItem*>(msgChange->mNewMsgItem))
+                        ev->mPostedEventCode = RsPostedEventCode::NEW_COMMENT;
+                    else
+                        if(nullptr != dynamic_cast<RsGxsVoteItem*>(msgChange->mNewMsgItem))
+                            ev->mPostedEventCode = RsPostedEventCode::NEW_VOTE;
+                        else
+                            ev->mPostedEventCode = RsPostedEventCode::NEW_MESSAGE;
+
+                    rsEvents->postEvent(ev);
 #ifdef POSTBASE_DEBUG
-			std::cerr << "p3PostBase::notifyChanges() Found Message Change Notification";
-			std::cerr << std::endl;
+                    std::cerr << "p3PostBase::notifyChanges() Found Message Change Notification: NEW/PUBLISHED ID=" << msgChange->mMsgId << " in group " << msgChange->mGroupId << ", thread ID = " << msgChange->mNewMsgItem->meta.mThreadId << std::endl;
 #endif
 
-			std::map<RsGxsGroupId, std::set<RsGxsMessageId> > &msgChangeMap = msgChange->msgChangeMap;
-			for(auto mit = msgChangeMap.begin(); mit != msgChangeMap.end(); ++mit)
-			{
+                }
+                    break;
+
+
+                case RsGxsNotify::TYPE_PROCESSED:
+                {
+                    auto ev = std::make_shared<RsGxsPostedEvent>();
+                    ev->mPostedMsgId = msgChange->mMsgId;
+                    ev->mPostedGroupId = msgChange->mGroupId;
+                    ev->mPostedEventCode = RsPostedEventCode::MESSAGE_VOTES_UPDATED;
+                    rsEvents->postEvent(ev);
 #ifdef POSTBASE_DEBUG
-				std::cerr << "p3PostBase::notifyChanges() Msgs for Group: " << mit->first;
-				std::cerr << std::endl;
+                    std::cerr << "p3PostBase::notifyChanges() Found Message Change Notification: PROCESSED ID=" << msgChange->mMsgId << " in group " << msgChange->mGroupId << std::endl;
 #endif
-				// To start with we are just going to trigger updates on these groups.
-				// FUTURE OPTIMISATION.
-				// It could be taken a step further and directly request these msgs for an update.
-				addGroupForProcessing(mit->first);
-
-				if (notify && msgChange->getType() == RsGxsNotify::TYPE_RECEIVED_NEW)
-				{
-					for (auto mit1 = mit->second.begin(); mit1 != mit->second.end(); ++mit1)
-					{
-						notify->AddFeedItem(RS_FEED_ITEM_POSTED_MSG, mit->first.toStdString(), mit1->toStdString());
-					}
-				}
-			}
-		}
-
-		/* pass on Group Changes to GUI */
-		if (groupChange)
-		{
+                }
+                    break;
+                default:
 #ifdef POSTBASE_DEBUG
-			std::cerr << "p3PostBase::notifyChanges() Found Group Change Notification";
-			std::cerr << std::endl;
+                    std::cerr << "p3PostBase::notifyChanges() Found Message Change Notification: type " << msgChange->getType() << " (ignored) " << msgChange->mMsgId << std::endl;
 #endif
+                    break;
+                }
+            }
+        }
 
-			std::list<RsGxsGroupId> &groupList = groupChange->mGrpIdList;
-			std::list<RsGxsGroupId>::iterator git;
-			for(git = groupList.begin(); git != groupList.end(); ++git)
-			{
+        RsGxsGroupChange *grpChange = dynamic_cast<RsGxsGroupChange *>(*it);
+
+        /* pass on Group Changes to GUI */
+        if (grpChange && rsEvents)
+        {
 #ifdef POSTBASE_DEBUG
-				std::cerr << "p3PostBase::notifyChanges() Incoming Group: " << *git;
-				std::cerr << std::endl;
+            std::cerr << "p3PostBase::notifyChanges() Found Group Change Notification";
+            std::cerr << std::endl;
 #endif
+            const RsGxsGroupId& group_id(grpChange->mGroupId);
 
-				if (notify && groupChange->getType() == RsGxsNotify::TYPE_RECEIVED_NEW)
-				{
-					notify->AddFeedItem(RS_FEED_ITEM_POSTED_NEW, git->toStdString());
-				}
-			}
-		}
-	}
-	receiveHelperChanges(changes);
+            switch(grpChange->getType())
+            {
+            case RsGxsNotify::TYPE_PROCESSED:	// happens when the group is subscribed
+            {
+                auto ev = std::make_shared<RsGxsPostedEvent>();
+                ev->mPostedGroupId = group_id;
+                ev->mPostedEventCode = RsPostedEventCode::SUBSCRIBE_STATUS_CHANGED;
+                rsEvents->postEvent(ev);
+            }
+                break;
+
+            case RsGxsNotify::TYPE_GROUP_SYNC_PARAMETERS_UPDATED:
+            {
+                auto ev = std::make_shared<RsGxsPostedEvent>();
+                ev->mPostedGroupId = group_id;
+                ev->mPostedEventCode = RsPostedEventCode::SYNC_PARAMETERS_UPDATED;
+                rsEvents->postEvent(ev);
+            }
+                break;
+
+          case RsGxsNotify::TYPE_GROUP_DELETED:
+           {
+               auto ev = std::make_shared<RsGxsPostedEvent>();
+               ev->mPostedGroupId = group_id;
+               ev->mPostedEventCode = RsPostedEventCode::BOARD_DELETED;
+
+               rsEvents->postEvent(ev);
+           }
+               break;
+
+            case RsGxsNotify::TYPE_STATISTICS_CHANGED:
+            {
+                auto ev = std::make_shared<RsGxsPostedEvent>();
+                ev->mPostedGroupId = group_id;
+                ev->mPostedEventCode = RsPostedEventCode::STATISTICS_CHANGED;
+                rsEvents->postEvent(ev);
+
+                RS_STACK_MUTEX(mKnownPostedMutex);
+                mKnownPosted[group_id] = time(nullptr);
+                IndicateConfigChanged();
+            }
+                break;
+
+            case RsGxsNotify::TYPE_UPDATED:
+            {
+                // Happens when the group data has changed. In this case we need to analyse the old and new group in order to detect possible notifications for clients
+
+                auto ev = std::make_shared<RsGxsPostedEvent>();
+                ev->mPostedGroupId = grpChange->mGroupId;
+                ev->mPostedEventCode = RsPostedEventCode::UPDATED_POSTED_GROUP;
+                rsEvents->postEvent(ev);
+            }
+                break;
+
+
+            case RsGxsNotify::TYPE_PUBLISHED:
+            case RsGxsNotify::TYPE_RECEIVED_NEW:
+            {
+                /* group received */
+
+                bool unknown;
+                {
+                    RS_STACK_MUTEX(mKnownPostedMutex);
+
+                    unknown = (mKnownPosted.find(grpChange->mGroupId) == mKnownPosted.end());
+                    mKnownPosted[group_id] = time(nullptr);
+                    IndicateConfigChanged();
+                }
+                if(unknown)
+                {
+                    auto ev = std::make_shared<RsGxsPostedEvent>();
+                    ev->mPostedGroupId = group_id;
+                    ev->mPostedEventCode = RsPostedEventCode::NEW_POSTED_GROUP;
+                    rsEvents->postEvent(ev);
 
 #ifdef POSTBASE_DEBUG
-	std::cerr << "p3PostBase::notifyChanges() -> receiveChanges()";
-	std::cerr << std::endl;
+                    std::cerr << "p3PostBase::notifyChanges() Incoming Group: " << group_id;
+                    std::cerr << std::endl;
 #endif
+                }
+                else
+                    RsInfo() << __PRETTY_FUNCTION__
+                             << " Not notifying already known forum "
+                             << group_id << std::endl;
+            }
+                break;
+
+            default:
+                RsErr() << " Got a GXS event of type " << grpChange->getType() << " Currently not handled." << std::endl;
+                break;
+            }
+        }
+
+        delete *it;
+    }
 }
 
 void	p3PostBase::service_tick()
@@ -184,6 +285,15 @@ void p3PostBase::setMessageReadStatus(uint32_t& token, const RsGxsGrpMsgIdPair&
 
 	setMsgStatusFlags(token, msgId, status, mask);
 
+	if (rsEvents)
+	{
+		auto ev = std::make_shared<RsGxsPostedEvent>();
+
+		ev->mPostedMsgId = msgId.second;
+		ev->mPostedGroupId = msgId.first;
+		ev->mPostedEventCode = RsPostedEventCode::READ_STATUS_CHANGED;
+		rsEvents->postEvent(ev);
+	}
 }
 
 
@@ -297,11 +407,7 @@ void p3PostBase::addGroupForProcessing(RsGxsGroupId grpId)
 	{
 		RsStackMutex stack(mPostBaseMtx); /********** STACK LOCKED MTX ******/
 		// no point having multiple lookups queued.
-		if (mBgGroupList.end() == std::find(mBgGroupList.begin(), 
-						mBgGroupList.end(), grpId))
-		{
-			mBgGroupList.push_back(grpId);
-		}
+		mBgGroupList.insert(grpId);
 	}
 }
 
@@ -334,8 +440,8 @@ void p3PostBase::background_requestUnprocessedGroup()
 			return;
 		}
 
-		grpId = mBgGroupList.front();
-		mBgGroupList.pop_front();
+		grpId = *mBgGroupList.begin();
+		mBgGroupList.erase(grpId);
 		mBgProcessing = true;
 	}
 
@@ -429,8 +535,6 @@ void p3PostBase::background_loadMsgs(const uint32_t &token, bool unprocessed)
 
 	// generate vector of changes to push to the GUI.
 	std::vector<RsGxsNotify *> changes;
-	RsGxsMsgChange *msgChanges = new RsGxsMsgChange(RsGxsNotify::TYPE_PROCESSED, false);
-
 
 	RsGxsGroupId groupId;
 	std::map<RsGxsGroupId, std::vector<RsGxsMsgItem*> >::iterator mit;
@@ -481,7 +585,7 @@ void p3PostBase::background_loadMsgs(const uint32_t &token, bool unprocessed)
 #endif
 
 				/* but we need to notify GUI about them */	
-				msgChanges->msgChangeMap[mit->first].insert((*vit)->meta.mMsgId);
+				changes.push_back(new RsGxsMsgChange(RsGxsNotify::TYPE_PROCESSED, mit->first,(*vit)->meta.mMsgId, false));
 			}
 			else if (NULL != (commentItem = dynamic_cast<RsGxsCommentItem *>(*vit)))
 			{
@@ -525,8 +629,7 @@ void p3PostBase::background_loadMsgs(const uint32_t &token, bool unprocessed)
 			else
 			{
 				/* unknown! */
-				std::cerr << "p3PostBase::background_processNewMessages() ERROR Strange NEW Message:";
-				std::cerr << std::endl;
+                std::cerr << "p3PostBase::background_processNewMessages() ERROR Strange NEW Message:" << std::endl;
 				std::cerr << "\t" << (*vit)->meta;
 				std::cerr << std::endl;
 	
@@ -577,20 +680,7 @@ void p3PostBase::background_loadMsgs(const uint32_t &token, bool unprocessed)
 	}
 
 	/* push updates of new Posts */
-	if (msgChanges->msgChangeMap.size() > 0)
-	{
-#ifdef POSTBASE_DEBUG
-        std::cerr << "p3PostBase::background_processNewMessages() -> receiveChanges()";
-        std::cerr << std::endl;
-#endif
-
-		changes.push_back(msgChanges);
-	 	receiveHelperChanges(changes);
-	}
-	else
-	{
-		delete(msgChanges);
-	}
+	notifyChanges(changes);
 
 	/* request the summary info from the parents */
 	uint32_t token_b;
@@ -655,7 +745,6 @@ void p3PostBase::background_updateVoteCounts(const uint32_t &token)
 
 	// generate vector of changes to push to the GUI.
 	std::vector<RsGxsNotify *> changes;
-	RsGxsMsgChange *msgChanges = new RsGxsMsgChange(RsGxsNotify::TYPE_PROCESSED, false);
 
 	for(mit = parentMsgList.begin(); mit != parentMsgList.end(); ++mit)
 	{
@@ -698,7 +787,8 @@ void p3PostBase::background_updateVoteCounts(const uint32_t &token)
 #endif
 
 				stats.increment(it->second);
-				msgChanges->msgChangeMap[mit->first].insert(vit->mMsgId);
+
+	            changes.push_back(new RsGxsMsgChange(RsGxsNotify::TYPE_PROCESSED,mit->first,vit->mMsgId, false));
 			}
 			else
 			{
@@ -730,20 +820,7 @@ void p3PostBase::background_updateVoteCounts(const uint32_t &token)
 		}
 	}
 
-	if (msgChanges->msgChangeMap.size() > 0)
-	{
-#ifdef POSTBASE_DEBUG
-        std::cerr << "p3PostBase::background_updateVoteCounts() -> receiveChanges()";
-        std::cerr << std::endl;
-#endif
-
-		changes.push_back(msgChanges);
-	 	receiveHelperChanges(changes);
-	}
-	else
-	{
-		delete(msgChanges);
-	}
+	notifyChanges(changes);
 
 	// DONE!.
 	background_cleanup();
@@ -770,12 +847,14 @@ bool p3PostBase::background_cleanup()
 
 
 	// Overloaded from GxsTokenQueue for Request callbacks.
-void p3PostBase::handleResponse(uint32_t token, uint32_t req_type)
+void p3PostBase::handleResponse(uint32_t token, uint32_t req_type
+                                , RsTokenService::GxsRequestStatus status)
 {
 #ifdef POSTBASE_DEBUG
-    std::cerr << "p3PostBase::handleResponse(" << token << "," << req_type << ")";
-    std::cerr << std::endl;
+	std::cerr << "p3PostBase::handleResponse(" << token << "," << req_type << "," << status << ")" << std::endl;
 #endif
+	if (status != RsTokenService::COMPLETE)
+		return; //For now, only manage Complete request
 
 	// stuff.
 	switch(req_type)
@@ -800,3 +879,153 @@ void p3PostBase::handleResponse(uint32_t token, uint32_t req_type)
 	}
 }
 
+static const uint32_t GXS_POSTED_CONFIG_MAX_TIME_NOTIFY_STORAGE = 86400*30*2 ; // ignore notifications for 2 months
+static const uint8_t  GXS_POSTED_CONFIG_SUBTYPE_NOTIFY_RECORD   = 0x01 ;
+
+struct RsGxsPostedNotifyRecordsItem: public RsItem
+{
+
+	RsGxsPostedNotifyRecordsItem()
+	    : RsItem(RS_PKT_VERSION_SERVICE,RS_SERVICE_GXS_TYPE_POSTED_CONFIG,GXS_POSTED_CONFIG_SUBTYPE_NOTIFY_RECORD)
+	{}
+
+    virtual ~RsGxsPostedNotifyRecordsItem() {}
+
+	void serial_process( RsGenericSerializer::SerializeJob j,
+	                     RsGenericSerializer::SerializeContext& ctx )
+	{ RS_SERIAL_PROCESS(records); }
+
+	void clear() {}
+
+	std::map<RsGxsGroupId,rstime_t> records;
+};
+
+class GxsPostedConfigSerializer : public RsServiceSerializer
+{
+public:
+	GxsPostedConfigSerializer() : RsServiceSerializer(RS_SERVICE_GXS_TYPE_POSTED_CONFIG) {}
+	virtual ~GxsPostedConfigSerializer() {}
+
+	RsItem* create_item(uint16_t service_id, uint8_t item_sub_id) const
+	{
+		if(service_id != RS_SERVICE_GXS_TYPE_POSTED_CONFIG)
+			return NULL;
+
+		switch(item_sub_id)
+		{
+		case GXS_POSTED_CONFIG_SUBTYPE_NOTIFY_RECORD: return new RsGxsPostedNotifyRecordsItem();
+		default:
+			return NULL;
+		}
+	}
+};
+
+bool p3PostBase::service_checkIfGroupIsStillUsed(const RsGxsGrpMetaData& meta)
+{
+#ifdef POSTBASE_DEBUG
+    std::cerr << "p3gxsChannels: Checking unused board: called by GxsCleaning." << std::endl;
+#endif
+
+    // request all group infos at once
+
+    rstime_t now = time(nullptr);
+
+    RS_STACK_MUTEX(mKnownPostedMutex);
+
+    auto it = mKnownPosted.find(meta.mGroupId);
+    bool unknown_posted = (it == mKnownPosted.end());
+
+#ifdef POSTBASE_DEBUG
+    std::cerr << "  Board " << meta.mGroupId ;
+#endif
+
+    if(unknown_posted)
+    {
+        // This case should normally not happen. It does because this board was never registered since it may
+        // arrived before this code was here
+
+#ifdef POSTBASE_DEBUG
+        std::cerr << ". Not known yet. Adding current time as new TS." << std::endl;
+#endif
+        mKnownPosted[meta.mGroupId] = now;
+        IndicateConfigChanged();
+
+        return true;
+    }
+    else
+    {
+        bool used_by_friends = (now < it->second + POSTED_UNUSED_BY_FRIENDS_DELAY);
+        bool subscribed = static_cast<bool>(meta.mSubscribeFlags & GXS_SERV::GROUP_SUBSCRIBE_SUBSCRIBED);
+
+#ifdef POSTBASE_DEBUG
+        std::cerr << ". subscribed: " << subscribed << ", used_by_friends: " << used_by_friends << " last TS: " << now - it->second << " secs ago (" << (now-it->second)/86400 << " days)";
+#endif
+        if(!subscribed && !used_by_friends)
+        {
+#ifdef POSTBASE_DEBUG
+            std::cerr << ". Scheduling for deletion" << std::endl;
+#endif
+            return false;
+        }
+        else
+        {
+#ifdef POSTBASE_DEBUG
+            std::cerr << ". Keeping!" << std::endl;
+#endif
+            return true;
+        }
+    }
+}
+
+bool p3PostBase::saveList(bool& cleanup, std::list<RsItem *>&saveList)
+{
+	cleanup = true ;
+
+	RsGxsPostedNotifyRecordsItem *item = new RsGxsPostedNotifyRecordsItem ;
+
+    {
+        RS_STACK_MUTEX(mKnownPostedMutex);
+        item->records = mKnownPosted ;
+    }
+
+	saveList.push_back(item) ;
+	return true;
+}
+
+bool p3PostBase::loadList(std::list<RsItem *>& loadList)
+{
+	while(!loadList.empty())
+	{
+		RsItem *item = loadList.front();
+		loadList.pop_front();
+
+		rstime_t now = time(NULL);
+
+		RsGxsPostedNotifyRecordsItem *fnr = dynamic_cast<RsGxsPostedNotifyRecordsItem*>(item) ;
+
+		if(fnr != NULL)
+		{
+            RS_STACK_MUTEX(mKnownPostedMutex);
+
+			mKnownPosted.clear();
+
+			for(auto it(fnr->records.begin());it!=fnr->records.end();++it)
+				if( now < it->second + GXS_POSTED_CONFIG_MAX_TIME_NOTIFY_STORAGE)
+					mKnownPosted.insert(*it) ;
+		}
+
+		delete item ;
+	}
+	return true;
+}
+
+RsSerialiser* p3PostBase::setupSerialiser()
+{
+	RsSerialiser* rss = new RsSerialiser;
+	rss->addSerialType(new GxsPostedConfigSerializer());
+
+	return rss;
+}
+
+
+
