@@ -20,17 +20,21 @@
 
 #include <QBuffer>
 #include <QMessageBox>
+#include <QToolTip>
 
 #include "IdEditDialog.h"
 #include "ui_IdEditDialog.h"
 #include "gui/common/UIStateHelper.h"
 #include "gui/common/AvatarDialog.h"
 #include "gui/gxs/GxsIdDetails.h"
-#include "util/TokenQueue.h"
+#include "util/qtthreadsutils.h"
 #include "util/misc.h"
+#include "gui/notifyqt.h"
 
-#include <retroshare/rsidentity.h>
-#include <retroshare/rspeers.h>
+#include "retroshare/rsidentity.h"
+#include "retroshare/rspeers.h"
+#include "gui/common/FilesDefs.h"
+#include "util/imageutil.h"
 
 #include <iostream>
 
@@ -43,10 +47,11 @@ IdEditDialog::IdEditDialog(QWidget *parent) :
     ui(new(Ui::IdEditDialog))
 {
 	mIsNew = true;
+    mAvatarIsSet = false;
 
 	ui->setupUi(this);
 
-	ui->headerFrame->setHeaderImage(QPixmap(":/icons/png/person.png"));
+    ui->headerFrame->setHeaderImage(FilesDefs::getPixmapFromQtResourcePath(":/icons/png/person.png"));
 	ui->headerFrame->setHeaderText(tr("Create New Identity"));
 
 	/* Setup UI helper */
@@ -73,8 +78,8 @@ IdEditDialog::IdEditDialog(QWidget *parent) :
 	/* Connect signals */
 	connect(ui->radioButton_GpgId, SIGNAL(toggled(bool)), this, SLOT(idTypeToggled(bool)));
 	connect(ui->radioButton_Pseudo, SIGNAL(toggled(bool)), this, SLOT(idTypeToggled(bool)));
-	connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(submit()));
-	connect(ui->buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+	connect(ui->postButton, SIGNAL(clicked()), this, SLOT(submit()));
+	connect(ui->cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
 
 	connect(ui->plainTextEdit_Tag, SIGNAL(textChanged()), this, SLOT(checkNewTag()));
 	connect(ui->pushButton_Tag, SIGNAL(clicked(bool)), this, SLOT(addRecognTag()));
@@ -84,24 +89,30 @@ IdEditDialog::IdEditDialog(QWidget *parent) :
 	connect(ui->toolButton_Tag4, SIGNAL(clicked(bool)), this, SLOT(rmTag4()));
 	connect(ui->toolButton_Tag5, SIGNAL(clicked(bool)), this, SLOT(rmTag5()));
 	connect(ui->avatarButton, SIGNAL(clicked(bool)), this, SLOT(changeAvatar()));
+	connect(ui->removeButton, SIGNAL(clicked(bool)), this, SLOT(removeAvatar()));
+
+    connect(ui->avatarLabel,SIGNAL(cleared()),this,SLOT(avatarCleared()));
+
+    ui->avatarLabel->setEnableClear(true);
+    ui->avatarLabel->setToolTip(tr("No Avatar chosen. A default image will be automatically displayed from your new identity."));
 
 	/* Initialize ui */
 	ui->lineEdit_Nickname->setMaxLength(RSID_MAXIMUM_NICKNAME_SIZE);
 
-	mIdQueue = new TokenQueue(rsIdentity->getTokenService(), this);
 	ui->pushButton_Tag->setEnabled(false);
 	ui->pushButton_Tag->hide(); // unfinished
 	ui->plainTextEdit_Tag->hide();
 	ui->label_TagCheck->hide();
+	ui->frame_Tags->setHidden(true);
+
+	updateInterface();
 }
 
-IdEditDialog::~IdEditDialog()
-{
-	delete(mIdQueue);
-}
+IdEditDialog::~IdEditDialog() {}
 
 void IdEditDialog::changeAvatar()
 {
+#ifdef TODO
 	AvatarDialog dialog(this);
 
 	dialog.setAvatar(mAvatar);
@@ -111,6 +122,26 @@ void IdEditDialog::changeAvatar()
 
 		setAvatar(newAvatar);
 	}
+#endif
+    // For now we use a simpler method since AvatarDialog is not finished yet; we use the thumbnail viewer to allow the user to
+    // select a proper scale/crop of a given image to make his/her avatar.
+
+    QString image_filename ;
+
+    if(!misc::getOpenFileName(this,RshareSettings::LASTDIR_IMAGES,tr("Import image"), tr("Image files (*.jpg *.png);;All files (*)"),image_filename))
+        return;
+
+    QImage img(image_filename);
+
+    ui->avatarLabel->setPicture(QPixmap::fromImage(img));
+    ui->avatarLabel->setEnableZoom(true);
+        ui->avatarLabel->setToolTip(tr("Use the mouse to zoom and adjust the image for your avatar. Hit Del to remove it."));
+    mAvatarIsSet = true;
+
+    // shows the tooltip for a while
+    QToolTip::showText( ui->avatarLabel->mapToGlobal( QPoint( 0, 0 ) ), ui->avatarLabel->toolTip() );
+
+	updateInterface();
 }
 
 void IdEditDialog::setupNewId(bool pseudo,bool enable_anon)
@@ -187,37 +218,77 @@ void IdEditDialog::updateIdType(bool pseudo)
 	}
 }
 
+void IdEditDialog::avatarCleared()
+{
+    setAvatar(QPixmap());
+}
+
 void IdEditDialog::setAvatar(const QPixmap &avatar)
 {
 	mAvatar = avatar;
 
 	if (!mAvatar.isNull()) {
-		ui->avatarLabel->setPixmap(mAvatar);
-	} else {
+        ui->avatarLabel->setPicture(avatar);
+        mAvatarIsSet = true;
+        ui->avatarLabel->setToolTip(tr("Use the mouse to zoom and adjust the image for your avatar. Hit Del to remove it."));
+    } else {
 		// we need to use the default pixmap here, generated from the ID
-		ui->avatarLabel->setPixmap(QPixmap::fromImage(GxsIdDetails::makeDefaultIcon(RsGxsId(mEditGroup.mMeta.mGroupId))));
-	}
+        ui->avatarLabel->setText(tr("No avatar chosen"));	// This clears up the image
+        mAvatarIsSet = false;
+        ui->avatarLabel->setToolTip(tr("No Avatar chosen. A default image will be automatically displayed from your new identity."));
+    }
 }
 
-void IdEditDialog::setupExistingId(const RsGxsGroupId &keyId)
+void IdEditDialog::setupExistingId(const RsGxsGroupId& keyId)
 {
 	setWindowTitle(tr("Edit identity"));
-	ui->headerFrame->setHeaderImage(QPixmap(":/icons/png/person.png"));
+    ui->headerFrame->setHeaderImage(FilesDefs::getPixmapFromQtResourcePath(":/icons/png/person.png"));
 	ui->headerFrame->setHeaderText(tr("Edit identity"));
+	ui->postButton->setText(tr("Update"));
+
+	mStateHelper->setLoading(IDEDITDIALOG_LOADID, true);
 
 	mIsNew = false;
 	mGroupId.clear();
 
-	mStateHelper->setLoading(IDEDITDIALOG_LOADID, true);
+	RsThread::async([this,keyId]()
+	{
+		std::vector<RsGxsIdGroup> datavector;
+		bool res = rsIdentity->getIdentitiesInfo(
+		            std::set<RsGxsId>({(RsGxsId)keyId}), datavector );
 
-	RsTokReqOptions opts;
-	opts.mReqType = GXS_REQUEST_TYPE_GROUP_DATA;
+		RsQThreadUtils::postToObject( [this,keyId,res,datavector]()
+		{
+			/* Here it goes any code you want to be executed on the Qt Gui
+			 * thread, for example to update the data model with new information
+			 * after a blocking call to RetroShare API complete, note that
+			 * Qt::QueuedConnection is important!
+			 */
 
-	std::list<RsGxsGroupId> groupIds;
-	groupIds.push_back(keyId);
+			mStateHelper->setLoading(IDEDITDIALOG_LOADID, false);
 
-	uint32_t token;
-    mIdQueue->requestGroupInfo(token, RS_TOKREQ_ANSTYPE_DATA, opts, groupIds, IDEDITDIALOG_LOADID);
+			/* get details from libretroshare */
+
+			if (!res || datavector.size() != 1)
+			{
+				std::cerr << __PRETTY_FUNCTION__ << " failed to collect group info for identity " << keyId << std::endl;
+
+				ui->lineEdit_KeyId->setText(tr("Error KeyID invalid"));
+				ui->lineEdit_Nickname->setText("");
+
+				ui->lineEdit_GpgHash->setText(tr("N/A"));
+				ui->lineEdit_GpgId->setText(tr("N/A"));
+				ui->lineEdit_GpgName->setText(tr("N/A"));
+				return;
+			}
+
+            loadExistingId(datavector[0]);
+
+		}, this );
+	});
+
+    // force resize of dialog, to hide empty space from the hidden recogn tags area
+    adjustSize();
 }
 
 void IdEditDialog::enforceNoAnonIds()
@@ -227,39 +298,14 @@ void IdEditDialog::enforceNoAnonIds()
     ui->radioButton_Pseudo->setEnabled(false);
 }
 
-void IdEditDialog::loadExistingId(uint32_t token)
+void IdEditDialog::loadExistingId(const RsGxsIdGroup& id_group)
 {
-	mStateHelper->setLoading(IDEDITDIALOG_LOADID, false);
-
-	/* get details from libretroshare */
-	std::vector<RsGxsIdGroup> datavector;
-	if (!rsIdentity->getGroupData(token, datavector))
-	{
-		ui->lineEdit_KeyId->setText(tr("Error getting key!"));
-		return;
-	}
-
-	if (datavector.size() != 1)
-	{
-		std::cerr << "IdDialog::insertIdDetails() Invalid datavector size";
-		std::cerr << std::endl;
-
-		ui->lineEdit_KeyId->setText(tr("Error KeyID invalid"));
-		ui->lineEdit_Nickname->setText("");
-
-		ui->lineEdit_GpgHash->setText(tr("N/A"));
-		ui->lineEdit_GpgId->setText(tr("N/A"));
-		ui->lineEdit_GpgName->setText(tr("N/A"));
-		return;
-	}
-
-	mEditGroup = datavector[0];
+	mEditGroup = id_group;
 	mGroupId = mEditGroup.mMeta.mGroupId;
 
 	QPixmap avatar;
-	if (mEditGroup.mImage.mSize > 0) {
-		avatar.loadFromData(mEditGroup.mImage.mData, mEditGroup.mImage.mSize, "PNG");
-	}
+	if (mEditGroup.mImage.mSize > 0)
+		GxsIdDetails::loadPixmapFromData(mEditGroup.mImage.mData, mEditGroup.mImage.mSize, avatar,GxsIdDetails::LARGE);
 
 	setAvatar(avatar);
 
@@ -310,9 +356,10 @@ void IdEditDialog::loadExistingId(uint32_t token)
 	}
 
 	// RecognTags.
-	ui->frame_Tags->setHidden(false);
+	ui->frame_Tags->setHidden(true);
 
 	loadRecognTags();
+	updateInterface();
 }
 
 #define MAX_RECOGN_TAGS 	5
@@ -493,13 +540,9 @@ void IdEditDialog::loadRecognTags()
 void IdEditDialog::submit()
 {
 	if (mIsNew)
-	{
 		createId();
-	}
 	else
-	{
 		updateId();
-	}
 }
 
 void IdEditDialog::createId()
@@ -527,38 +570,58 @@ void IdEditDialog::createId()
     params.nickname = groupname.toUtf8().constData();
 	params.isPgpLinked = (ui->radioButton_GpgId->isChecked());
 
-	if (!mAvatar.isNull())
-	{
-		QByteArray ba;
-		QBuffer buffer(&ba);
+    if(mAvatarIsSet)
+    {
+        mAvatar = ui->avatarLabel->extractCroppedScaledPicture();
 
-		buffer.open(QIODevice::WriteOnly);
-		mAvatar.save(&buffer, "PNG"); // writes image into ba in PNG format
+        QByteArray ba;
+        QBuffer buffer(&ba);
 
-		params.mImage.copy((uint8_t *) ba.data(), ba.size());
-	}
-	else
-		params.mImage.clear();
+        bool has_transparency = ImageUtil::hasAlphaContent(mAvatar.toImage());
 
-	uint32_t token = 0;
-	rsIdentity->createIdentity(token, params);
+        buffer.open(QIODevice::WriteOnly);
+        mAvatar.save(&buffer, has_transparency?"PNG":"JPG"); // writes image into ba in PNG format
 
-	ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+        params.mImage.copy((uint8_t *) ba.data(), ba.size());
+    }
+    else
+        params.mImage.clear();
 
-	mIdQueue->queueRequest(token, 0, 0, IDEDITDIALOG_CREATEID);
-}
+    RsGxsId keyId;
+    std::string gpg_password;
 
-void IdEditDialog::idCreated(uint32_t token)
-{
-	if (!rsIdentity->acknowledgeGrp(token, mGroupId)) {
-		std::cerr << "IdDialog::idCreated() acknowledgeGrp failed";
-		std::cerr << std::endl;
+    if(params.isPgpLinked)
+    {
+		std::string gpg_name = rsPeers->getGPGName(rsPeers->getGPGOwnId());
+        bool cancelled;
 
-		reject();
-		return;
-	}
+        rsNotify->clearPgpPassphrase(); // just in case
 
-	accept();
+        if(!NotifyQt::getInstance()->askForPassword(tr("Profile password needed.").toStdString(),
+		                                            gpg_name + " (" + rsPeers->getOwnId().toStdString() + ")",
+		                                            false,
+		                                            gpg_password,cancelled))
+		{
+			QMessageBox::critical(NULL,tr("Identity creation failed"),tr("Cannot create an identity linked to your profile without your profile password."));
+			return;
+        }
+
+    }
+
+    if(rsIdentity->createIdentity(keyId,params.nickname,params.mImage,!params.isPgpLinked,gpg_password))
+    {
+		ui->postButton->setEnabled(false);
+
+        if(!keyId.isNull())
+        {
+            QMessageBox::information(NULL,tr("Identity creation success"),tr("Your new identity was successfuly created, its ID is %1.").arg(QString::fromStdString(keyId.toStdString())));
+            close();
+        }
+        else
+			QMessageBox::critical(NULL,tr("Identity creation failed"),tr("Cannot create identity. Something went wrong."));
+    }
+    else
+        QMessageBox::critical(NULL,tr("Identity creation failed"),tr("Cannot create identity. Something went wrong. Check your profile password."));
 }
 
 void IdEditDialog::updateId()
@@ -581,38 +644,68 @@ void IdEditDialog::updateId()
 
     mEditGroup.mMeta.mGroupName = groupname.toUtf8().constData();
 
+    mAvatar = ui->avatarLabel->extractCroppedScaledPicture();
+
 	if (!mAvatar.isNull())
 	{
 		QByteArray ba;
 		QBuffer buffer(&ba);
 
-		buffer.open(QIODevice::WriteOnly);
-		mAvatar.save(&buffer, "PNG"); // writes image into ba in PNG format
+        bool has_transparency = ImageUtil::hasAlphaContent(mAvatar.toImage());
+
+        buffer.open(QIODevice::WriteOnly);
+        mAvatar.save(&buffer, has_transparency?"PNG":"JPG"); // writes image into ba in PNG format
 
 		mEditGroup.mImage.copy((uint8_t *) ba.data(), ba.size());
 	}
 	else
 		mEditGroup.mImage.clear();
 
-	uint32_t dummyToken = 0;
-	rsIdentity->updateIdentity(dummyToken, mEditGroup);
+    RsGxsId keyId;
+    std::string gpg_password;
+
+    if(mEditGroup.mPgpLinked)
+    {
+        std::string gpg_name = rsPeers->getGPGName(rsPeers->getGPGOwnId());
+        bool cancelled;
+
+        rsNotify->clearPgpPassphrase(); // just in case
+
+        if(!NotifyQt::getInstance()->askForPassword(tr("Profile password needed.").toStdString(),
+                                                    gpg_name + " (" + rsPeers->getOwnId().toStdString() + ")",
+                                                    false,
+                                                    gpg_password,cancelled))
+        {
+            QMessageBox::critical(NULL,tr("Identity creation failed"),tr("Cannot create an identity linked to your profile without your profile password."));
+            return;
+        }
+    }
+
+    if(!rsIdentity->updateIdentity(RsGxsId(mEditGroup.mMeta.mGroupId),mEditGroup.mMeta.mGroupName,mEditGroup.mImage,mEditGroup.mPgpId.isNull(),gpg_password))
+    {
+        QMessageBox::critical(NULL,tr("Identity update failed"),tr("Cannot update identity. Something went wrong. Check your profile password."));
+        return;
+    }
 
 	accept();
 }
 
-void IdEditDialog::loadRequest(const TokenQueue */*queue*/, const TokenRequest &req)
+void IdEditDialog::removeAvatar()
 {
-	std::cerr << "IdDialog::loadRequest() UserType: " << req.mUserType;
-	std::cerr << std::endl;
-
-	switch (req.mUserType) {
-	case IDEDITDIALOG_LOADID:
-		loadExistingId(req.mToken);
-		break;
-
-	case IDEDITDIALOG_CREATEID:
-		idCreated(req.mToken);
-		break;
-	}
+	ui->avatarLabel->setPicture(QPixmap());
+	mEditGroup.mImage.clear();
 
+	updateInterface();
+}
+
+void IdEditDialog::updateInterface()
+{
+	const QPixmap *pixmap = ui->avatarLabel->pixmap();
+	if (pixmap && !pixmap->isNull()) {
+		ui->removeButton->setEnabled(true);
+	} else if (mEditGroup.mImage.mSize != NULL) {
+		ui->removeButton->setEnabled(true);
+	} else {
+		ui->removeButton->setEnabled(false);
+	}
 }
