@@ -21,6 +21,7 @@
 #include <QColorDialog>
 #include <QDesktopServices>
 #include <QIcon>
+#include <QInputDialog>
 #include <QMessageBox>
 #include <QPixmap>
 #include <QStatusBar>
@@ -31,16 +32,19 @@
 #include <retroshare/rsplugin.h>
 #include <retroshare/rsconfig.h>
 
+#ifdef MESSENGER_WINDOW
+#include "MessengerWindow.h"
+#endif
+
 #include "rshare.h"
 #include "MainWindow.h"
 #include "ui_MainWindow.h"
-#include "MessengerWindow.h"
 #include "HomePage.h"
 #include "NetworkDialog.h"
 #include "gui/FileTransfer/SearchDialog.h"
 #include "gui/FileTransfer/SharedFilesDialog.h"
 #include "gui/FileTransfer/TransfersDialog.h"
-#include "MessagesDialog.h"
+#include "gui/msgs/MessagesDialog.h"
 #include "PluginsPage.h"
 #include "NewsFeed.h"
 #include "ShareManager.h"
@@ -100,6 +104,12 @@
 #ifdef RS_USE_WIKI
 #include "gui/WikiPoos/WikiDialog.h"
 #endif
+#ifdef RS_USE_WIRE
+#include "gui/TheWire/WireDialog.h"
+#endif
+#ifdef RS_USE_PHOTO
+#include "gui/PhotoShare/PhotoShare.h"
+#endif
 #include "gui/Posted/PostedDialog.h"
 #include "gui/statistics/StatisticsWindow.h"
 
@@ -110,7 +120,7 @@
 #include "common/StatusDefs.h"
 #include "gui/notifyqt.h"
 
-#ifdef ENABLE_WEBUI
+#ifdef RS_WEBUI
 #	include "settings/WebuiPage.h"
 #endif
 
@@ -118,8 +128,11 @@
 #include <unistd.h>
 
 #define IMAGE_QUIT              ":/icons/png/exit.png"
-#define IMAGE_PREFERENCES       ":/icons/png/options.png"
+#define IMAGE_PREFERENCES       ":/icons/png/options2.png"
 #define IMAGE_ABOUT             ":/icons/png/info.png"
+#define IMAGE_STATS             ":/icons/png/netgraph2.png"
+#define IMAGE_CLOSE             ":/icons/png/exit2.png"
+
 #define IMAGE_ADDFRIEND         ":/icons/png/invite.png"
 #define IMAGE_RETROSHARE        ":/icons/logo_128.png"
 #define IMAGE_NOONLINE          ":/icons/logo_0_connected_128.png"
@@ -127,23 +140,15 @@
 #define IMAGE_TWOONLINE         ":/icons/logo_2_connected_128.png"
 #define IMAGE_OVERLAY           ":/icons/star_overlay_128.png"
 
-#define IMAGE_BWGRAPH           ":/images/ksysguard.png"
-#define IMAGE_MESSENGER         ":/images/rsmessenger48.png"
-#define IMAGE_CLOSE             ":/images/close_normal.png"
-#define IMAGE_BLOCK         	":/images/blockdevice.png"
+#define IMAGE_BWGRAPH           ":/icons/png/bandwidth.png"
 #define IMAGE_COLOR         	":/images/highlight.png"
-#define IMAGE_GAMES             ":/images/kgames.png"
-#define IMAGE_PHOTO             ":/images/lphoto.png"
 #define IMAGE_NEWRSCOLLECTION   ":/images/library.png"
 #define IMAGE_ADDSHARE          ":/images/directoryadd_24x24_shadow.png"
 #define IMAGE_OPTIONS           ":/images/settings.png"
-#define IMAGE_UNFINISHED        ":/images/underconstruction.png"
-#define IMAGE_MINIMIZE          ":/images/window_nofullscreen.png"
-#define IMAGE_MAXIMIZE          ":/images/window_fullscreen.png"
+#define IMAGE_MINIMIZE          ":/icons/fullscreen.png"
+#define IMAGE_MAXIMIZE          ":/icons/fullscreen-exit.png"
 
 #define IMAGE_PLUGINS           ":/images/extension_32.png"
-#define IMAGE_BLOGS             ":/images/kblogger.png"
-#define IMAGE_DHT               ":/images/dht16.png"
 
 /*static*/ bool MainWindow::hiddenmode = false;
 
@@ -172,6 +177,16 @@ MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags flags)
     ui = new Ui::MainWindow;
     trayIcon = NULL;
 
+	friendsDialog=NULL;
+	idDialog=NULL;
+	chatLobbyDialog=NULL;
+	settingsDialog=NULL;
+	transfersDialog=NULL;
+	messagesDialog=NULL;
+	gxschannelDialog=NULL;
+	gxsforumDialog=NULL;
+	postedDialog=NULL;
+
     /* Invoke the Qt Designer generated QObject setup routine */
     ui->setupUi(this);
 
@@ -231,12 +246,13 @@ MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags flags)
     statusBar()->setVisible(Settings->valueFromGroup("StatusBar", "ShowStatusBar", QVariant(true)).toBool());
 
     /* initialize combobox in status bar */
-    statusComboBox = new QComboBox(statusBar());
+    statusComboBox = new RSComboBox(statusBar());
     statusComboBox->setVisible(Settings->valueFromGroup("StatusBar", "ShowStatus", QVariant(true)).toBool());
     statusComboBox->setFocusPolicy(Qt::ClickFocus);
     initializeStatusObject(statusComboBox, true);
 
     QWidget *widget = new QWidget();
+    widget->setObjectName("trans_statusComboBoxFrame");
     QHBoxLayout *hbox = new QHBoxLayout();
     hbox->setMargin(0);
     hbox->setSpacing(6);
@@ -359,7 +375,14 @@ MainWindow::~MainWindow()
     delete soundStatus;
     delete toasterDisable;
     delete sysTrayStatus;
+    delete trayIcon;
+    delete trayMenu;
+//  delete notifyMenu; // already deleted by the deletion of trayMenu
+    StatisticsWindow::releaseInstance();
+
+#ifdef MESSENGER_WINDOW
     MessengerWindow::releaseInstance();
+#endif
 #ifdef UNFINISHED
     delete applicationWindow;
 #endif
@@ -401,12 +424,22 @@ void MainWindow::initStackedPage()
   PeopleDialog *peopleDialog = NULL;
   addPage(peopleDialog = new PeopleDialog(ui->stackPages), grp, &notify);
   #endif
-  addPage(newsFeed = new NewsFeed(ui->stackPages), grp, &notify);
 #ifdef RS_USE_WIKI
   WikiDialog *wikiDialog = NULL;
   addPage(wikiDialog = new WikiDialog(ui->stackPages), grp, &notify);
 #endif
 
+#ifdef RS_USE_WIRE
+  WireDialog *wireDialog = NULL;
+  addPage(wireDialog = new WireDialog(ui->stackPages), grp, &notify);
+#endif
+
+#ifdef RS_USE_PHOTO
+  PhotoShare *photoDialog = NULL;
+  addPage(photoDialog = new PhotoShare(ui->stackPages), grp, &notify);
+#endif
+
+
  std::cerr << "Looking for interfaces in existing plugins:" << std::endl;
  for(int i = 0;i<rsPlugins->nbPlugins();++i)
  {
@@ -433,10 +466,8 @@ void MainWindow::initStackedPage()
  }
 
 #ifndef RS_RELEASE_VERSION
-#ifdef PLUGINMGR
   addPage(pluginsPage = new gui::PluginsPage(ui->stackPages), grp, NULL);
 #endif
-#endif
 
 #undef GETSTARTED_GUI
 #ifdef GETSTARTED_GUI
@@ -450,12 +481,15 @@ void MainWindow::initStackedPage()
   }
 #endif
 
+
+  addPage(newsFeed = new NewsFeed(ui->stackPages), grp, &notify);
+
   //List All notify before Setting was created
-  QList<QPair<MainPage*, QPair<QAction*, QListWidgetItem*> > >::iterator notifyIt;
-  for (notifyIt = notify.begin(); notifyIt != notify.end(); ++notifyIt) {
-      UserNotify *userNotify = notifyIt->first->getUserNotify(this);
+  for (auto notifyIt:notify)
+  {
+      UserNotify *userNotify = notifyIt.first->getUserNotify();
       if (userNotify) {
-          userNotify->initialize(ui->toolBarPage, notifyIt->second.first, notifyIt->second.second);
+          userNotify->initialize(ui->toolBarPage, notifyIt.second.first, notifyIt.second.second);
           connect(userNotify, SIGNAL(countChanged()), this, SLOT(updateTrayCombine()));
           userNotifyList.push_back(userNotify);
       }
@@ -469,7 +503,7 @@ void MainWindow::initStackedPage()
 
 
 #ifdef UNFINISHED
-  addAction(new QAction(QIcon(IMAGE_UNFINISHED), tr("Unfinished"), ui->toolBar), &MainWindow::showApplWindow, SLOT(showApplWindow()));
+  addAction(new QAction(QIcon(), tr("Unfinished"), ui->toolBar), &MainWindow::showApplWindow, SLOT(showApplWindow()));
   ui->toolBarAction->addSeparator();
   notify += applicationWindow->getNotify();
 #endif
@@ -571,7 +605,7 @@ void MainWindow::displayDiskSpaceWarning(int loc,int size_limit_mb)
 void MainWindow::createTrayIcon()
 {
     /** Tray icon Menu **/
-    QMenu *trayMenu = new QMenu(this);
+    trayMenu = new QMenu(this);
     if (sysTrayStatus) sysTrayStatus->trayMenu = trayMenu;
     QObject::connect(trayMenu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));
     toggleVisibilityAction = trayMenu->addAction(QIcon(IMAGE_RETROSHARE), tr("Show/Hide"), this, SLOT(toggleVisibilitycontextmenu()));
@@ -586,17 +620,21 @@ void MainWindow::createTrayIcon()
     notifyMenu->menuAction()->setVisible(false);
 
     trayMenu->addSeparator();
-    trayMenu->addAction(QIcon(IMAGE_MESSENGER), tr("Open Messenger"), this, SLOT(showMessengerWindow()));
+#ifdef MESSENGER_WINDOW
+    trayMenu->addAction(QIcon(), tr("Open Messenger"), this, SLOT(showMessengerWindow()));
+#endif
     trayMenu->addAction(QIcon(IMAGE_MESSAGES), tr("Open Messages"), this, SLOT(showMess()));
-#ifdef ENABLE_WEBUI
+#ifdef RS_JSONAPI
+#ifdef RS_WEBUI
     trayMenu->addAction(QIcon(":/images/emblem-web.png"), tr("Show web interface"), this, SLOT(showWebinterface()));
-#endif // ENABLE_WEBUI
+#endif
+#endif
     trayMenu->addAction(QIcon(IMAGE_BWGRAPH), tr("Bandwidth Graph"), this, SLOT(showBandwidthGraph()));
-    trayMenu->addAction(QIcon(IMAGE_DHT), tr("Statistics"), this, SLOT(showStatisticsWindow()));
+    trayMenu->addAction(QIcon(IMAGE_STATS), tr("Statistics"), this, SLOT(showStatisticsWindow()));
 
 
 #ifdef UNFINISHED
-    trayMenu->addAction(QIcon(IMAGE_UNFINISHED), tr("Applications"), this, SLOT(showApplWindow()));
+    trayMenu->addAction(QIcon(), tr("Applications"), this, SLOT(showApplWindow()));
 #endif
     trayMenu->addAction(QIcon(IMAGE_PREFERENCES), tr("Options"), this, SLOT(showSettings()));
     trayMenu->addAction(QIcon(IMG_HELP), tr("Help"), this, SLOT(showHelpDialog()));
@@ -730,9 +768,12 @@ void MainWindow::updateStatus()
     float downKb = 0;
     float upKb = 0;
     rsConfig->GetCurrentDataRates(downKb, upKb);
+	uint64_t down = 0;
+	uint64_t up = 0;
+	rsConfig->GetTrafficSum(down, up);
 
     if (ratesstatus)
-        ratesstatus->getRatesStatus(downKb, upKb);
+        ratesstatus->getRatesStatus(downKb, down, upKb, up);
 
     if(torstatus)
         torstatus->getTorStatus();
@@ -1038,7 +1079,9 @@ void SetForegroundWindowInternal(HWND hWnd)
 			return _instance->gxsforumDialog;
 		case Posted:
 			return _instance->postedDialog;
-	}
+        case Home:
+            return _instance->homePage;
+    }
 
    return NULL;
 }
@@ -1082,11 +1125,13 @@ void MainWindow::showSettings()
     showWindow(MainWindow::Options);
 }
 
+#ifdef MESSENGER_WINDOW
 /** Shows Messenger window */
 void MainWindow::showMessengerWindow()
 {
     MessengerWindow::showYourself();
 }
+#endif
 
 /** Shows Statistics window */
 void MainWindow::showStatisticsWindow()
@@ -1094,12 +1139,14 @@ void MainWindow::showStatisticsWindow()
     StatisticsWindow::showYourself();
 }
 
-#ifdef ENABLE_WEBUI
+#ifdef RS_JSONAPI
+#ifdef RS_WEBUI
 void MainWindow::showWebinterface()
 {
     WebuiPage::showWebui();
 }
 #endif // ENABLE_WEBUI
+#endif 
 
 /** Shows Application window */
 #ifdef UNFINISHED
@@ -1185,6 +1232,8 @@ void MainWindow::toggleVisibility(QSystemTrayIcon::ActivationReason e)
             }
             raise();
             activateWindow();
+            if(torstatus)
+                torstatus->reset();
         } else {
             hide();
         }
@@ -1251,7 +1300,7 @@ static void setStatusObject(QObject *pObject, int nStatus)
         }
         return;
     }
-    QComboBox *pComboBox = dynamic_cast<QComboBox*>(pObject);
+    RSComboBox *pComboBox = dynamic_cast<RSComboBox*>(pObject);
     if (pComboBox) {
         /* set index of combobox */
         int nIndex = pComboBox->findData(nStatus, Qt::UserRole);
@@ -1340,7 +1389,7 @@ void MainWindow::initializeStatusObject(QObject *pObject, bool bConnect)
         }
     } else {
         /* initialize combobox */
-        QComboBox *pComboBox = dynamic_cast<QComboBox*>(pObject);
+        RSComboBox *pComboBox = dynamic_cast<RSComboBox*>(pObject);
         if (pComboBox) {
             pComboBox->addItem(QIcon(StatusDefs::imageStatus(RS_STATUS_ONLINE)), StatusDefs::name(RS_STATUS_ONLINE), RS_STATUS_ONLINE);
             pComboBox->addItem(QIcon(StatusDefs::imageStatus(RS_STATUS_BUSY)), StatusDefs::name(RS_STATUS_BUSY), RS_STATUS_BUSY);
@@ -1521,13 +1570,13 @@ void MainWindow::processLastArgs()
 	if (opModeStatus) {
 		QString opmode = Rshare::opmode().toLower();
 		if (opmode == "noturtle") {
-			opModeStatus->setCurrentIndex(RS_OPMODE_NOTURTLE - 1);
+			opModeStatus->setCurrentIndex(static_cast<typename std::underlying_type<RsOpMode>::type>(RsOpMode::NOTURTLE) - 1);
 		} else if (opmode == "gaming") {
-			opModeStatus->setCurrentIndex(RS_OPMODE_GAMING - 1);
+			opModeStatus->setCurrentIndex(static_cast<typename std::underlying_type<RsOpMode>::type>(RsOpMode::GAMING) - 1);
 		} else if (opmode == "minimal") {
-			opModeStatus->setCurrentIndex(RS_OPMODE_MINIMAL - 1);
+			opModeStatus->setCurrentIndex(static_cast<typename std::underlying_type<RsOpMode>::type>(RsOpMode::MINIMAL) - 1);
 		} else if (opmode != "") {
-			opModeStatus->setCurrentIndex(RS_OPMODE_FULL - 1);
+			opModeStatus->setCurrentIndex(static_cast<typename std::underlying_type<RsOpMode>::type>(RsOpMode::FULL) - 1);
 		}
 		opModeStatus->setOpMode();
 	} else {
@@ -1564,7 +1613,7 @@ void MainWindow::switchVisibilityStatus(StatusElement e,bool b)
 //{
 //    ServicePermissionDialog::showYourself();
 //}
-QComboBox *MainWindow::statusComboBoxInstance()
+RSComboBox *MainWindow::statusComboBoxInstance()
 {
 	return statusComboBox;
 }
@@ -1579,6 +1628,11 @@ NATStatus *MainWindow::natstatusInstance()
 	return natstatus;
 }
 
+void MainWindow::torstatusReset()
+{
+	if(torstatus != nullptr)
+		torstatus->reset();
+}
 DHTStatus *MainWindow::dhtstatusInstance()
 {
 	return dhtstatus;
@@ -1639,3 +1693,19 @@ void MainWindow::setCompactStatusMode(bool compact)
 	ratesstatus->setCompactMode(compact);
 	//opModeStatus: TODO Show only ???
 }
+
+Gui_InputDialogReturn MainWindow::guiInputDialog(const QString& windowTitle, const QString& labelText, QLineEdit::EchoMode textEchoMode, bool modal)
+{
+
+	QInputDialog dialog(this);
+	dialog.setWindowTitle(windowTitle);
+	dialog.setLabelText(labelText);
+	dialog.setTextEchoMode(textEchoMode);
+	dialog.setModal(modal);
+
+	Gui_InputDialogReturn ret;
+	ret.execReturn = dialog.exec();
+	ret.textValue = dialog.textValue();
+
+	return ret;
+}
