@@ -20,7 +20,8 @@
 
 #include "RemoteDirModel.h"
 
-#include "RsAutoUpdatePage.h"
+#include <retroshare-gui/RsAutoUpdatePage.h>
+
 #include "gui/common/FilesDefs.h"
 #include "gui/common/GroupDefs.h"
 #include "gui/common/RsCollection.h"
@@ -29,6 +30,7 @@
 #include "retroshare/rsfiles.h"
 #include "retroshare/rspeers.h"
 #include "util/misc.h"
+#include "util/qtthreadsutils.h"
 #include "retroshare/rsexpr.h"
 
 #include <QDir>
@@ -46,6 +48,15 @@
  * #define RDM_SEARCH_DEBUG
  ****/
 
+#define REMOTEDIRMODEL_COLUMN_NAME          0
+#define REMOTEDIRMODEL_COLUMN_FILENB        1
+#define REMOTEDIRMODEL_COLUMN_SIZE          2
+#define REMOTEDIRMODEL_COLUMN_AGE           3
+#define REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS 4
+#define REMOTEDIRMODEL_COLUMN_WN_VISU_DIR   5
+#define REMOTEDIRMODEL_COLUMN_COUNT         6
+#define RETROSHARE_DIR_MODEL_FILTER_STRING "filtered"
+
 static const uint32_t FLAT_VIEW_MAX_REFS_PER_SECOND       = 2000 ;
 static const size_t   FLAT_VIEW_MAX_REFS_TABLE_SIZE       = 10000 ; //
 static const uint32_t FLAT_VIEW_MIN_DELAY_BETWEEN_UPDATES = 120 ;	// dont rebuild ref list more than every 2 mins.
@@ -55,6 +66,7 @@ RetroshareDirModel::RetroshareDirModel(bool mode, QObject *parent)
   , ageIndicator(IND_ALWAYS)
   , RemoteMode(mode)//, nIndex(1), indexSet(1) /* ass zero index cant be used */
   , mLastRemote(false), mLastReq(0), mUpdating(false)
+  , mEventHandlerId(0)
 {
 #if QT_VERSION < QT_VERSION_CHECK (5, 0, 0)
 	setSupportedDragActions(Qt::CopyAction);
@@ -62,6 +74,37 @@ RetroshareDirModel::RetroshareDirModel(bool mode, QObject *parent)
 	treeStyle();
 
 	mDirDetails.ref = (void*)intptr_t(0xffffffff) ;
+
+	rsEvents->registerEventsHandler(
+	            [this](std::shared_ptr<const RsEvent> event)
+	                  {
+	                  	RsQThreadUtils::postToObject( [this,event]()  {  handleEvent_main_thread(event);  });
+	                  }
+	            , mEventHandlerId
+	            , RsEventType::SHARED_DIRECTORIES );
+}
+
+RetroshareDirModel::~RetroshareDirModel()
+{
+    rsEvents->unregisterEventsHandler(mEventHandlerId);
+}
+void RetroshareDirModel::handleEvent_main_thread(std::shared_ptr<const RsEvent> event)
+{
+    if(event->mType != RsEventType::SHARED_DIRECTORIES) return;
+
+    const RsSharedDirectoriesEvent *fe = dynamic_cast<const RsSharedDirectoriesEvent*>(event.get());
+    if(!fe)
+        return;
+
+    switch (fe->mEventCode)
+    {
+    case RsSharedDirectoriesEventCode::EXTRA_LIST_FILE_ADDED:
+    case RsSharedDirectoriesEventCode::EXTRA_LIST_FILE_REMOVED:
+        update();
+        break;
+    default:
+        break;
+    }
 }
 
 TreeStyle_RDM::TreeStyle_RDM(bool mode)
@@ -100,11 +143,9 @@ static bool isNewerThanEpoque(uint32_t ts)
 
 void RetroshareDirModel::treeStyle()
 {
-	categoryIcon.addPixmap(QPixmap(":/images/folder16.png"),
-	                     QIcon::Normal, QIcon::Off);
-	categoryIcon.addPixmap(QPixmap(":/images/folder_video.png"),
-	                     QIcon::Normal, QIcon::On);
-	peerIcon = QIcon(":/images/user/identity16.png");
+	categoryIcon.addPixmap(FilesDefs::getPixmapFromQtResourcePath(":/icons/folder.png"), QIcon::Normal, QIcon::Off);
+	categoryIcon.addPixmap(FilesDefs::getPixmapFromQtResourcePath(":/icons/folderopen.png"), QIcon::Normal, QIcon::On);
+	peerIcon = FilesDefs::getIconFromQtResourcePath(":/icons/folder-account.svg");
 }
 void TreeStyle_RDM::update()
 {
@@ -171,10 +212,10 @@ bool TreeStyle_RDM::hasChildren(const QModelIndex &parent) const
 	}
 	/* PERSON/DIR*/
 #ifdef RDM_DEBUG
-	std::cerr << "lookup PER/DIR #" << details.count;
+    std::cerr << "lookup PER/DIR #" << details.size;
 	std::cerr << std::endl;
 #endif
-    return (details.count > 0); /* do we have children? */
+    return (details.children.size() > 0); /* do we have children? */
 }
 bool FlatStyle_RDM::hasChildren(const QModelIndex &parent) const
 {
@@ -230,7 +271,7 @@ int TreeStyle_RDM::rowCount(const QModelIndex &parent) const
 
 	/* else PERSON/DIR*/
 #ifdef RDM_DEBUG
-	std::cerr << "lookup PER/DIR #" << details.count;
+    std::cerr << "lookup PER/DIR #" << details.size;
 	std::cerr << std::endl;
 #endif
 	if ((details.type == DIR_TYPE_ROOT) && !_showEmpty && RemoteMode)
@@ -239,14 +280,14 @@ int TreeStyle_RDM::rowCount(const QModelIndex &parent) const
 		//Scan all children to know if they are empty.
 		//And save their real row index
 		//Prefer do like that than modify requestDirDetails with a new flag (rsFiles->RequestDirDetails)
-		for(uint64_t i = 0; i < details.count; ++i)
+        for(uint64_t i = 0; i < details.children.size(); ++i)
 		{
-			if (requestDirDetails(details.children[i].ref, RemoteMode,childDetails) && (childDetails.count > 0))
+            if (requestDirDetails(details.children[i].ref, RemoteMode,childDetails) && (childDetails.children.size() > 0))
 				_parentRow.push_back(i);
 		}
 		return _parentRow.size();
 	}
-	return details.count;
+    return details.children.size();
 }
 int FlatStyle_RDM::rowCount(const QModelIndex &parent) const
 {
@@ -263,11 +304,11 @@ int FlatStyle_RDM::rowCount(const QModelIndex &parent) const
 
 int TreeStyle_RDM::columnCount(const QModelIndex &/*parent*/) const
 {
-	return COLUMN_COUNT;
+	return REMOTEDIRMODEL_COLUMN_COUNT;
 }
 int FlatStyle_RDM::columnCount(const QModelIndex &/*parent*/) const
 {
-	return COLUMN_COUNT;
+	return REMOTEDIRMODEL_COLUMN_COUNT;
 }
 
 QString RetroshareDirModel::getFlagsString(FileStorageFlags flags)
@@ -337,31 +378,32 @@ QString RetroshareDirModel::getAgeIndicatorString(const DirDetails &details) con
 const QIcon& RetroshareDirModel::getFlagsIcon(FileStorageFlags flags)
 {
     static QIcon *static_icons[8] = {NULL};
+    if(static_icons[0] == NULL)
+        static_icons[0] = new QIcon();
 
     int n=0;
     if(flags & DIR_FLAGS_ANONYMOUS_DOWNLOAD) n += 1 ;
     if(flags & DIR_FLAGS_ANONYMOUS_SEARCH  ) n += 2 ;
     if(flags & DIR_FLAGS_BROWSABLE         ) n += 4 ;
-    n-= 1;
 
     if(static_icons[n] == NULL)
     {
         QList<QIcon> icons ;
 
         if(flags & DIR_FLAGS_ANONYMOUS_SEARCH)
-            icons.push_back(QIcon(":icons/search_red_128.png")) ;
+            icons.push_back(FilesDefs::getIconFromQtResourcePath(":icons/search_red_128.png")) ;
         else
-            icons.push_back(QIcon(":icons/void_128.png")) ;
+            icons.push_back(FilesDefs::getIconFromQtResourcePath(":icons/void_128.png")) ;
 
         if(flags & DIR_FLAGS_ANONYMOUS_DOWNLOAD)
-            icons.push_back(QIcon(":icons/anonymous_blue_128.png")) ;
+            icons.push_back(FilesDefs::getIconFromQtResourcePath(":icons/anonymous_blue_128.png")) ;
         else
-            icons.push_back(QIcon(":icons/void_128.png")) ;
+            icons.push_back(FilesDefs::getIconFromQtResourcePath(":icons/void_128.png")) ;
 
         if(flags & DIR_FLAGS_BROWSABLE)
-            icons.push_back(QIcon(":icons/browsable_green_128.png")) ;
+            icons.push_back(FilesDefs::getIconFromQtResourcePath(":icons/browsable_green_128.png")) ;
         else
-            icons.push_back(QIcon(":icons/void_128.png")) ;
+            icons.push_back(FilesDefs::getIconFromQtResourcePath(":icons/void_128.png")) ;
 
         QPixmap pix ;
         GxsIdDetails::GenerateCombinedPixmap(pix, icons, 128);
@@ -383,7 +425,7 @@ QVariant RetroshareDirModel::filterRole(const DirDetails& details, int /*coln*/)
 
 QVariant RetroshareDirModel::decorationRole(const DirDetails& details,int coln) const
 {
-    if(coln == COLUMN_FRIEND_ACCESS)
+    if(coln == REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS)
     {
         if(details.type == DIR_TYPE_PERSON) return QVariant() ;
 
@@ -399,13 +441,13 @@ QVariant RetroshareDirModel::decorationRole(const DirDetails& details,int coln)
         time_t now = time(NULL) ;
 
 		if(ageIndicator != IND_ALWAYS && now > details.max_mtime + ageIndicator)
-			return QIcon(":/images/folder_grey.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_grey.png");
 		else if (ageIndicator == IND_LAST_DAY )
-			return QIcon(":/images/folder_green.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_green.png");
 		else if (ageIndicator == IND_LAST_WEEK )
-			return QIcon(":/images/folder_yellow.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_yellow.png");
 		else if (ageIndicator == IND_LAST_MONTH )
-			return QIcon(":/images/folder_red.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_red.png");
 		else
 			return (QIcon(peerIcon));
 	}
@@ -414,13 +456,13 @@ QVariant RetroshareDirModel::decorationRole(const DirDetails& details,int coln)
         time_t now = time(NULL) ;
 
 		if(ageIndicator != IND_ALWAYS && now > details.max_mtime + ageIndicator)
-			return QIcon(":/images/folder_grey.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_grey.png");
 		else if (ageIndicator == IND_LAST_DAY )
-			return QIcon(":/images/folder_green.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_green.png");
 		else if (ageIndicator == IND_LAST_WEEK )
-			return QIcon(":/images/folder_yellow.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_yellow.png");
 		else if (ageIndicator == IND_LAST_MONTH )
-			return QIcon(":/images/folder_red.png");
+			return FilesDefs::getIconFromQtResourcePath(":/images/folder_red.png");
 		else
 			return QIcon(categoryIcon);
 	}
@@ -428,9 +470,9 @@ QVariant RetroshareDirModel::decorationRole(const DirDetails& details,int coln)
 	{
 		// extensions predefined
         if(details.hash.isNull())
-            return QIcon(":/images/reset.png") ; // file is being hashed
+            return FilesDefs::getIconFromQtResourcePath(":/images/reset.png") ; // file is being hashed
         else
-            return FilesDefs::getIconFromFilename(QString::fromUtf8(details.name.c_str()));
+            return FilesDefs::getIconFromFileType(QString::fromUtf8(details.name.c_str()));
 	}
 	else
 		return QVariant();
@@ -451,7 +493,7 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 	{
 		switch(coln)
 		{
-		case COLUMN_NAME: {
+		case REMOTEDIRMODEL_COLUMN_NAME: {
 				//SharedDirStats stats ;
 				QString res ;
 
@@ -464,7 +506,7 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 
 				return res ;
 			}
-		case COLUMN_FILENB: {
+		case REMOTEDIRMODEL_COLUMN_FILENB: {
 				SharedDirStats stats ;
 
 				if(RemoteMode)
@@ -472,7 +514,7 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 				else if(details.id == rsPeers->getOwnId())
 					rsFiles->getSharedDirStatistics(rsPeers->getOwnId(),stats) ;
                 else
-                    stats.total_number_of_files = details.count;
+                    stats.total_number_of_files = details.children.size();
 
 				if(stats.total_number_of_files > 0)
 				{
@@ -483,7 +525,7 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 				}
 				return tr("Empty");
 			}
-		case COLUMN_SIZE: {
+		case REMOTEDIRMODEL_COLUMN_SIZE: {
 				SharedDirStats stats ;
 
 				if(RemoteMode)
@@ -498,7 +540,7 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 
 				return QString();
 			}
-		case COLUMN_AGE:
+		case REMOTEDIRMODEL_COLUMN_AGE:
 				if(!isNewerThanEpoque(details.max_mtime))
 					return QString();
 				else if(details.id != rsPeers->getOwnId())
@@ -514,17 +556,29 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 	{
 		switch(coln)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				return QString::fromUtf8(details.name.c_str());
-			case COLUMN_FILENB:
+			case REMOTEDIRMODEL_COLUMN_FILENB:
 				return  QVariant();
-			case COLUMN_SIZE:
-				return  misc::friendlyUnit(details.count);
-			case COLUMN_AGE:
-				return  (details.type == DIR_TYPE_FILE)?(misc::timeRelativeToNow(details.max_mtime)):QString();
-			case COLUMN_FRIEND_ACCESS:
+			case REMOTEDIRMODEL_COLUMN_SIZE:
+                return  misc::friendlyUnit(details.size);
+			case REMOTEDIRMODEL_COLUMN_AGE:
+			{
+				if(details.type == DIR_TYPE_FILE)
+					return misc::timeRelativeToNow(details.max_mtime);
+				else if(details.type == DIR_TYPE_EXTRA_FILE)
+				{
+					FileInfo fi;
+					if (rsFiles->FileDetails(details.hash, RS_FILE_HINTS_EXTRA , fi))
+						return misc::timeRelativeToNow((rstime_t)fi.age-(30 * 3600 * 24)); // AFI_DEFAULT_PERIOD
+					return QString();
+				}
+				else
+					return QString();
+			}
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS:
 				return QVariant();
-			case COLUMN_WN_VISU_DIR:
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR:
 				return getGroupsString(details.flags,details.parent_groups) ;
 
 			default:
@@ -535,22 +589,22 @@ QVariant TreeStyle_RDM::displayRole(const DirDetails& details,int coln) const
 	{
 		switch(coln)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				return QString::fromUtf8(details.name.c_str());
 				break;
-			case COLUMN_FILENB:
-				if (details.count > 1)
+			case REMOTEDIRMODEL_COLUMN_FILENB:
+                if (details.children.size() > 1)
 				{
-					return QString::number(details.count) + " " + tr("Files");
+                    return QString::number(details.children.size()) + " " + tr("Files");
 				}
-				return QString::number(details.count) + " " + tr("File");
-			case COLUMN_SIZE:
-				return QVariant();
-			case COLUMN_AGE:
+                return QString::number(details.children.size()) + " " + tr("File");
+			case REMOTEDIRMODEL_COLUMN_SIZE:
+                return  misc::friendlyUnit(details.size);
+            case REMOTEDIRMODEL_COLUMN_AGE:
 				return misc::timeRelativeToNow(details.max_mtime);
-			case COLUMN_FRIEND_ACCESS:
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS:
 				return QVariant();
-			case COLUMN_WN_VISU_DIR:
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR:
 				return getGroupsString(details.flags,details.parent_groups) ;
 
 			default:
@@ -605,12 +659,12 @@ QVariant FlatStyle_RDM::displayRole(const DirDetails& details,int coln) const
 	if (details.type == DIR_TYPE_FILE || details.type == DIR_TYPE_EXTRA_FILE) /* File */
 		switch(coln)
 		{
-			case COLUMN_NAME: return QString::fromUtf8(details.name.c_str());
-			case COLUMN_FILENB: return QString();
-			case COLUMN_SIZE: return misc::friendlyUnit(details.count);
-			case COLUMN_AGE: return misc::timeRelativeToNow(details.max_mtime);
-			case COLUMN_FRIEND_ACCESS: return QString::fromUtf8(rsPeers->getPeerName(details.id).c_str());
-			case COLUMN_WN_VISU_DIR: return computeDirectoryPath(details);
+			case REMOTEDIRMODEL_COLUMN_NAME: return QString::fromUtf8(details.name.c_str());
+			case REMOTEDIRMODEL_COLUMN_FILENB: return QString();
+            case REMOTEDIRMODEL_COLUMN_SIZE: return misc::friendlyUnit(details.size);
+			case REMOTEDIRMODEL_COLUMN_AGE: return misc::timeRelativeToNow(details.max_mtime);
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS: return QString::fromUtf8(rsPeers->getPeerName(details.id).c_str());
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR: return computeDirectoryPath(details);
 			default:
 				return QVariant() ;
 		}
@@ -630,9 +684,9 @@ QVariant TreeStyle_RDM::sortRole(const QModelIndex& /*index*/,const DirDetails&
 	{
 		switch(coln)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				return (RemoteMode)?(QString::fromUtf8(rsPeers->getPeerName(details.id).c_str())):tr("My files");
-			case COLUMN_FILENB: {
+			case REMOTEDIRMODEL_COLUMN_FILENB: {
 				SharedDirStats stats ;
 
 				if(RemoteMode)
@@ -642,7 +696,7 @@ QVariant TreeStyle_RDM::sortRole(const QModelIndex& /*index*/,const DirDetails&
 
 				return (qulonglong) stats.total_number_of_files;
 			}
-			case COLUMN_SIZE: {
+			case REMOTEDIRMODEL_COLUMN_SIZE: {
 				SharedDirStats stats ;
 
 				if(RemoteMode)
@@ -652,7 +706,7 @@ QVariant TreeStyle_RDM::sortRole(const QModelIndex& /*index*/,const DirDetails&
 
 				return (qulonglong) stats.total_shared_size;
 			}
-			case COLUMN_AGE:
+			case REMOTEDIRMODEL_COLUMN_AGE:
 				return details.max_mtime;
 			default:
 				return QString();
@@ -662,17 +716,17 @@ QVariant TreeStyle_RDM::sortRole(const QModelIndex& /*index*/,const DirDetails&
 	{
 		switch(coln)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				return QString::fromUtf8(details.name.c_str());
-			case COLUMN_FILENB:
+			case REMOTEDIRMODEL_COLUMN_FILENB:
 				return (qulonglong) 0;
-			case COLUMN_SIZE:
-				return (qulonglong) details.count;
-			case COLUMN_AGE:
+			case REMOTEDIRMODEL_COLUMN_SIZE:
+                return (qulonglong) details.size;
+			case REMOTEDIRMODEL_COLUMN_AGE:
 				return details.max_mtime;
-			case COLUMN_FRIEND_ACCESS:
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS:
 				return getFlagsString(details.flags);
-			case COLUMN_WN_VISU_DIR:
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR:
 				{
 					QString ind("");
 					if (ageIndicator != IND_ALWAYS)
@@ -687,15 +741,15 @@ QVariant TreeStyle_RDM::sortRole(const QModelIndex& /*index*/,const DirDetails&
 	{
 		switch(coln)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				return QString::fromUtf8(details.name.c_str());
-			case COLUMN_FILENB:
-				return (qulonglong) details.count;
-			case COLUMN_SIZE:
+			case REMOTEDIRMODEL_COLUMN_FILENB:
+                return (qulonglong) details.children.size();
+			case REMOTEDIRMODEL_COLUMN_SIZE:
 				return (qulonglong) 0;
-			case COLUMN_AGE:
+			case REMOTEDIRMODEL_COLUMN_AGE:
 				return details.max_mtime;
-			case COLUMN_FRIEND_ACCESS:
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS:
 				return getFlagsString(details.flags);
 			default:
 				return tr("DIR");
@@ -715,12 +769,12 @@ QVariant FlatStyle_RDM::sortRole(const QModelIndex& /*index*/,const DirDetails&
 	{
 		switch(coln)
 		{
-			case COLUMN_NAME: return QString::fromUtf8(details.name.c_str());
-			case COLUMN_FILENB: return (qulonglong) 0;
-			case COLUMN_SIZE: return (qulonglong) details.count;
-			case COLUMN_AGE: return  details.max_mtime;
-			case COLUMN_FRIEND_ACCESS: return QString::fromUtf8(rsPeers->getPeerName(details.id).c_str());
-			case COLUMN_WN_VISU_DIR: {
+			case REMOTEDIRMODEL_COLUMN_NAME: return QString::fromUtf8(details.name.c_str());
+			case REMOTEDIRMODEL_COLUMN_FILENB: return (qulonglong) 0;
+            case REMOTEDIRMODEL_COLUMN_SIZE: return (qulonglong) details.size;
+			case REMOTEDIRMODEL_COLUMN_AGE: return  details.max_mtime;
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS: return QString::fromUtf8(rsPeers->getPeerName(details.id).c_str());
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR: {
 				RS_STACK_MUTEX(_ref_mutex) ;
 
 				return computeDirectoryPath(details);
@@ -748,7 +802,7 @@ QVariant RetroshareDirModel::data(const QModelIndex &index, int role) const
 
 	if (role == Qt::TextAlignmentRole)
 	{
-		if((coln == COLUMN_FILENB) || (coln == COLUMN_SIZE))
+		if((coln == REMOTEDIRMODEL_COLUMN_FILENB) || (coln == REMOTEDIRMODEL_COLUMN_SIZE))
 			return int( Qt::AlignRight | Qt::AlignVCenter);
 		else
 			return QVariant();
@@ -850,7 +904,7 @@ QVariant TreeStyle_RDM::headerData(int section, Qt::Orientation orientation, int
 		int defw = QFontMetricsF(QWidget().font()).width(headerData(section,Qt::Horizontal,Qt::DisplayRole).toString()) ;
 		int defh = QFontMetricsF(QWidget().font()).height();
 
-		if (section < COLUMN_AGE)
+		if (section < REMOTEDIRMODEL_COLUMN_AGE)
 		{
 			defw = 200/16.0*defh;
 		}
@@ -864,7 +918,7 @@ QVariant TreeStyle_RDM::headerData(int section, Qt::Orientation orientation, int
 	{
 		switch(section)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				if (RemoteMode)
                     if(mUpdating)
 						return tr("Friends Directories [updating...]");
@@ -875,18 +929,18 @@ QVariant TreeStyle_RDM::headerData(int section, Qt::Orientation orientation, int
 						return tr("My Directories [updating...]");
 					else
 						return tr("My Directories");
-			case COLUMN_FILENB:
+			case REMOTEDIRMODEL_COLUMN_FILENB:
 				return tr("# Files");
-			case COLUMN_SIZE:
+			case REMOTEDIRMODEL_COLUMN_SIZE:
 				return tr("Size");
-			case COLUMN_AGE:
+			case REMOTEDIRMODEL_COLUMN_AGE:
 				return tr("Age");
-			case COLUMN_FRIEND_ACCESS:
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS:
 				if (RemoteMode)
 					return tr("Friend");
 				else
 					return tr("Access");
-			case COLUMN_WN_VISU_DIR:
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR:
 				if (RemoteMode)
 					return tr("What's new");
 				else
@@ -904,7 +958,7 @@ QVariant FlatStyle_RDM::headerData(int section, Qt::Orientation orientation, int
 		int defw = QFontMetricsF(QWidget().font()).width(headerData(section,Qt::Horizontal,Qt::DisplayRole).toString()) ;
 		int defh = QFontMetricsF(QWidget().font()).height();
 
-		if (section < COLUMN_AGE)
+		if (section < REMOTEDIRMODEL_COLUMN_AGE)
 		{
 			defw = defh*200/16.0;
 		}
@@ -918,24 +972,24 @@ QVariant FlatStyle_RDM::headerData(int section, Qt::Orientation orientation, int
 	{
 		switch(section)
 		{
-			case COLUMN_NAME:
+			case REMOTEDIRMODEL_COLUMN_NAME:
 				if (RemoteMode)
 				{
 					return tr("Friends Directories");
 				}
 				return tr("My Directories");
-			case COLUMN_FILENB:
+			case REMOTEDIRMODEL_COLUMN_FILENB:
 				return tr("# Files");
-			case COLUMN_SIZE:
+			case REMOTEDIRMODEL_COLUMN_SIZE:
 				return tr("Size");
-			case COLUMN_AGE:
+			case REMOTEDIRMODEL_COLUMN_AGE:
 				return tr("Age");
-			case COLUMN_FRIEND_ACCESS:
+			case REMOTEDIRMODEL_COLUMN_FRIEND_ACCESS:
 				if(RemoteMode)
 					return tr("Friend");
 				else
 					return tr("Share Flags");
-			case COLUMN_WN_VISU_DIR:
+			case REMOTEDIRMODEL_COLUMN_WN_VISU_DIR:
 				return tr("Directory");
 		}
 		return tr("Column %1").arg(section);
@@ -1051,7 +1105,7 @@ QModelIndex TreeStyle_RDM::parent( const QModelIndex & index ) const
 	std::cerr << "Creating index 3 row=" << details.prow << ", column=" << 0 << ", ref=" << (void*)details.parent << std::endl;
 #endif
 
-	return createIndex(details.prow, COLUMN_NAME, details.parent);
+	return createIndex(details.prow, REMOTEDIRMODEL_COLUMN_NAME, details.parent);
 }
 QModelIndex FlatStyle_RDM::parent( const QModelIndex & index ) const
 {
@@ -1104,15 +1158,9 @@ Qt::ItemFlags RetroshareDirModel::flags( const QModelIndex & index ) const
 /* Callback from Core*/
 void RetroshareDirModel::preMods()
 {
-    emit layoutAboutToBeChanged();
     mUpdating = true ;
-#if QT_VERSION < 0x050000
-	reset();
-#else
-	beginResetModel();
-	endResetModel();
-#endif
 
+	beginResetModel();
 #ifdef RDM_DEBUG
 	std::cerr << "RetroshareDirModel::preMods()" << std::endl;
 #endif
@@ -1121,20 +1169,14 @@ void RetroshareDirModel::preMods()
 /* Callback from Core*/
 void RetroshareDirModel::postMods()
 {
-//	emit layoutAboutToBeChanged();
     mUpdating = false ;
-#if QT_VERSION >= 0x040600
-	beginResetModel();
-#endif
-
 #ifdef RDM_DEBUG
 	std::cerr << "RetroshareDirModel::postMods()" << std::endl;
 #endif
 
-#if QT_VERSION >= 0x040600
 	endResetModel();
-#endif
-	emit layoutChanged();
+
+    emit dataChanged(createIndex(0,0,(void*)NULL), createIndex(rowCount()-1,REMOTEDIRMODEL_COLUMN_COUNT-1,(void*)NULL));
 }
 
 void FlatStyle_RDM::postMods()
@@ -1254,8 +1296,7 @@ void RetroshareDirModel::downloadSelected(const QModelIndexList &list,bool inter
 			   std::cerr << std::endl;
 			   std::list<RsPeerId> srcIds;
 			   srcIds.push_back(details.id);
-			   rsFiles -> FileRequest(details.name, details.hash,
-			                          details.count, "", RS_FILE_REQ_ANONYMOUS_ROUTING, srcIds);
+               rsFiles -> FileRequest(details.name, details.hash, details.size, "", RS_FILE_REQ_ANONYMOUS_ROUTING, srcIds);
 		   }
 		   /* if it is a dir, copy all files included*/
 		   else if (details.type == DIR_TYPE_DIR)
@@ -1276,7 +1317,7 @@ void RetroshareDirModel::downloadDirectory(const DirDetails & dirDetails, int pr
 		QString cleanPath = QDir::cleanPath(QString::fromUtf8(rsFiles->getDownloadDirectory().c_str()) + "/" + QString::fromUtf8(dirDetails.path.substr(prefixLen).c_str()));
 
 		srcIds.push_back(dirDetails.id);
-		rsFiles->FileRequest(dirDetails.name, dirDetails.hash, dirDetails.count, cleanPath.toUtf8().constData(), RS_FILE_REQ_ANONYMOUS_ROUTING, srcIds);
+        rsFiles->FileRequest(dirDetails.name, dirDetails.hash, dirDetails.size, cleanPath.toUtf8().constData(), RS_FILE_REQ_ANONYMOUS_ROUTING, srcIds);
 	}
 	else if (dirDetails.type & DIR_TYPE_DIR)
 	{
@@ -1308,7 +1349,7 @@ void RetroshareDirModel::getDirDetailsFromSelect (const QModelIndexList &list, s
 	QModelIndexList::const_iterator it;
 	for(it = list.begin(); it != list.end(); ++it)
 	{
-		if(it->column()==COLUMN_FILENB)
+		if(it->column()==REMOTEDIRMODEL_COLUMN_FILENB)
 		{
 			void *ref = it -> internalPointer();
 
@@ -1345,7 +1386,7 @@ void RetroshareDirModel::getFileInfoFromIndexList(const QModelIndexList& list, s
 	std::set<std::string> already_in ;
 
 	for(QModelIndexList::const_iterator it(list.begin()); it != list.end(); ++it)
-		if(it->column()==COLUMN_NAME)
+		if(it->column()==REMOTEDIRMODEL_COLUMN_NAME)
 		{
 			void *ref = it -> internalPointer();
 
@@ -1361,7 +1402,7 @@ void RetroshareDirModel::getFileInfoFromIndexList(const QModelIndexList& list, s
 			std::cerr << "::::::::::::FileRecommend:::: " << std::endl;
 			std::cerr << "Name: " << details.name << std::endl;
 			std::cerr << "Hash: " << details.hash << std::endl;
-			std::cerr << "Size: " << details.count << std::endl;
+            std::cerr << "Size: " << details.size << std::endl;
 			std::cerr << "Path: " << details.path << std::endl;
 #endif
 			// Note: for directories, the returned hash, is the peer id, so if we collect
@@ -1382,7 +1423,7 @@ void RetroshareDirModel::getFileInfoFromIndexList(const QModelIndexList& list, s
  * OLD RECOMMEND SYSTEM - DISABLED
  ******/
 
-void RetroshareDirModel::openSelected(const QModelIndexList &qmil)
+void RetroshareDirModel::openSelected(const QModelIndexList &qmil, bool openDir)
 {
 #ifdef RDM_DEBUG
 	std::cerr << "RetroshareDirModel::openSelected()" << std::endl;
@@ -1405,12 +1446,15 @@ void RetroshareDirModel::openSelected(const QModelIndexList &qmil)
 
 		QDir dir(QString::fromUtf8((*it).path.c_str()));
 		QString dest;
-		if ((*it).type & DIR_TYPE_FILE || (*it).type & DIR_TYPE_EXTRA_FILE) {
+		if ((*it).type & DIR_TYPE_FILE || (!openDir && (*it).type & DIR_TYPE_EXTRA_FILE)) {
 			dest = dir.absoluteFilePath(QString::fromUtf8(it->name.c_str()));
 		} else if ((*it).type & DIR_TYPE_DIR) {
 			dest = dir.absolutePath();
+		} else if (openDir) // extra
+		{
+			QDir d = QFileInfo(it->name.c_str()).absoluteDir();
+			dest = d.absolutePath();
 		}
-
 		std::cerr << "Opening this file: " << dest.toStdString() << std::endl ;
 
 		RsUrlHandler::openUrl(QUrl::fromLocalFile(dest));
@@ -1423,7 +1467,7 @@ void RetroshareDirModel::openSelected(const QModelIndexList &qmil)
 
 void RetroshareDirModel::getFilePath(const QModelIndex& index, std::string& fullpath)
 {
-    void *ref = index.sibling(index.row(),COLUMN_FILENB).internalPointer();
+    void *ref = index.sibling(index.row(),REMOTEDIRMODEL_COLUMN_FILENB).internalPointer();
 
     DirDetails details ;
 
@@ -1556,7 +1600,7 @@ QMimeData * RetroshareDirModel::mimeData ( const QModelIndexList & indexes ) con
 		std::cerr << "::::::::::::FileDrag:::: " << std::endl;
 		std::cerr << "Name: " << details.name << std::endl;
 		std::cerr << "Hash: " << details.hash << std::endl;
-		std::cerr << "Size: " << details.count << std::endl;
+        std::cerr << "Size: " << details.size << std::endl;
 		std::cerr << "Path: " << details.path << std::endl;
 #endif
 
@@ -1576,9 +1620,8 @@ QMimeData * RetroshareDirModel::mimeData ( const QModelIndexList & indexes ) con
 			continue; /* duplicate */
 		}
 
-        drags[details.hash] = details.count;
-
-        QString line = QString("%1/%2/%3/").arg(QString::fromUtf8(details.name.c_str()), QString::fromStdString(details.hash.toStdString()), QString::number(details.count));
+        drags[details.hash] = details.children.size();
+        QString line = QString("%1/%2/%3/").arg(QString::fromUtf8(details.name.c_str()), QString::fromStdString(details.hash.toStdString()), QString::number(details.size));
 
 		if (RemoteMode)
 		{
