@@ -55,6 +55,7 @@
 #include "gui/chat/ChatUserNotify.h"
 #include "gui/connect/ConnectProgressDialog.h"
 #include "gui/common/ElidedLabel.h"
+#include "gui/common/FilesDefs.h"
 
 #include "FriendList.h"
 #include "ui_FriendList.h"
@@ -65,12 +66,12 @@
 #define IMAGE_EXPORTFRIEND       ":/images/user/friend_suggestion16.png"
 #define IMAGE_ADDFRIEND          ":/images/user/add_user16.png"
 #define IMAGE_FRIENDINFO         ":/images/info16.png"
-#define IMAGE_CHAT               ":/images/chat_24.png"
-#define IMAGE_MSG                ":/images/mail_new.png"
-#define IMAGE_CONNECT            ":/images/connect_friend.png"
+#define IMAGE_CHAT               ":/icons/png/chats.png"
+#define IMAGE_MSG                ":/icons/mail/write-mail.png"
+#define IMAGE_CONNECT            ":/icons/connection.svg"
 #define IMAGE_COPYLINK           ":/images/copyrslink.png"
 #define IMAGE_GROUP16            ":/images/user/group16.png"
-#define IMAGE_EDIT               ":/images/edit_16.png"
+#define IMAGE_EDIT               ":/icons/png/pencil-edit-button.png"
 #define IMAGE_REMOVE             ":/images/delete.png"
 #define IMAGE_EXPAND             ":/images/edit_add24.png"
 #define IMAGE_COLLAPSE           ":/images/edit_remove24.png"
@@ -126,8 +127,7 @@ FriendList::FriendList(QWidget *parent) :
 #ifdef RS_DIRECT_CHAT
 	connect(ui->peerTreeWidget, SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)), this, SLOT(chatfriend(QTreeWidgetItem *)));
 #else
-	connect( ui->peerTreeWidget, SIGNAL(itemClicked(QTreeWidgetItem *, int)),
-	         this, SLOT(expandItem(QTreeWidgetItem *)) );
+	connect( ui->peerTreeWidget, SIGNAL(itemClicked(QTreeWidgetItem *, int)), this, SLOT(expandItem(QTreeWidgetItem *)) );
 #endif
 
     connect(NotifyQt::getInstance(), SIGNAL(groupsChanged(int)), this, SLOT(groupsChanged()));
@@ -298,8 +298,9 @@ void FriendList::peerTreeWidgetCustomPopupMenu()
 
     QMenu *contextMenu = new QMenu(this);
 
-    QWidget *widget = new QWidget(contextMenu);
-    widget->setStyleSheet( ".QWidget{background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1,stop:0 #FEFEFE, stop:1 #E8E8E8); border: 1px solid #CCCCCC;}");
+    QFrame *widget = new QFrame(contextMenu);
+    widget->setObjectName("gradFrame"); //Use qss
+    //widget->setStyleSheet( ".QWidget{background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1,stop:0 #FEFEFE, stop:1 #E8E8E8); border: 1px solid #CCCCCC;}");
 
     // create menu header
     QHBoxLayout *hbox = new QHBoxLayout(widget);
@@ -307,12 +308,14 @@ void FriendList::peerTreeWidgetCustomPopupMenu()
     hbox->setSpacing(6);
 
     QLabel *iconLabel = new QLabel(widget);
-    QPixmap pix = QPixmap(":/images/user/friends24.png").scaledToHeight(QFontMetricsF(iconLabel->font()).height()*1.5);
+    iconLabel->setObjectName("trans_Icon");
+    QPixmap pix = FilesDefs::getPixmapFromQtResourcePath(":/images/user/friends24.png").scaledToHeight(QFontMetricsF(iconLabel->font()).height()*1.5);
     iconLabel->setPixmap(pix);
     iconLabel->setMaximumSize(iconLabel->frameSize().height() + pix.height(), pix.width());
     hbox->addWidget(iconLabel);
 
     QLabel *textLabel = new QLabel("<strong>RetroShare</strong>", widget);
+    textLabel->setObjectName("trans_Text");
     hbox->addWidget(textLabel);
 
     QSpacerItem *spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
@@ -353,25 +356,25 @@ void FriendList::peerTreeWidgetCustomPopupMenu()
              {
                  bool standard = c->data(COLUMN_DATA, ROLE_STANDARD).toBool();
 #ifdef RS_DIRECT_CHAT
-                 contextMenu->addAction(QIcon(IMAGE_MSG), tr("Send message to whole group"), this, SLOT(msgfriend()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_MSG), tr("Send message to whole group"), this, SLOT(msgfriend()));
                  contextMenu->addSeparator();
 #endif // RS_DIRECT_CHAT
-                 contextMenu->addAction(QIcon(IMAGE_EDIT), tr("Edit Group"), this, SLOT(editGroup()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_EDIT), tr("Edit Group"), this, SLOT(editGroup()));
 
-                 QAction *action = contextMenu->addAction(QIcon(IMAGE_REMOVE), tr("Remove Group"), this, SLOT(removeGroup()));
+                 QAction *action = contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_REMOVE), tr("Remove Group"), this, SLOT(removeGroup()));
                  action->setDisabled(standard);
              }
              break;
          case TYPE_GPG:
         {
 #ifdef RS_DIRECT_CHAT
-             contextMenu->addAction(QIcon(IMAGE_CHAT), tr("Chat"), this, SLOT(chatfriendproxy()));
-             contextMenu->addAction(QIcon(IMAGE_MSG), tr("Send message"), this, SLOT(msgfriend()));
+             contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_CHAT), tr("Chat"), this, SLOT(chatfriendproxy()));
+             contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_MSG), tr("Send message"), this, SLOT(msgfriend()));
              contextMenu->addSeparator();
 #endif // RS_DIRECT_CHAT
 
-             contextMenu->addAction(QIcon(IMAGE_FRIENDINFO), tr("Profile details"), this, SLOT(configurefriend()));
-             contextMenu->addAction(QIcon(IMAGE_DENYFRIEND), tr("Deny connections"), this, SLOT(removefriend()));
+             contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_FRIENDINFO), tr("Profile details"), this, SLOT(configurefriend()));
+             contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_DENYFRIEND), tr("Deny connections"), this, SLOT(removefriend()));
 
              if(mShowGroups)
              {
@@ -413,8 +416,8 @@ void FriendList::peerTreeWidgetCustomPopupMenu()
                      }
                  }
 
-                 QMenu *groupsMenu = contextMenu->addMenu(QIcon(IMAGE_GROUP16), tr("Groups"));
-                 groupsMenu->addAction(QIcon(IMAGE_EXPAND), tr("Create new group"), this, SLOT(createNewGroup()));
+                 QMenu *groupsMenu = contextMenu->addMenu(FilesDefs::getIconFromQtResourcePath(IMAGE_GROUP16), tr("Groups"));
+                 groupsMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_EXPAND), tr("Create new group"), this, SLOT(createNewGroup()));
 
                  if (addToGroupMenu || moveToGroupMenu || foundGroup) {
                      if (addToGroupMenu) {
@@ -446,24 +449,24 @@ void FriendList::peerTreeWidgetCustomPopupMenu()
          case TYPE_SSL:
              {
 #ifdef RS_DIRECT_CHAT
-                 contextMenu->addAction(QIcon(IMAGE_CHAT), tr("Chat"), this, SLOT(chatfriendproxy()));
-                 contextMenu->addAction(QIcon(IMAGE_MSG), tr("Send message to this node"), this, SLOT(msgfriend()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_CHAT), tr("Chat"), this, SLOT(chatfriendproxy()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_MSG), tr("Send message to this node"), this, SLOT(msgfriend()));
                  contextMenu->addSeparator();
 #endif // RS_DIRECT_CHAT
 
-                 contextMenu->addAction(QIcon(IMAGE_FRIENDINFO), tr("Node details"), this, SLOT(configurefriend()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_FRIENDINFO), tr("Node details"), this, SLOT(configurefriend()));
 
                  if (type == TYPE_GPG || type == TYPE_SSL) {
-                     contextMenu->addAction(QIcon(IMAGE_EXPORTFRIEND), tr("Recommend this node to..."), this, SLOT(recommendfriend()));
+                     contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_EXPORTFRIEND), tr("Recommend this node to..."), this, SLOT(recommendfriend()));
                  }
 
 				 if(!rsPeers->isHiddenNode(rsPeers->getOwnId()) || rsPeers->isHiddenNode( RsPeerId(getRsId(c)) ))
-					 contextMenu->addAction(QIcon(IMAGE_CONNECT), tr("Attempt to connect"), this, SLOT(connectfriend()));
+                     contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_CONNECT), tr("Attempt to connect"), this, SLOT(connectfriend()));
 
-                 contextMenu->addAction(QIcon(IMAGE_COPYLINK), tr("Copy certificate link"), this, SLOT(copyFullCertificate()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_COPYLINK), tr("Copy certificate link"), this, SLOT(copyFullCertificate()));
 
                  //this is a SSL key
-                 contextMenu->addAction(QIcon(IMAGE_REMOVEFRIEND), tr("Remove Friend Node"), this, SLOT(removefriend()));
+                 contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_REMOVEFRIEND), tr("Remove Friend Node"), this, SLOT(removefriend()));
 
              }
          }
@@ -472,12 +475,12 @@ void FriendList::peerTreeWidgetCustomPopupMenu()
 
     contextMenu->addSeparator();
 
-    QAction *action = contextMenu->addAction(QIcon(IMAGE_PASTELINK), tr("Paste certificate link"), this, SLOT(pastePerson()));
+    QAction *action = contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_PASTELINK), tr("Paste certificate link"), this, SLOT(pastePerson()));
     if (RSLinkClipboard::empty(RetroShareLink::TYPE_CERTIFICATE))
         action->setDisabled(true);
 
-    contextMenu->addAction(QIcon(IMAGE_EXPAND), tr("Expand all"), ui->peerTreeWidget, SLOT(expandAll()));
-    contextMenu->addAction(QIcon(IMAGE_COLLAPSE), tr("Collapse all"), ui->peerTreeWidget, SLOT(collapseAll()));
+    contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_EXPAND), tr("Expand all"), ui->peerTreeWidget, SLOT(expandAll()));
+    contextMenu->addAction(FilesDefs::getIconFromQtResourcePath(IMAGE_COLLAPSE), tr("Collapse all"), ui->peerTreeWidget, SLOT(collapseAll()));
 
     contextMenu = ui->peerTreeWidget->createStandardContextMenu(contextMenu);
 
@@ -717,8 +720,8 @@ void FriendList::insertPeers()
 
                 groupItem->setChildIndicatorPolicy(QTreeWidgetItem::DontShowIndicatorWhenChildless);
                 groupItem->setTextAlignment(COLUMN_NAME, Qt::AlignLeft | Qt::AlignVCenter);
-                groupItem->setIcon(COLUMN_NAME, QIcon(IMAGE_GROUP24));
-                groupItem->setForeground(COLUMN_NAME, QBrush(textColorGroup()));
+                groupItem->setIcon(COLUMN_NAME, FilesDefs::getIconFromQtResourcePath(IMAGE_GROUP24));
+                groupItem->setData(COLUMN_NAME, Qt::ForegroundRole, textColorGroup());
 
                 /* used to find back the item */
                 QString strID = QString::fromStdString(groupInfo->id.toStdString());
@@ -1007,13 +1010,14 @@ void FriendList::insertPeers()
                     sslItem->setHidden(false);
                     gpg_connected = true;
 
-                    sslOverlayIcon = QPixmap(StatusDefs::imageStatus(bestRSState));
+                    sslOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(StatusDefs::imageStatus(bestRSState));
 
                     connectStateString = StatusDefs::name(rsState);
 
                     if (rsState == 0) {
                         sslFont.setBold(true);
                         sslColor = mTextColorStatus[RS_STATUS_ONLINE];
+
                     } else {
                         sslFont = StatusDefs::font(rsState);
                         sslColor = mTextColorStatus[rsState];
@@ -1024,9 +1028,9 @@ void FriendList::insertPeers()
                     peerState = PEER_STATE_AVAILABLE;
 
                     if (sslDetail.connectState) {
-                        sslOverlayIcon = QPixmap(":/images/connect_creating.png");
+                        sslOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(":/images/connect_creating.png");
                     } else {
-                        sslOverlayIcon = QPixmap(StatusDefs::imageStatus(RS_STATUS_ONLINE));
+                        sslOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(StatusDefs::imageStatus(RS_STATUS_ONLINE));
                     }
 
                     connectStateString = StatusDefs::name(RS_STATUS_ONLINE);
@@ -1037,9 +1041,9 @@ void FriendList::insertPeers()
                     peerState = PEER_STATE_OFFLINE;
                     sslItem->setHidden(mHideUnconnected);
                     if (sslDetail.connectState) {
-                        sslOverlayIcon = QPixmap(":/images/connect_creating.png");
+                        sslOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(":/images/connect_creating.png");
                     } else {
-                        sslOverlayIcon = QPixmap(StatusDefs::imageStatus(RS_STATUS_OFFLINE));
+                        sslOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(StatusDefs::imageStatus(RS_STATUS_OFFLINE));
                     }
 
                     connectStateString = StatusDefs::connectStateWithoutTransportTypeString(sslDetail);
@@ -1099,7 +1103,7 @@ void FriendList::insertPeers()
 
                 if (std::find(privateChatIds.begin(), privateChatIds.end(), sslDetail.id) != privateChatIds.end()) {
                     // private chat is available
-                    sslOverlayIcon = QPixmap(":/images/chat.png");
+                    sslOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(":/images/chat.png");
                     gpg_hasPrivateChat = true;
                 }
                 sslItem->setIcon(COLUMN_NAME, createAvatar(sslAvatar, sslOverlayIcon));
@@ -1110,8 +1114,8 @@ void FriendList::insertPeers()
                 for (int i = 0; i < columnCount; ++i) {
                     sslItem->setData(i, ROLE_SORT_STATE, peerState);
 
-                    sslItem->setTextColor(i, sslColor);
-                    sslItem->setFont(i, sslFont);
+                    sslItem->setData(i, Qt::ForegroundRole, sslColor);
+                    sslItem->setData(i, Qt::FontRole, sslFont);
                 }
             }
 
@@ -1138,7 +1142,7 @@ void FriendList::insertPeers()
                 gpgFont = StatusDefs::font(bestRSState);
 
                 if (showInfoAtGpgItem) {
-                    gpgOverlayIcon = QPixmap(StatusDefs::imageStatus(bestRSState));
+                    gpgOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(StatusDefs::imageStatus(bestRSState));
 
                     if (mShowState) {
                         gpgText = StatusDefs::name(bestRSState);
@@ -1164,7 +1168,7 @@ void FriendList::insertPeers()
                         gpgText += tr("Available");
                     }
 
-                    gpgOverlayIcon = QPixmap(IMAGE_AVAILABLE);
+                    gpgOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(IMAGE_AVAILABLE);
                 }
             } else {
                 bestPeerState = PEER_STATE_OFFLINE;
@@ -1178,15 +1182,15 @@ void FriendList::insertPeers()
                         gpgText += StatusDefs::name(RS_STATUS_OFFLINE);
                     }
 
-                    gpgOverlayIcon = QPixmap(StatusDefs::imageStatus(RS_STATUS_OFFLINE));
+                    gpgOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(StatusDefs::imageStatus(RS_STATUS_OFFLINE));
                 }
             }
 
             if (gpg_hasPrivateChat) {
-                gpgOverlayIcon = QPixmap(":/images/chat.png");
+                gpgOverlayIcon = FilesDefs::getPixmapFromQtResourcePath(":/images/chat.png");
             }
 
-            gpgItem->setIcon(COLUMN_NAME, createAvatar(bestAvatar.isNull() ? QPixmap(AVATAR_DEFAULT_IMAGE) : bestAvatar, gpgOverlayIcon));
+            gpgItem->setIcon(COLUMN_NAME, createAvatar(bestAvatar.isNull() ? FilesDefs::getPixmapFromQtResourcePath(AVATAR_DEFAULT_IMAGE) : bestAvatar, gpgOverlayIcon));
 
             /* Create or get gpg label */
             ElidedLabel *gpgNameLabel = NULL;
@@ -1222,8 +1226,8 @@ void FriendList::insertPeers()
             for (int i = 0; i < columnCount; ++i) {
                 gpgItem->setData(i, ROLE_SORT_STATE, bestPeerState);
 
-                gpgItem->setTextColor(i, gpgColor);
-                gpgItem->setFont(i, gpgFont);
+                gpgItem->setData(i, Qt::ForegroundRole, gpgColor);
+                gpgItem->setData(i, Qt::FontRole, gpgFont);
             }
 
             if (openPeers.find(gpgId) != openPeers.end()) {
@@ -1965,7 +1969,7 @@ bool FriendList::exportFriendlist(QString &fileName)
 /**
  * @brief helper function to show a message box
  */
-void showXMLParsingError()
+static void showXMLParsingError()
 {
     // show error to user
     QMessageBox mbox;
