@@ -18,6 +18,7 @@
  *                                                                             *
  *******************************************************************************/
 
+#include "gui/common/FilesDefs.h"
 #include "RsCollectionDialog.h"
 
 #include "RsCollection.h"
@@ -63,7 +64,9 @@ class FSMSortFilterProxyModel : public QSortFilterProxyModel
 {
 public:
 	FSMSortFilterProxyModel( QObject *parent) : QSortFilterProxyModel(parent)
-	{}
+    {
+        setDynamicSortFilter(false);	// essential to avoid random crashes
+    }
 
 protected:
 	virtual bool lessThan(const QModelIndex &left, const QModelIndex &right) const
@@ -142,7 +145,7 @@ RsCollectionDialog::RsCollectionDialog(const QString& collectionFileName
 	setWindowTitle(QString("%1 - %2").arg(windowTitle()).arg(QFileInfo(_fileName).completeBaseName()));
 	
 	
-	ui.headerFrame->setHeaderImage(QPixmap(":/images/library64.png"));
+    ui.headerFrame->setHeaderImage(FilesDefs::getPixmapFromQtResourcePath(":/icons/collections.png"));
 
 	if(creation)
 	{
@@ -200,7 +203,6 @@ RsCollectionDialog::RsCollectionDialog(const QString& collectionFileName
 	connect(_dirModel, SIGNAL(directoryLoaded(QString)), this, SLOT(directoryLoaded(QString)));
 
 	_tree_proxyModel = new FSMSortFilterProxyModel(this);
-	_tree_proxyModel->setDynamicSortFilter(true);
 	_tree_proxyModel->setSourceModel(_dirModel);
 	_tree_proxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
 	_tree_proxyModel->setSortRole(Qt::DisplayRole);
@@ -254,7 +256,7 @@ void RsCollectionDialog::openDestinationDirectoryMenu()
 		contextMnu.addAction(QString::fromUtf8((*it).filename.c_str()), this, SLOT(setDestinationDirectory()))->setData(QString::fromUtf8( (*it).filename.c_str() ) ) ;
 	}
 
-	contextMnu.addAction( QIcon(IMAGE_SEARCH),tr("Specify..."),this,SLOT(chooseDestinationDirectory()));
+    contextMnu.addAction( FilesDefs::getIconFromQtResourcePath(IMAGE_SEARCH),tr("Specify..."),this,SLOT(chooseDestinationDirectory()));
 
 	contextMnu.exec(QCursor::pos()) ;
 }
@@ -537,8 +539,9 @@ bool RsCollectionDialog::addChild(QTreeWidgetItem* parent, const std::vector<Col
 
 			if (colFileInfo.filename_has_wrong_characters)
 			{
+				//TODO (Phenom): Add qproperty for these text colors in stylesheets
 				wrong_chars = true ;
-				item->setTextColor(COLUMN_FILE, QColor(255,80,120)) ;
+				item->setData(COLUMN_FILE, Qt::ForegroundRole, QColor(255,80,120)) ;
 			}
 
 			if (parentsFounds.empty()) {
@@ -639,7 +642,7 @@ void RsCollectionDialog::updateSizes()
 	uint64_t total_size = 0 ;
 	uint32_t total_count = 0 ;
 
-	for(uint32_t i=0;i<ui._fileEntriesTW->topLevelItemCount();++i)
+	for(int i=0;i<ui._fileEntriesTW->topLevelItemCount();++i)
 	{
 		total_size  += ui._fileEntriesTW->topLevelItem(i)->data(COLUMN_SIZE ,ROLE_SELSIZE ).toULongLong();
 		total_count += ui._fileEntriesTW->topLevelItem(i)->data(COLUMN_FILEC,ROLE_SELFILEC).toULongLong();
