@@ -33,11 +33,14 @@
 #include <QUrlQuery>
 #endif
 
+#include "gui/common/FilesDefs.h"
+#include "gui/RetroShareLink.h"
 #include "gui/settings/rsharesettings.h"
 #include "util/misc.h"
 #include "ConnectFriendWizard.h"
 #include "ui_ConnectFriendWizard.h"
 #include "gui/common/PeerDefs.h"
+#include "gui/connect/ConfCertDialog.h"
 #include "gui/notifyqt.h"
 #include "gui/common/GroupDefs.h"
 #include "gui/msgs/MessageComposer.h"
@@ -78,39 +81,37 @@ ConnectFriendWizard::ConnectFriendWizard(QWidget *parent) :
 {
 	ui->setupUi(this);
 
+    error = 0;
 	mTitleFontSize = 0; // Standard
 	mTitleFontWeight = 0; // Standard
-
-    // (csoler) I'm hiding this, since it is not needed anymore with the new Home page.
-    ui->userFrame->hide();
 	
-	ui->userFileFrame->hide(); // in homepage dropmenu now
+//	ui->userFileFrame->hide(); // in homepage dropmenu now
 
 // this define comes from Qt example. I don't have mac, so it wasn't tested
 #ifndef Q_OS_MAC
 	setWizardStyle(ModernStyle);
 #endif
 
-	setStartId(Page_Intro);
+	setStartId(Page_Text);
 
 // at this moment I don't know, what information should be in help
 //	setOption(HaveHelpButton, true);
 //	connect(this, SIGNAL(helpRequested()), this, SLOT(showHelp()));
 
-	setPixmap(QWizard::LogoPixmap, QPixmap(":/icons/invite64.png"));
+    setPixmap(QWizard::LogoPixmap, FilesDefs::getPixmapFromQtResourcePath(":/icons/invite64.png"));
 
 // we have no good pictures for watermarks
-//	setPixmap(QWizard::WatermarkPixmap, QPixmap(":/images/connectFriendWatermark.png"));
+//	setPixmap(QWizard::WatermarkPixmap, FilesDefs::getPixmapFromQtResourcePath(":/images/connectFriendWatermark.png"));
 
 	/* register global fields */
 	ui->ErrorMessagePage->registerField("errorMessage", ui->messageLabel, "text");
 
 	/* disable not used pages */
-	ui->foffRadioButton->hide();
-	ui->rsidRadioButton->hide();
+	//ui->foffRadioButton->hide();
+	//ui->rsidRadioButton->hide();
 	
-	ui->cp_Label->hide();
-	ui->requestinfolabel->hide();
+	ui->info_Label_FrdReq->hide();
+	ui->info_Label_Request->hide();
 	
     connect(ui->acceptNoSignGPGCheckBox,SIGNAL(toggled(bool)), ui->_options_GB,SLOT(setEnabled(bool))) ;
     connect(ui->addKeyToKeyring_CB,SIGNAL(toggled(bool)), ui->acceptNoSignGPGCheckBox,SLOT(setChecked(bool))) ;
@@ -129,6 +130,8 @@ ConnectFriendWizard::ConnectFriendWizard(QWidget *parent) :
     body += "\n" + GetStartedDialog::GetCutBelowText();
 	body += "\n\n" + QString::fromUtf8(rsPeers->GetRetroshareInvite().c_str());
 	
+    mIsShortInvite = false;
+
 	std::string advsetting;
 	if(rsConfig->getConfigurationOption(RS_CONFIG_ADVANCED, advsetting) && (advsetting == "YES"))
 	{
@@ -136,7 +139,6 @@ ConnectFriendWizard::ConnectFriendWizard(QWidget *parent) :
 	}
 	else
 	{
-		ui->userFrame->hide(); // certificates page - top half with own cert and it's functions
 		ui->cp_Frame->hide(); // Advanced options - key sign, whitelist, direct source ...
 		AdvancedVisible=false;
 		ui->trustLabel->hide();
@@ -147,15 +149,11 @@ ConnectFriendWizard::ConnectFriendWizard(QWidget *parent) :
 	switch (rsFiles->filePermDirectDL())
 	{
 		case RS_FILE_PERM_DIRECT_DL_YES:
-			ui->_direct_transfer_CB->setIcon(QIcon(":/icons/warning_yellow_128.png"));
-			ui->_direct_transfer_CB->setToolTip(ui->_direct_transfer_CB->toolTip().append(tr("\nWarning: In your File-Transfer option, you select allow direct download to Yes.")));
-			ui->_direct_transfer_CB_2->setIcon(QIcon(":/icons/warning_yellow_128.png"));
+            ui->_direct_transfer_CB_2->setIcon(FilesDefs::getIconFromQtResourcePath(":/icons/warning_yellow_128.png"));
 			ui->_direct_transfer_CB_2->setToolTip(ui->_direct_transfer_CB_2->toolTip().append(tr("\nWarning: In your File-Transfer option, you select allow direct download to Yes.")));
 			break ;
 		case RS_FILE_PERM_DIRECT_DL_NO:
-			ui->_direct_transfer_CB->setIcon(QIcon(":/icons/warning_yellow_128.png"));
-			ui->_direct_transfer_CB->setToolTip(ui->_direct_transfer_CB->toolTip().append(tr("\nWarning: In your File-Transfer option, you select allow direct download to No.")));
-			ui->_direct_transfer_CB_2->setIcon(QIcon(":/icons/warning_yellow_128.png"));
+            ui->_direct_transfer_CB_2->setIcon(FilesDefs::getIconFromQtResourcePath(":/icons/warning_yellow_128.png"));
 			ui->_direct_transfer_CB_2->setToolTip(ui->_direct_transfer_CB_2->toolTip().append(tr("\nWarning: In your File-Transfer option, you select allow direct download to No.")));
 			break ;
 
@@ -277,7 +275,6 @@ QString ConnectFriendWizard::getErrorString(uint32_t error_code)
 void ConnectFriendWizard::setCertificate(const QString &certificate, bool friendRequest)
 {
 	if (certificate.isEmpty()) {
-		setStartId(Page_Intro);
 		return;
 	}
 
@@ -299,17 +296,43 @@ void ConnectFriendWizard::setCertificate(const QString &certificate, bool friend
 		{
 			mCertificate = certificate.toUtf8().constData();
 
-			// Cyril: I disabled this because it seems to be not used anymore.
-			//setStartId(friendRequest ? Page_FriendRequest : Page_Conclusion);
 			setStartId(Page_Conclusion);
 			if (friendRequest){
-				ui->cp_Label->show();
-				ui->requestinfolabel->show();
+				ui->info_Label_FrdReq->show();
+				ui->info_Label_Request->show();
 				setTitleText(ui->ConclusionPage, tr("Friend request"));
 				ui->ConclusionPage->setSubTitle(tr("Details about the request"));
+				setButtonText(QWizard::FinishButton	, tr("Accept"));
 			}
 		}
-    } else {
+    }
+    else if(rsPeers->parseShortInvite(certificate.toUtf8().constData(),peerDetails,cert_load_error_code))
+    {
+        mIsShortInvite = true;
+
+		if(peerDetails.id == rsPeers->getOwnId())
+		{
+			setField("errorMessage", tr("This is your own certificate! You would not want to make friend with yourself. Wouldn't you?") ) ;
+			error = false;
+			setStartId(Page_ErrorMessage);
+		}
+		else
+		{
+			mCertificate = certificate.toUtf8().constData();
+
+			setStartId(Page_Conclusion);
+
+			if (friendRequest){
+				ui->info_Label_FrdReq->show();
+				ui->info_Label_Request->show();
+				setTitleText(ui->ConclusionPage, tr("Friend request"));
+				ui->ConclusionPage->setSubTitle(tr("Details about the request"));
+				setButtonText(QWizard::FinishButton	, tr("Accept"));
+			}
+		}
+	}
+    else
+    {
 		// error message
 		setField("errorMessage", tr("Certificate Load Failed") + ": \n\n" + getErrorString(cert_load_error_code)) ;
 		setStartId(Page_ErrorMessage);
@@ -318,23 +341,49 @@ void ConnectFriendWizard::setCertificate(const QString &certificate, bool friend
 
 void ConnectFriendWizard::setGpgId(const RsPgpId &gpgId, const RsPeerId &sslId, bool friendRequest)
 {
-	if (!rsPeers->getGPGDetails(gpgId, peerDetails)) {
-		setField("errorMessage", tr("Cannot get peer details of PGP key %1").arg(QString::fromStdString(gpgId.toStdString())));
+	if(sslId == rsPeers->getOwnId())
+	{
+		setField("errorMessage", tr("This is your own certificate! You would not want to make friend with yourself. Wouldn't you?") ) ;
 		setStartId(Page_ErrorMessage);
-		return;
+		error = false;
 	}
 
-	/* Set ssl id when available */
-	peerDetails.id = sslId;
+	if (!rsPeers->getGPGDetails(gpgId, peerDetails))
+    {
+		mIsShortInvite = true;
 
-    //setStartId(friendRequest ? Page_FriendRequest : Page_Conclusion);
-    setStartId(Page_Conclusion);
-    if (friendRequest){
-    ui->cp_Label->show();
-    ui->requestinfolabel->show();
-    setTitleText(ui->ConclusionPage,tr("Friend request"));
-    ui->ConclusionPage->setSubTitle(tr("Details about the request"));
-    }
+        peerDetails.id = sslId;
+        peerDetails.gpg_id = gpgId;
+        peerDetails.skip_pgp_signature_validation = true;
+
+		mCertificate.clear();
+
+		setStartId(Page_Conclusion);
+
+		if (friendRequest){
+			ui->info_Label_FrdReq->show();
+			ui->info_Label_Request->show();
+			setTitleText(ui->ConclusionPage, tr("Friend request"));
+			ui->ConclusionPage->setSubTitle(tr("Details about the request"));
+			setButtonText(QWizard::FinishButton	, tr("Accept"));
+		}
+	}
+    else
+	{
+		/* Set ssl id when available */
+		peerDetails.id = sslId;
+		mIsShortInvite = false;
+
+		//setStartId(friendRequest ? Page_FriendRequest : Page_Conclusion);
+		setStartId(Page_Conclusion);
+		if (friendRequest){
+			ui->info_Label_FrdReq->show();
+			ui->info_Label_Request->show();
+			setTitleText(ui->ConclusionPage,tr("Friend request"));
+			ui->ConclusionPage->setSubTitle(tr("Details about the request"));
+			setButtonText(QWizard::FinishButton	, tr("Accept"));
+		}
+	}
 }
 
 ConnectFriendWizard::~ConnectFriendWizard()
@@ -361,18 +410,10 @@ static void fillGroups(ConnectFriendWizard *wizard, QComboBox *comboBox, const Q
 void ConnectFriendWizard::initializePage(int id)
 {
 	switch ((Page) id) {
-	case Page_Intro:
-		ui->textRadioButton->setChecked(true);
-		break;
 	case Page_Text:
-		connect(ui->userCertHelpButton, SIGNAL( clicked()), this, SLOT(showHelpUserCert()));
-		connect(ui->userCertIncludeSignaturesButton, SIGNAL(clicked()), this, SLOT(toggleSignatureState()));
-		connect(ui->userCertOldFormatButton, SIGNAL(clicked()), this, SLOT(toggleFormatState()));
-		connect(ui->userCertCopyButton, SIGNAL(clicked()), this, SLOT(copyCert()));
+
 		connect(ui->userCertPasteButton, SIGNAL(clicked()), this, SLOT(pasteCert()));
 		connect(ui->userCertOpenButton, SIGNAL(clicked()), this, SLOT(openCert()));
-		connect(ui->userCertSaveButton, SIGNAL(clicked()), this, SLOT(saveCert()));
-		connect(ui->userCertMailButton, SIGNAL(clicked()), this, SLOT(runEmailClient()));
 		connect(ui->friendCertEdit, SIGNAL(textChanged()), this, SLOT(friendCertChanged()));
 
 		cleanfriendCertTimer = new QTimer(this);
@@ -380,62 +421,11 @@ void ConnectFriendWizard::initializePage(int id)
 		cleanfriendCertTimer->setInterval(1000); // 1 second
 		connect(cleanfriendCertTimer, SIGNAL(timeout()), this, SLOT(cleanFriendCert()));
 
-		ui->userCertOldFormatButton->setChecked(false); 
-		ui->userCertOldFormatButton->hide() ;
-
-		toggleFormatState(true);
-		toggleSignatureState(false);
-		updateOwnCert();
-
 		cleanFriendCert();
 
-		break;
-	case Page_Cert:
-		connect(ui->userFileCreateButton, SIGNAL(clicked()), this, SLOT(generateCertificateCalled()));
-		connect(ui->friendFileNameOpenButton, SIGNAL(clicked()), this, SLOT(loadFriendCert()));
-
-		ui->friendFileNameEdit->setAcceptFile(true);
-
-		ui->CertificatePage->registerField("friendCertificateFile*", ui->friendFileNameEdit);
-		break;
-	case Page_Foff:
-		ui->userSelectionCB->addItem(tr("Any peer I've not signed"));
-		ui->userSelectionCB->addItem(tr("Friends of my friends who already trust me"));
-		ui->userSelectionCB->addItem(tr("Signed peers showing as denied"));
-
-		ui->selectedPeersTW->setHorizontalHeaderItem(0, new QTableWidgetItem(tr("")));
-		ui->selectedPeersTW->setHorizontalHeaderItem(1, new QTableWidgetItem(tr("Peer name")));
-		ui->selectedPeersTW->setHorizontalHeaderItem(2, new QTableWidgetItem(tr("Also signed by")));
-		ui->selectedPeersTW->setHorizontalHeaderItem(3, new QTableWidgetItem(tr("Peer id")));
-
-		connect(ui->makeFriendButton, SIGNAL(clicked()), this, SLOT(signAllSelectedUsers()));
-		connect(ui->userSelectionCB, SIGNAL(activated(int)), this, SLOT(updatePeersList(int)));
-
-		updatePeersList(ui->userSelectionCB->currentIndex());
-
-		ui->FofPage->setComplete(false);
-		break;
-	case Page_Rsid:
-		ui->RsidPage->registerField("friendRSID*", ui->friendRsidEdit);
 		break;
 	case Page_WebMail:
 
-	case Page_Email:
-		{
-		ui->EmailPage->registerField("addressEdit*", ui->addressEdit);
-		ui->EmailPage->registerField("subjectEdit*", ui->subjectEdit);
-
-		ui->subjectEdit->setText(tr("RetroShare Invitation"));
-		ui->inviteTextEdit->setPlainText(GetStartedDialog::GetInviteText());
-		
-		QString body = ui->inviteTextEdit->toPlainText();
-
-		body += "\n" + GetStartedDialog::GetCutBelowText();
-		body += "\n\n" + QString::fromUtf8(rsPeers->GetRetroshareInvite().c_str());
-
-		ui->inviteTextEdit->setPlainText(body);
-		}
-		break;
 	case Page_ErrorMessage:
 		break;
 	case Page_Conclusion:
@@ -460,8 +450,9 @@ void ConnectFriendWizard::initializePage(int id)
 			}
 
         sockaddr_storage addr ;
-
+#ifdef DEBUG_FRIENDWIZARD
 		std::cerr << "Cert IP = " << peerDetails.extAddr << std::endl;
+#endif
 
         if(sockaddr_storage_ipv4_aton(addr,peerDetails.extAddr.c_str()) && sockaddr_storage_isValidNet(addr))
         {
@@ -543,10 +534,10 @@ void ConnectFriendWizard::initializePage(int id)
 				}
 			}
 
-			ui->cp_Label->setText(tr("You have a friend request from") + " " + QString::fromUtf8(peerDetails.name.c_str()));
+			ui->info_Label_FrdReq->setText(tr("You have a friend request from") + " " + QString::fromUtf8(peerDetails.name.c_str()));
 			ui->nameEdit->setText(QString::fromUtf8(peerDetails.name.c_str()));
 			ui->trustEdit->setText(trustString);
-			ui->emailEdit->setText(QString::fromUtf8(peerDetails.email.c_str()));
+			ui->profileIdEdit->setText(QString::fromStdString(peerDetails.gpg_id.toStdString()));
 			QString loc = QString::fromUtf8(peerDetails.location.c_str());
 			if (!loc.isEmpty())
 			{
@@ -563,9 +554,38 @@ void ConnectFriendWizard::initializePage(int id)
 			}
 
 			ui->nodeEdit->setText(loc);
-			ui->ipEdit->setText(QString::fromStdString(peerDetails.isHiddenNode ? peerDetails.hiddenNodeAddress : peerDetails.extAddr));
+
+            QString s;
+
+            if(peerDetails.isHiddenNode)
+                s += QString::fromStdString(peerDetails.hiddenNodeAddress);
+            else
+            {
+                if(peerDetails.localAddr!="0.0.0.0")// This is not so nice, but because we deal we string there's no way
+                    s += QString::fromStdString(peerDetails.localAddr)+":"+QString::number(peerDetails.localPort);		// to ask about if the ip is null. We really need a proper IP class.
+
+                if(peerDetails.extAddr!="0.0.0.0")
+                {
+                    if(!s.isNull()) s += " / " ;
+                    s += QString::fromStdString(peerDetails.extAddr) + ":"+QString::number(peerDetails.extPort);
+                }
+
+                if(!peerDetails.dyndns.empty())
+                    s += " (" + QString::fromStdString(peerDetails.dyndns) + ")" ;
+            }
+            ui->ipEdit->setText(s);
 			ui->signersEdit->setPlainText(ts);
 
+			ui->knownIpLabel->setHidden(peerDetails.ipAddressList.empty());
+			ui->knownIpEdit->setHidden(peerDetails.ipAddressList.empty());
+			{
+				QString ipList;
+				for(auto& it : peerDetails.ipAddressList)
+					ipList.append(QString::fromStdString(it) + "\n");
+
+				ui->knownIpEdit->setPlainText(ipList);
+			}
+
 			fillGroups(this, ui->groupComboBox, groupId);
 			
 			if(peerDetails.isHiddenNode)
@@ -576,88 +596,27 @@ void ConnectFriendWizard::initializePage(int id)
 				ui->_addIPToWhiteList_ComboBox_2->addItem("(Hidden node)") ;
 				int S = QFontMetricsF(ui->ipEdit->font()).height() ;
 				ui->ipEdit->setToolTip("This is a Hidden node - you need tor/i2p proxy to connect");
-				ui->ipLabel->setPixmap(QPixmap(":/images/anonymous_128_blue.png").scaledToHeight(S*2,Qt::SmoothTransformation));
+                ui->ipLabel->setPixmap(FilesDefs::getPixmapFromQtResourcePath(":/images/anonymous_128_blue.png").scaledToHeight(S*2,Qt::SmoothTransformation));
 				ui->ipLabel->setToolTip("This is a Hidden node - you need tor/i2p proxy to connect");
 			}
-
-			if(peerDetails.email.empty())
+			if(mIsShortInvite)
 			{
-				ui->emailLabel->hide(); // is it ever used?
-				ui->emailEdit->hide();
+				if(ui->nameEdit->text().isEmpty())
+					ui->nameEdit->setText(QString::fromUtf8(peerDetails.name.c_str()));
+				ui->addKeyToKeyring_CB->setChecked(false);
+				ui->addKeyToKeyring_CB->setEnabled(false);
+				ui->signersEdit->hide();
+				ui->signersLabel->hide();
+				ui->signGPGCheckBox->setChecked(false);
+				ui->signGPGCheckBox->setEnabled(false);
+				ui->acceptNoSignGPGCheckBox->setChecked(true);
+				ui->acceptNoSignGPGCheckBox->setEnabled(false);
 			}
+
 			ui->ipEdit->setTextInteractionFlags(Qt::TextSelectableByMouse);
 
 		}
 		break;
-	case Page_FriendRequest:
-		{
-			std::cerr << "Friend request page id : " << peerDetails.id << "; gpg_id : " << peerDetails.gpg_id << std::endl;
-
-			ui->fr_avatar->setFrameType(AvatarWidget::NORMAL_FRAME);
-			setPixmap(QWizard::LogoPixmap, QPixmap(":/images/user/user_request48.png"));
-
-			ui->fr_signGPGCheckBox->setChecked(false);
-
-			//set the radio button to sign the GPG key
-			if (peerDetails.accept_connection && !peerDetails.ownsign) {
-				//gpg key connection is already accepted, don't propose to accept it again
-				ui->fr_acceptNoSignGPGCheckBox->hide();
-				ui->fr_signGPGCheckBox->show();
-				ui->fr_acceptNoSignGPGCheckBox->setChecked(false);
-			}
-			if (!peerDetails.accept_connection && peerDetails.ownsign) {
-				//gpg key is already signed, don't propose to sign it again
-				ui->fr_acceptNoSignGPGCheckBox->setChecked(true);
-				ui->fr_signGPGCheckBox->hide();
-			}
-			if (!peerDetails.accept_connection && !peerDetails.ownsign) {
-				ui->fr_acceptNoSignGPGCheckBox->setChecked(true);
-				ui->fr_signGPGCheckBox->show();
-				ui->fr_acceptNoSignGPGCheckBox->show();
-			}
-			if (peerDetails.accept_connection && peerDetails.ownsign) {
-				ui->fr_acceptNoSignGPGCheckBox->setChecked(false);
-				ui->fr_acceptNoSignGPGCheckBox->hide();
-				ui->fr_signGPGCheckBox->hide();
-			}
-
-			ui->fr_nameEdit->setText(QString::fromUtf8(peerDetails.name.c_str()));
-			ui->fr_emailEdit->setText(QString::fromUtf8(peerDetails.email.c_str()));
-
-			QString loc = QString::fromUtf8(peerDetails.location.c_str());
-			if (!loc.isEmpty())
-			{
-				loc += " (";
-				loc += QString::fromStdString(peerDetails.id.toStdString());
-				loc += ")";
-			}
-			else
-			{
-				if (!peerDetails.id.isNull())
-				{
-					loc += QString::fromStdString(peerDetails.id.toStdString());
-				}
-			}
-
-			ui->fr_nodeEdit->setText(loc);
-			
-			ui->fr_InfoTopLabel->setText(tr("You have a friend request from") + " " + QString::fromUtf8(peerDetails.name.c_str()));
-
-			fillGroups(this, ui->fr_groupComboBox, groupId);
-		}
-		break;
-	case Page_FriendRecommendations:
-		ui->frec_recommendList->setHeaderText(tr("Recommend friends"));
-		ui->frec_recommendList->setModus(FriendSelectionWidget::MODUS_CHECK);
-		ui->frec_recommendList->setShowType(FriendSelectionWidget::SHOW_GROUP | FriendSelectionWidget::SHOW_SSL);
-		ui->frec_recommendList->start();
-
-		ui->frec_toList->setHeaderText(tr("To"));
-		ui->frec_toList->setModus(FriendSelectionWidget::MODUS_CHECK);
-		ui->frec_toList->start();
-
-		ui->frec_messageEdit->setText(MessageComposer::recommendMessage());
-		break;
 	}
 }
 
@@ -691,10 +650,9 @@ static void sendMail(QString sAddress, QString sSubject, QString sBody)
 
 bool ConnectFriendWizard::validateCurrentPage()
 {
-	error = true;
+	error = false;
 
 	switch ((Page) currentId()) {
-	case Page_Intro:
 	case Page_WebMail:
 		break;
 	case Page_Text:
@@ -702,7 +660,8 @@ bool ConnectFriendWizard::validateCurrentPage()
 			std::string certstr = ui->friendCertEdit->toPlainText().toUtf8().constData();
 			uint32_t cert_load_error_code;
 
-			if (rsPeers->loadDetailsFromStringCert(certstr, peerDetails, cert_load_error_code)) {
+			if (rsPeers->loadDetailsFromStringCert(certstr, peerDetails, cert_load_error_code) || rsPeers->parseShortInvite(certstr,peerDetails,cert_load_error_code))
+            {
 				mCertificate = certstr;
 #ifdef FRIEND_WIZARD_DEBUG
 				std::cerr << "ConnectFriendWizard got id : " << peerDetails.id << "; gpg_id : " << peerDetails.gpg_id << std::endl;
@@ -711,121 +670,20 @@ bool ConnectFriendWizard::validateCurrentPage()
 				if(peerDetails.id == rsPeers->getOwnId())
 				{
 					setField("errorMessage", tr("This is your own certificate! You would not want to make friend with yourself. Wouldn't you?") ) ;
-					error = false;
+					error = true;
 				}
 
 				break;
 			}
 			// error message
 			setField("errorMessage", tr("Certificate Load Failed") + ": \n\n" + getErrorString(cert_load_error_code)) ;
-			error = false;
+			error = true;
 			break;
 		}
-	case Page_Cert:
-		{
-			QString fn = ui->friendFileNameEdit->text();
-			if (QFile::exists(fn)) {
-				//Todo: move read from file to p3Peers::loadCertificateFromFile
-
-				// read from file
-				std::string certstr;
-				QFile CertFile(fn);
-				if (CertFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
-					certstr = QString(CertFile.readAll()).toStdString();
-					CertFile.close();
-				}
-
-				if (certstr.empty()) {
-					setField("errorMessage", QString(tr("Certificate Load Failed:can't read from file %1")).arg(fn+" ") );
-					error = false;
-					break;
-				}
-
-				uint32_t cert_error_code;
-				if (rsPeers->loadDetailsFromStringCert(certstr, peerDetails, cert_error_code)) {
-					mCertificate = certstr;
-#ifdef FRIEND_WIZARD_DEBUG
-					std::cerr << "ConnectFriendWizard got id : " << peerDetails.id << "; gpg_id : " << peerDetails.gpg_id << std::endl;
-#endif
-
-					if(peerDetails.id == rsPeers->getOwnId())
-					{
-						setField("errorMessage", tr("This is your own certificate! You would not want to make friend with yourself. Wouldn't you?") ) ;
-						error = false;
-					}
-				} else {
-					setField("errorMessage", QString(tr("Certificate Load Failed:something is wrong with %1")).arg(fn) + " : " + getErrorString(cert_error_code));
-					error = false;
-				}
-			} else {
-				setField("errorMessage", QString(tr("Certificate Load Failed:file %1 not found")).arg(fn));
-				error = false;
-			}
-			break;
-		}
-	case Page_Foff:
-		break;
-	case Page_Rsid:
-		{
-			QString rsidstring = ui->friendRsidEdit->text();
-
-			if (rsidstring.isEmpty()) {
-				return false;
-			}
-
-			// search for peer id in string
-			RsPeerId rsidstr = PeerDefs::idFromRsid(rsidstring, false);
-
-			if (rsidstr.isNull() || !rsPeers->getPeerDetails(rsidstr, peerDetails)) {
-				setField("errorMessage", tr("This Peer %1 is not available in your Network").arg(rsidstring));
-				error = false;
-			}
-			break;
-		}
-	case Page_Email:
-		{
-			QString mailaddresses = ui->addressEdit->text();
-			if (mailaddresses.isEmpty()) {
-				return false;
-			}
-
-			QString body = ui->inviteTextEdit->toPlainText();
-
-			body += "\n" + GetStartedDialog::GetCutBelowText();
-			body += "\n\n" + QString::fromUtf8(rsPeers->GetRetroshareInvite().c_str());
-
-			sendMail (mailaddresses, ui->subjectEdit->text(), body);
-		}
-		break;
 	case Page_ErrorMessage:
 		break;
 	case Page_Conclusion:
 		break;
-	case Page_FriendRequest:
-		break;
-	case Page_FriendRecommendations:
-		{
-            std::set<RsPeerId> recommendIds;
-            ui->frec_recommendList->selectedIds<RsPeerId,FriendSelectionWidget::IDTYPE_SSL>(recommendIds, false);
-
-			if (recommendIds.empty()) {
-				QMessageBox::warning(this, "RetroShare", tr("Please select at least one friend for recommendation."), QMessageBox::Ok, QMessageBox::Ok);
-				return false;
-			}
-
-            std::set<RsPeerId> toIds;
-            ui->frec_toList->selectedIds<RsPeerId,FriendSelectionWidget::IDTYPE_SSL>(toIds, false);
-
-			if (toIds.empty()) {
-				QMessageBox::warning(this, "RetroShare", tr("Please select at least one friend as recipient."), QMessageBox::Ok, QMessageBox::Ok);
-				return false;
-			}
-
-            std::set<RsPeerId>::iterator toId;
-			for (toId = toIds.begin(); toId != toIds.end(); ++toId) {
-				MessageComposer::recommendFriend(recommendIds, *toId, ui->frec_messageEdit->toHtml(), true);
-			}
-		}
 	}
 
 	return true;
@@ -834,26 +692,10 @@ bool ConnectFriendWizard::validateCurrentPage()
 int ConnectFriendWizard::nextId() const
 {
 	switch ((Page) currentId()) {
-	case Page_Intro:
-		if (ui->textRadioButton->isChecked()) return Page_Text;
-		if (ui->certRadioButton->isChecked()) return Page_Cert;
-		if (ui->foffRadioButton->isChecked()) return Page_Foff;
-		if (ui->rsidRadioButton->isChecked()) return Page_Rsid;
-		if (ui->webmailRadioButton->isChecked()) return Page_WebMail;
-		if (ui->emailRadioButton->isChecked()) return Page_Email;
-		if (ui->friendRecommendationsRadioButton->isChecked()) return Page_FriendRecommendations;
-		return ConnectFriendWizard::Page_Foff;
-	case Page_Text:
-	case Page_Cert:
-	case Page_Rsid:
-		return error ? ConnectFriendWizard::Page_Conclusion : ConnectFriendWizard::Page_ErrorMessage;
-	case Page_Foff:
+    case Page_Text:		return error?Page_ErrorMessage:Page_Conclusion;
 	case Page_WebMail:
-	case Page_Email:
 	case Page_ErrorMessage:
 	case Page_Conclusion:
-	case Page_FriendRequest:
-	case Page_FriendRecommendations:
 		return -1;
 	}
 
@@ -864,12 +706,7 @@ ServicePermissionFlags ConnectFriendWizard::serviceFlags() const
 {
     ServicePermissionFlags flags(0) ;
 
-    if (hasVisitedPage(Page_FriendRequest))
-    {
-        if(  ui->_direct_transfer_CB->isChecked()) flags |= RS_NODE_PERM_DIRECT_DL ;
-        if(  ui->_allow_push_CB->isChecked()) flags |= RS_NODE_PERM_ALLOW_PUSH ;
-        if(  ui->_require_WL_CB->isChecked()) flags |= RS_NODE_PERM_REQUIRE_WL ;
-    } else if (hasVisitedPage(Page_Conclusion)) {
+if (hasVisitedPage(Page_Conclusion)) {
         if(  ui->_direct_transfer_CB_2->isChecked()) flags |= RS_NODE_PERM_DIRECT_DL ;
         if(  ui->_allow_push_CB_2->isChecked()) flags |= RS_NODE_PERM_ALLOW_PUSH ;
         if(  ui->_require_WL_CB_2->isChecked()) flags |= RS_NODE_PERM_REQUIRE_WL ;
@@ -888,17 +725,14 @@ void ConnectFriendWizard::accept()
 		sign = ui->signGPGCheckBox->isChecked();
 		accept_connection = ui->acceptNoSignGPGCheckBox->isChecked();
 		add_key_to_keyring = ui->addKeyToKeyring_CB->isChecked() ;
-	} else if (hasVisitedPage(Page_FriendRequest)) {
-		std::cerr << "ConnectFriendWizard::accept() called with page friend request visited" << std::endl;
-
-		sign = ui->fr_signGPGCheckBox->isChecked();
-		accept_connection = ui->fr_acceptNoSignGPGCheckBox->isChecked();
-	} else {
+	} else  {
 		QDialog::accept();
 		return;
 	}
 
-	if (!mCertificate.empty() && add_key_to_keyring)
+    // add the profile pgp key to keyring
+
+    if(!mIsShortInvite && !mCertificate.empty() && add_key_to_keyring)
 	{
 		RsPgpId pgp_id ;
 		RsPeerId ssl_id ;
@@ -913,33 +747,72 @@ void ConnectFriendWizard::accept()
 
 	bool runProgressDialog = false;
 
+    // add the peer as friend, either with or without pgp signature validation, depending on whether we have the key or not
+    // Note: that is different than having a short invite or not.
+
+    // first, set data related to profile key.
+
 	if(accept_connection && !peerDetails.gpg_id.isNull()) 
 	{
 		std::cerr << "ConclusionPage::validatePage() accepting GPG key for connection." << std::endl;
-        rsPeers->addFriend(peerDetails.id, peerDetails.gpg_id,serviceFlags()) ;
-        rsPeers->setServicePermissionFlags(peerDetails.gpg_id,serviceFlags()) ;
 
-    if(ui->_addIPToWhiteList_CB_2->isChecked())
-    {
-        sockaddr_storage addr ;
-        if(sockaddr_storage_ipv4_aton(addr,peerDetails.extAddr.c_str()) && sockaddr_storage_isValidNet(addr))
+        if(sign)
         {
-            std::cerr << "ConclusionPage::adding IP " << sockaddr_storage_tostring(addr) << " to whitelist." << std::endl;
-            rsBanList->addIpRange(addr,ui->_addIPToWhiteList_ComboBox_2->currentIndex(),RSBANLIST_TYPE_WHITELIST,std::string(tr("Added with certificate from %1").arg(ui->nameEdit->text()).toUtf8().constData()));
+            std::cerr << "ConclusionPage::validatePage() signing GPG key." << std::endl;
+            bool prev_is_bad = false;
+
+            for(int i=0;i<3;++i)
+            {
+                std::string pgp_name = rsPeers->getGPGName(rsPeers->getGPGOwnId());
+                bool cancelled;
+                std::string pgp_password;
+
+                if(!NotifyQt::getInstance()->askForPassword(tr("Profile password needed.").toStdString(), pgp_name + " (" + rsPeers->getOwnId().toStdString() + ")", prev_is_bad, pgp_password,cancelled))
+                {
+                    QMessageBox::critical(NULL,tr("Identity creation failed"),tr("Cannot create an identity linked to your profile without your profile password."));
+                    return;
+                }
+
+                if(rsPeers->signGPGCertificate(peerDetails.gpg_id,pgp_password))
+                {
+                    prev_is_bad = false;
+                    break;
+                }
+                else
+                    prev_is_bad = true;
+            }
+
+            if(prev_is_bad)
+            {
+                QMessageBox::warning(nullptr,tr("Signature failed"),tr("Signature failed. Uncheck the key signature box if you want to make friends without signing the friends' certificate"));
+                return;
+            }
         }
-    }
+        if(peerDetails.skip_pgp_signature_validation)
+			rsPeers->addSslOnlyFriend(peerDetails.id, peerDetails.gpg_id,peerDetails);
+		else
+			rsPeers->addFriend(peerDetails.id, peerDetails.gpg_id,serviceFlags()) ;
 
-		if(sign)
+		rsPeers->setServicePermissionFlags(peerDetails.gpg_id,serviceFlags()) ;
+
+		if(ui->_addIPToWhiteList_CB_2->isChecked())
 		{
-			std::cerr << "ConclusionPage::validatePage() signing GPG key." << std::endl;
-			rsPeers->signGPGCertificate(peerDetails.gpg_id); //bye default sign set accept_connection to true;
-			rsPeers->setServicePermissionFlags(peerDetails.gpg_id,serviceFlags()) ;
-		} 
+			sockaddr_storage addr ;
+			if(sockaddr_storage_ipv4_aton(addr,peerDetails.extAddr.c_str()) && sockaddr_storage_isValidNet(addr))
+			{
+				std::cerr << "ConclusionPage::adding IP " << sockaddr_storage_tostring(addr) << " to whitelist." << std::endl;
+				rsBanList->addIpRange(addr,ui->_addIPToWhiteList_ComboBox_2->currentIndex(),RSBANLIST_TYPE_WHITELIST,std::string(tr("Added with certificate from %1").arg(ui->nameEdit->text()).toUtf8().constData()));
+			}
+		}
 
-		if (!groupId.isEmpty()) 
-            rsPeers->assignPeerToGroup(RsNodeGroupId(groupId.toStdString()), peerDetails.gpg_id, true);
+
+
+		if (!groupId.isEmpty())
+			rsPeers->assignPeerToGroup(RsNodeGroupId(groupId.toStdString()), peerDetails.gpg_id, true);
 	}
 
+    // Then set data related to node location
+
 	if ((accept_connection) && (!peerDetails.id.isNull()))
 	{
 		runProgressDialog = true;
@@ -970,9 +843,7 @@ void ConnectFriendWizard::accept()
 				rsPeers->setDynDNS(peerDetails.id, peerDetails.dyndns);
 			}
 			for(auto&& ipr : peerDetails.ipAddressList)
-				rsPeers->addPeerLocator(
-				            peerDetails.id,
-				            RsUrl(ipr.substr(0, ipr.find(' '))) );
+				rsPeers->addPeerLocator( peerDetails.id, RsUrl(ipr.substr(0, ipr.find(' '))) );
 		}
 
 	}
@@ -985,58 +856,11 @@ void ConnectFriendWizard::accept()
 	}
 
 	NotifyQt::getInstance()->notifyListChange(NOTIFY_LIST_NEIGHBOURS,1) ;
-
 	QDialog::accept();
 }
 
 //============================= TextPage =====================================
 
-void ConnectFriendWizard::updateOwnCert()
-{
-	std::string invite = rsPeers->GetRetroshareInvite( rsPeers->getOwnId(),
-	            ui->userCertIncludeSignaturesButton->isChecked() );
-
-	std::cerr << "TextPage() getting Invite: " << invite << std::endl;
-
-	ui->userCertEdit->setPlainText(QString::fromUtf8(invite.c_str()));
-}
-
-void ConnectFriendWizard::toggleFormatState(bool doUpdate)
-{
-	if (ui->userCertOldFormatButton->isChecked()) 
-	{
-		ui->userCertOldFormatButton->setToolTip(tr("Use new certificate format (safer, more robust)"));
-		ui->userCertOldFormatButton->setIcon(QIcon(":/images/ledoff1.png")) ;
-	}
-	else 
-	{
-		ui->userCertOldFormatButton->setToolTip(tr("Use old (backward compatible) certificate format"));
-		ui->userCertOldFormatButton->setIcon(QIcon(":/images/ledon1.png")) ;
-	}
-
-	if (doUpdate) {
-		updateOwnCert();
-	}
-}
-
-void ConnectFriendWizard::toggleSignatureState(bool doUpdate)
-{
-	if (ui->userCertIncludeSignaturesButton->isChecked()) {
-		ui->userCertIncludeSignaturesButton->setToolTip(tr("Remove signatures"));
-	} else {
-		ui->userCertIncludeSignaturesButton->setToolTip(tr("Include signatures"));
-	}
-
-	if (doUpdate) {
-		updateOwnCert();
-	}
-}
-
-void ConnectFriendWizard::runEmailClient()
-{
-	sendMail("", tr("RetroShare Invite"), ui->userCertEdit->toPlainText());
-}
-
 void ConnectFriendWizard::friendCertChanged()
 {
 	ui->TextPage->setComplete(false);
@@ -1047,30 +871,51 @@ void ConnectFriendWizard::cleanFriendCert()
 {
 	bool certValid = false;
 	QString errorMsg ;
-	std::string cert = ui->friendCertEdit->toPlainText().toUtf8().constData();
+	QString certDetail;
+
+    std::string cert ;
+    RetroShareLink rslink(ui->friendCertEdit->toPlainText());
+
+    if(rslink.valid() && rslink.type() == RetroShareLink::TYPE_CERTIFICATE)
+        cert = rslink.radix().toStdString();
+    else
+        cert = ui->friendCertEdit->toPlainText().toUtf8().constData();
+
+#ifdef DEBUG_FRIENDWIZARD
+    std::cerr << "Friend cert:\"" << cert << "\"" << std::endl;
+#endif
 
 	if (cert.empty()) {
-		ui->friendCertCleanLabel->setPixmap(QPixmap(":/images/delete.png"));
 		ui->friendCertCleanLabel->setToolTip("");
-		ui->friendCertCleanLabel->setStyleSheet("");
 		errorMsg = tr("");
 
 	} else {
 		std::string cleanCert;
-		int error_code;
+		uint32_t error_code;
+		RsPeerDetails details;
 
-		if (rsPeers->cleanCertificate(cert, cleanCert, error_code)) {
+		if (rsPeers->cleanCertificate(cert, cleanCert, mIsShortInvite, error_code, details))
+		{
 			certValid = true;
-			if (cert != cleanCert) {
-				disconnect(ui->friendCertEdit, SIGNAL(textChanged()), this, SLOT(friendCertChanged()));
+
+			if (cert != cleanCert)
+			{
 				QTextCursor textCursor = ui->friendCertEdit->textCursor();
-				ui->friendCertEdit->setPlainText(QString::fromUtf8(cleanCert.c_str()));
-				ui->friendCertEdit->setTextCursor(textCursor);
-				ui->friendCertCleanLabel->setStyleSheet("");
-				connect(ui->friendCertEdit, SIGNAL(textChanged()), this, SLOT(friendCertChanged()));
+
+				whileBlocking(ui->friendCertEdit)->setPlainText(QString::fromUtf8(cleanCert.c_str()));
+				whileBlocking(ui->friendCertEdit)->setTextCursor(textCursor);
+
+                // use dummy flags so that the content of the description is driven by what's in the "details" variable.
+                RetroshareInviteFlags dummy_flags = RetroshareInviteFlags::ALL;
+                certDetail = ConfCertDialog::getCertificateDescription(details,false,mIsShortInvite,dummy_flags);
 			}
-			errorMsg = tr("Certificate appears to be valid");
-			ui->friendCertCleanLabel->setPixmap(QPixmap(":/images/accepted16.png"));
+			
+			if (mIsShortInvite)
+				errorMsg = tr("Valid Retroshare ID") + (mIsShortInvite?" (Short format)":" (plain format with profile key)");
+			else
+				errorMsg = tr("Valid certificate") ;
+
+			ui->friendCertCleanLabel->setPixmap(FilesDefs::getPixmapFromQtResourcePath(":/images/accepted16.png"));
 		} else {
 			if (error_code > 0) {
 				switch (error_code) {
@@ -1086,36 +931,25 @@ void ConnectFriendWizard::cleanFriendCert()
 
 				default:
 					errorMsg = tr("Not a valid Retroshare certificate!") ;
-					ui->friendCertCleanLabel->setStyleSheet("QLabel#friendCertCleanLabel {border: 2px solid red; border-radius: 6px;}");
 				}
 			}
-			ui->friendCertCleanLabel->setPixmap(QPixmap(":/images/delete.png"));
 		}
 	}
 
-	ui->friendCertCleanLabel->setPixmap(certValid ? QPixmap(":/images/accepted16.png") : QPixmap(":/images/delete.png"));
-	ui->friendCertCleanLabel->setToolTip(errorMsg);
+	ui->friendCertCleanLabel->setPixmap(certValid ? FilesDefs::getPixmapFromQtResourcePath(":/images/accepted16.png") : FilesDefs::getPixmapFromQtResourcePath(":/images/delete.png"));
+	ui->friendCertCleanLabel->setToolTip("<p>" + errorMsg + (certValid ? "\n" + certDetail : "") + "</p>");
 	ui->friendCertCleanLabel->setText(errorMsg);
+	ui->friendCertCleanLabel->setProperty("WrongValue", !certValid && !errorMsg.isEmpty());
+	ui->friendCertCleanLabel->style()->unpolish(ui->friendCertCleanLabel);
+	ui->friendCertCleanLabel->style()->polish(  ui->friendCertCleanLabel);
 
 	ui->TextPage->setComplete(certValid);
 }
 
-void ConnectFriendWizard::showHelpUserCert()
-{
-	QMessageBox::information(this, tr("Connect Friend Help"), tr("You can copy this text and send it to your friend via email or some other way"));
-}
-
-void ConnectFriendWizard::copyCert()
-{
-	QClipboard *clipboard = QApplication::clipboard();
-	clipboard->setText(ui->userCertEdit->toPlainText());
-	QMessageBox::information(this, "RetroShare", tr("Your Cert is copied to Clipboard, paste and send it to your friend via email or some other way"));
-}
-
 void ConnectFriendWizard::pasteCert()
 {
 	QClipboard *clipboard = QApplication::clipboard();
-	ui->friendCertEdit->setPlainText(clipboard->text());
+    ui->friendCertEdit->setPlainText(clipboard->text());
 }
 
 void ConnectFriendWizard::openCert()
@@ -1134,23 +968,7 @@ void ConnectFriendWizard::openCert()
 	}
 }
 
-void ConnectFriendWizard::saveCert()
-{
-	QString fileName = QFileDialog::getSaveFileName(this, tr("Save as..."), "", tr("RetroShare Certificate (*.rsc );;All Files (*)"));
-	if (fileName.isEmpty())
-		return;
-
-	QFile file(fileName);
-	if (!file.open(QFile::WriteOnly))
-		return;
-
-	//Todo: move save to file to p3Peers::SaveCertificateToFile
-
-	QTextStream ts(&file);
-	ts.setCodec(QTextCodec::codecForName("UTF-8"));
-	ts << ui->userCertEdit->document()->toPlainText();
-}
-
+#ifdef TO_BE_REMOVED
 //========================== CertificatePage =================================
 
 void ConnectFriendWizard::loadFriendCert()
@@ -1330,7 +1148,9 @@ void ConnectFriendWizard::signAllSelectedUsers()
 
 //============================= RsidPage =====================================
 
+
 //============================ Emailpage =====================================
+#endif
 
 //========================= ErrorMessagePage =================================
 
@@ -1393,6 +1213,7 @@ void ConnectFriendWizard::toggleAdvanced()
 	{
 		ui->cp_Frame->show();
 		ui->toggleadvancedButton->setText("Hide advanced options");
+		resize(sizeHint());
 		AdvancedVisible=true;
 	}
 }
