@@ -18,32 +18,34 @@
  *                                                                             *
  *******************************************************************************/
 
+#include "ChatPage.h"
+
+#include "rsharesettings.h"
+#include "gui/MainWindow.h"
+#include "gui/notifyqt.h"
+#include "gui/RetroShareLink.h"
+#include "gui/chat/ChatDialog.h"
+#include "util/misc.h"
+
+#include "retroshare/rsconfig.h"
+#include "retroshare/rshistory.h"
+#include "retroshare/rsmsgs.h"
+#include "retroshare/rsnotify.h"
+#include "retroshare/rspeers.h"
+
 #include <QColorDialog>
 #include <QMenu>
 #include <QMessageBox>
+
 #include <time.h>
 
-#include <retroshare/rsnotify.h>
-#include <retroshare/rsmsgs.h>
-#include <retroshare/rspeers.h>
-#include "ChatPage.h"
-#include <gui/RetroShareLink.h>
-#include "gui/chat/ChatDialog.h"
-#include "gui/notifyqt.h"
-#include "rsharesettings.h"
-#include "util/misc.h"
-#include <retroshare/rsconfig.h>
-
-#include <retroshare/rshistory.h>
-#include <retroshare/rsmsgs.h>
-
 #define VARIANT_STANDARD    "Standard"
-#define IMAGE_CHAT_CREATE   ":/images/add_24x24.png"
-#define IMAGE_CHAT_OPEN     ":/images/typing.png"
+#define IMAGE_CHAT_CREATE   ":/icons/png/add.png"
+#define IMAGE_CHAT_OPEN     ":/icons/png/typing.png"
 #define IMAGE_CHAT_DELETE   ":/images/deletemail24.png"
 #define IMAGE_CHAT_COPY     ":/images/copyrslink.png"
 
-QString ChatPage::loadStyleInfo(ChatStyle::enumStyleType type, QComboBox *style_CB, QComboBox *comboBox, QString &styleVariant)
+QString ChatPage::loadStyleInfo(ChatStyle::enumStyleType type, QComboBox *style_CB, QComboBox *var_CB, QString &styleVariant)
 {
     QList<ChatStyleInfo> styles;
     QList<ChatStyleInfo>::iterator style;
@@ -95,13 +97,13 @@ QString ChatPage::loadStyleInfo(ChatStyle::enumStyleType type, QComboBox *style_
 
     /* now the combobox should be filled */
 
-    int index = comboBox->findText(styleVariant);
+    int index = var_CB->findText(styleVariant);
 
     if (index != -1) {
-        whileBlocking(comboBox)->setCurrentIndex(index);
+        whileBlocking(var_CB)->setCurrentIndex(index);
     } else {
-        if (comboBox->count()) {
-            whileBlocking(comboBox)->setCurrentIndex(0);
+        if (var_CB->count()) {
+            whileBlocking(var_CB)->setCurrentIndex(0);
         }
     }
     return stylePath;
@@ -123,9 +125,6 @@ void ChatPage::updateFontsAndEmotes()
 /** Saves the changes on this page */
 void ChatPage::updateChatParams()
 {
-	// state of distant Chat combobox
-	Settings->setValue("DistantChat", ui.distantChatComboBox->currentIndex());
-
 	Settings->setChatScreenFont(fontTempChat.toString());
 	NotifyQt::getInstance()->notifyChatFontChanged();
 
@@ -133,6 +132,7 @@ void ChatPage::updateChatParams()
 	Settings->setChatSendAsPlainTextByDef(ui.sendAsPlainTextByDef->isChecked());
 	Settings->setChatLoadEmbeddedImages(ui.loadEmbeddedImages->isChecked());
 	Settings->setChatDoNotSendIsTyping(ui.DontSendTyping->isChecked());
+	Settings->setShrinkChatTextEdit(ui.shrinkChatTextEdit->isChecked());
 }
 
 void ChatPage::updateChatSearchParams()
@@ -166,19 +166,23 @@ void ChatPage::updateHistoryParams()
 	Settings->setPublicChatHistoryCount(ui.publicChatLoadCount->value());
 	Settings->setPrivateChatHistoryCount(ui.privateChatLoadCount->value());
 	Settings->setLobbyChatHistoryCount(ui.lobbyChatLoadCount->value());
+	Settings->setDistantChatHistoryCount(ui.distantChatLoadCount->value());
 
 	rsHistory->setEnable(RS_HISTORY_TYPE_PUBLIC , ui.publicChatEnable->isChecked());
 	rsHistory->setEnable(RS_HISTORY_TYPE_PRIVATE, ui.privateChatEnable->isChecked());
 	rsHistory->setEnable(RS_HISTORY_TYPE_LOBBY  , ui.lobbyChatEnable->isChecked());
+	rsHistory->setEnable(RS_HISTORY_TYPE_DISTANT, ui.distantChatEnable->isChecked());
 
 	rsHistory->setSaveCount(RS_HISTORY_TYPE_PUBLIC , ui.publicChatSaveCount->value());
     rsHistory->setSaveCount(RS_HISTORY_TYPE_PRIVATE, ui.privateChatSaveCount->value());
     rsHistory->setSaveCount(RS_HISTORY_TYPE_LOBBY  , ui.lobbyChatSaveCount->value());
+	rsHistory->setSaveCount(RS_HISTORY_TYPE_DISTANT, ui.distantChatSaveCount->value());
+
 }
 
 void ChatPage::updatePublicStyle()
 {
-	ChatStyleInfo info = ui.publicStyle->itemData(ui.historyStyle->currentIndex(),Qt::UserRole).value<ChatStyleInfo>();
+	ChatStyleInfo info = ui.publicStyle->itemData(ui.publicStyle->currentIndex(),Qt::UserRole).value<ChatStyleInfo>();
 
 	if (publicStylePath != info.stylePath || publicStyleVariant != ui.publicComboBoxVariant->currentText()) {
 		Settings->setPublicChatStyle(info.stylePath, ui.publicComboBoxVariant->currentText());
@@ -188,7 +192,7 @@ void ChatPage::updatePublicStyle()
 
 void ChatPage::updatePrivateStyle()
 {
-	ChatStyleInfo info = ui.privateStyle->itemData(ui.historyStyle->currentIndex(),Qt::UserRole).value<ChatStyleInfo>();
+	ChatStyleInfo info = ui.privateStyle->itemData(ui.privateStyle->currentIndex(),Qt::UserRole).value<ChatStyleInfo>();
 
 	if (privateStylePath != info.stylePath || privateStyleVariant != ui.privateComboBoxVariant->currentText()) {
 		Settings->setPrivateChatStyle(info.stylePath, ui.privateComboBoxVariant->currentText());
@@ -206,7 +210,7 @@ void ChatPage::updateHistoryStyle()
 	}
 }
 
-void ChatPage::updateHistoryStorage() { rsHistory->setMaxStorageDuration(ui.max_storage_period->value() * 86400) ; }
+void ChatPage::updateHistoryStorage() { rsHistory->setMaxStorageDuration(ui.max_storage_period->value() * 86400) ; } //24H
 
 
 /** Constructor */
@@ -220,7 +224,14 @@ ChatPage::ChatPage(QWidget * parent, Qt::WindowFlags flags)
     ui.minimumContrastLabel->hide();
     ui.minimumContrast->hide();
 #endif
-	connect(ui.distantChatComboBox,        SIGNAL(currentIndexChanged(int)), this, SLOT(distantChatComboBoxChanged(int)));
+    connect(ui.chatLobbies_CountFollowingText, SIGNAL(toggled(bool)),ui.chatLobbies_TextToNotify,SLOT(setEnabled(bool)));
+    connect(ui.chatLobbies_CountUnRead,        SIGNAL(toggled(bool)),this, SLOT(updateChatLobbyUserNotify())) ;
+    connect(ui.chatLobbies_CheckNickName,      SIGNAL(toggled(bool)),this, SLOT(updateChatLobbyUserNotify())) ;
+    connect(ui.chatLobbies_CountFollowingText, SIGNAL(toggled(bool)),this, SLOT(updateChatLobbyUserNotify())) ;
+    connect(ui.chatLobbies_TextToNotify,       SIGNAL(textChanged(QString)),this, SLOT(updateChatLobbyUserNotify()));
+    connect(ui.chatLobbies_TextCaseSensitive,  SIGNAL(toggled(bool)),this, SLOT(updateChatLobbyUserNotify())) ;
+
+    connect(ui.distantChatComboBox,        SIGNAL(currentIndexChanged(int)), this, SLOT(distantChatComboBoxChanged(int)));
 
 	connect(ui.checkBox_emoteprivchat,     SIGNAL(toggled(bool)),            this, SLOT(updateFontsAndEmotes()));
 	connect(ui.checkBox_emotegroupchat,    SIGNAL(toggled(bool)),            this, SLOT(updateFontsAndEmotes()));
@@ -235,6 +246,7 @@ ChatPage::ChatPage(QWidget * parent, Qt::WindowFlags flags)
 	connect(ui.sendAsPlainTextByDef,       SIGNAL(toggled(bool)),            this, SLOT(updateChatParams()));
 	connect(ui.loadEmbeddedImages,         SIGNAL(toggled(bool)),            this, SLOT(updateChatParams()));
 	connect(ui.DontSendTyping,             SIGNAL(toggled(bool)),            this, SLOT(updateChatParams()));
+	connect(ui.shrinkChatTextEdit,         SIGNAL(toggled(bool)),            this, SLOT(updateChatParams()));
 
 	connect(ui.sbSearch_CharToStart,       SIGNAL(valueChanged(int)),        this, SLOT(updateChatSearchParams()));
 	connect(ui.cbSearch_CaseSensitively,   SIGNAL(toggled(bool)),            this, SLOT(updateChatSearchParams()));
@@ -247,13 +259,19 @@ ChatPage::ChatPage(QWidget * parent, Qt::WindowFlags flags)
 
 	connect(ui.publicChatLoadCount,        SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
 	connect(ui.privateChatLoadCount,       SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
+	connect(ui.distantChatLoadCount,       SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
 	connect(ui.lobbyChatLoadCount,         SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
+	
 	connect(ui.publicChatEnable,           SIGNAL(toggled(bool)),            this, SLOT(updateHistoryParams()));
 	connect(ui.privateChatEnable,          SIGNAL(toggled(bool)),            this, SLOT(updateHistoryParams()));
+	connect(ui.distantChatEnable,          SIGNAL(toggled(bool)),            this, SLOT(updateHistoryParams()));
 	connect(ui.lobbyChatEnable,            SIGNAL(toggled(bool)),            this, SLOT(updateHistoryParams()));
+	
 	connect(ui.publicChatSaveCount,        SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
 	connect(ui.privateChatSaveCount,       SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
 	connect(ui.lobbyChatSaveCount,         SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
+	connect(ui.distantChatSaveCount,       SIGNAL(valueChanged(int)),        this, SLOT(updateHistoryParams()));
+
 
     connect(ui.publicStyle,                SIGNAL(currentIndexChanged(int)),   this, SLOT(updatePublicStyle())) ;
     connect(ui.publicComboBoxVariant,      SIGNAL(currentIndexChanged(int)), this, SLOT(updatePublicStyle())) ;
@@ -276,7 +294,63 @@ ChatPage::ChatPage(QWidget * parent, Qt::WindowFlags flags)
 	connect(ui.publicStyle,                SIGNAL(currentIndexChanged(int)), this,SLOT(on_publicList_currentRowChanged(int)));
 	connect(ui.privateStyle,               SIGNAL(currentIndexChanged(int)), this,SLOT(on_privateList_currentRowChanged(int)));
 	connect(ui.historyStyle,               SIGNAL(currentIndexChanged(int)), this,SLOT(on_historyList_currentRowChanged(int)));
+
+	//Resize system size square
+	qreal logicalPPCX = ui.systemSize_Frame->logicalDpiX()/2.54;
+	qreal logicalPPCY = ui.systemSize_Frame->logicalDpiY()/2.54;
+
+	ui.systemSize_Frame->setMinimumSize(logicalPPCX,logicalPPCY);
+	ui.systemSize_Frame->setMaximumSize(logicalPPCX,logicalPPCY);
+	ui.systemSize_Frame->setToolTip(QString("Your system reports a screen with %1 DPI (Dots Per Inches) = %2 PPC (Pixels Per Centimeters).")
+	                                .arg(ui.systemSize_Frame->logicalDpiX()).arg(logicalPPCX));
+
+	//Resize system font size square
+	qreal ptFontSize = (qreal)QApplication::font().pointSize();
+	qreal pxFontSize = (qreal)QFontMetrics(QApplication::font()).height();
+	qreal px1ptSize = pxFontSize / ptFontSize;
+	//1 pt = 1/72 inch = 2.54/72 cm
+	qreal px1cmSize = (px1ptSize * 72.0) / 2.54;
+
+	ui.systemFontSize_Frame->setMinimumSize(px1cmSize,px1cmSize);
+	ui.systemFontSize_Frame->setMaximumSize(px1cmSize,px1cmSize);
+
+	ui.systemFontSize_Frame->setToolTip(QString("Your default font is %1 points high = %2 mm. Qt print it with %3 pixels.\nSo Font use %4 PPC (Pixels Per Centimeters).")
+	                                    .arg(ptFontSize).arg((ptFontSize*25.4)/72.0).arg(pxFontSize).arg(px1cmSize));
+
+
+    /* Add user notify */
+    const QList<UserNotify*> &userNotifyList = MainWindow::getInstance()->getUserNotifyList() ;
+    QList<UserNotify*>::const_iterator it;
+
+    mChatLobbyUserNotify = nullptr;
+
+    for (it = userNotifyList.begin(); it != userNotifyList.end(); ++it)
+    {
+        //UserNotify *userNotify = *it;
+
+        //To get ChatLobbyUserNotify Settings
+
+        if(!mChatLobbyUserNotify)
+            mChatLobbyUserNotify = dynamic_cast<ChatLobbyUserNotify*>(*it);
+    }
+
+    // hide the broadcast page: it only shows "show/hide emoticons, which is really not much for an entire page
+    ui.tabWidget->removeTab(3);
 }
+
+void ChatPage::updateChatLobbyUserNotify()
+{
+    if(!mChatLobbyUserNotify)
+        return ;
+
+    mChatLobbyUserNotify->setCountUnRead(ui.chatLobbies_CountUnRead->isChecked()) ;
+    mChatLobbyUserNotify->setCheckForNickName(ui.chatLobbies_CheckNickName->isChecked()) ;
+    mChatLobbyUserNotify->setCountSpecificText(ui.chatLobbies_CountFollowingText->isChecked()) ;
+    mChatLobbyUserNotify->setTextToNotify(ui.chatLobbies_TextToNotify->document()->toPlainText());
+    mChatLobbyUserNotify->setTextCaseSensitive(ui.chatLobbies_TextCaseSensitive->isChecked());
+}
+
+
 void ChatPage::updateChatFlags()
 {
 	uint chatflags   = 0;
@@ -318,9 +392,23 @@ ChatPage::load()
     whileBlocking(ui.minimumContrast)->setValue(Settings->value("MinimumContrast", 4.5).toDouble());
     Settings->endGroup();
 
-	     // state of distant Chat combobox
-    int index = Settings->value("DistantChat", 0).toInt();
-    whileBlocking(ui.distantChatComboBox)->setCurrentIndex(index);
+    // state of distant Chat combobox
+
+    switch(rsMsgs->getDistantChatPermissionFlags())
+    {
+        default:
+        case RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_NONE:
+            whileBlocking(ui.distantChatComboBox)->setCurrentIndex(0);
+            break ;
+
+        case RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_NON_CONTACTS:
+            whileBlocking(ui.distantChatComboBox)->setCurrentIndex(1);
+            break ;
+
+        case RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_EVERYBODY:
+            whileBlocking(ui.distantChatComboBox)->setCurrentIndex(2);
+            break ;
+    }
 
     fontTempChat.fromString(Settings->getChatScreenFont());
 
@@ -328,6 +416,7 @@ ChatPage::load()
     whileBlocking(ui.sendAsPlainTextByDef)->setChecked(Settings->getChatSendAsPlainTextByDef());
     whileBlocking(ui.loadEmbeddedImages)->setChecked(Settings->getChatLoadEmbeddedImages());
     whileBlocking(ui.DontSendTyping)->setChecked(Settings->getChatDoNotSendIsTyping());
+    whileBlocking(ui.shrinkChatTextEdit)->setChecked(Settings->getShrinkChatTextEdit());
 
 	std::string advsetting;
 	if(rsConfig->getConfigurationOption(RS_CONFIG_ADVANCED, advsetting) && (advsetting == "YES"))
@@ -349,14 +438,18 @@ ChatPage::load()
     whileBlocking(ui.publicChatLoadCount)->setValue(Settings->getPublicChatHistoryCount());
     whileBlocking(ui.privateChatLoadCount)->setValue(Settings->getPrivateChatHistoryCount());
     whileBlocking(ui.lobbyChatLoadCount)->setValue(Settings->getLobbyChatHistoryCount());
+	whileBlocking(ui.distantChatLoadCount)->setValue(Settings->getDistantChatHistoryCount());
 
     whileBlocking(ui.publicChatEnable)->setChecked(rsHistory->getEnable(RS_HISTORY_TYPE_PUBLIC));
     whileBlocking(ui.privateChatEnable)->setChecked(rsHistory->getEnable(RS_HISTORY_TYPE_PRIVATE));
     whileBlocking(ui.lobbyChatEnable)->setChecked(rsHistory->getEnable(RS_HISTORY_TYPE_LOBBY));
+	whileBlocking(ui.distantChatEnable)->setChecked(rsHistory->getEnable(RS_HISTORY_TYPE_DISTANT));
 
     whileBlocking(ui.publicChatSaveCount)->setValue(rsHistory->getSaveCount(RS_HISTORY_TYPE_PUBLIC));
     whileBlocking(ui.privateChatSaveCount)->setValue(rsHistory->getSaveCount(RS_HISTORY_TYPE_PRIVATE));
     whileBlocking(ui.lobbyChatSaveCount)->setValue(rsHistory->getSaveCount(RS_HISTORY_TYPE_LOBBY));
+	whileBlocking(ui.distantChatSaveCount)->setValue(rsHistory->getSaveCount(RS_HISTORY_TYPE_DISTANT));
+
     
     // using fontTempChat.rawname() does not always work!
     // see http://doc.qt.digia.com/qt-maemo/qfont.html#rawName
@@ -364,7 +457,7 @@ ChatPage::load()
     whileBlocking(ui.labelChatFontPreview)->setText(fontname[0]);
     whileBlocking(ui.labelChatFontPreview)->setFont(fontTempChat);
 
-	whileBlocking(ui.max_storage_period)->setValue(rsHistory->getMaxStorageDuration()/86400) ;
+	whileBlocking(ui.max_storage_period)->setValue(rsHistory->getMaxStorageDuration()/86400) ;//24H
 
     /* Load styles */
     publicStylePath = loadStyleInfo(ChatStyle::TYPE_PUBLIC, ui.publicStyle, ui.publicComboBoxVariant, publicStyleVariant);
@@ -409,6 +502,15 @@ ChatPage::load()
 		 ui._collected_contacts_LW->insertItem(0,item) ;
 	 }
 #endif
+
+    if (mChatLobbyUserNotify){
+        whileBlocking(ui.chatLobbies_CountUnRead)->setChecked(mChatLobbyUserNotify->isCountUnRead());
+        whileBlocking(ui.chatLobbies_CheckNickName)->setChecked(mChatLobbyUserNotify->isCheckForNickName());
+        whileBlocking(ui.chatLobbies_CountFollowingText)->setChecked(mChatLobbyUserNotify->isCountSpecificText()) ;
+        whileBlocking(ui.chatLobbies_TextToNotify)->setEnabled(mChatLobbyUserNotify->isCountSpecificText()) ;
+        whileBlocking(ui.chatLobbies_TextToNotify)->setPlainText(mChatLobbyUserNotify->textToNotify());
+        whileBlocking(ui.chatLobbies_TextCaseSensitive)->setChecked(mChatLobbyUserNotify->isTextCaseSensitive());
+    }
 }
 
 void ChatPage::on_pushButtonChangeChatFont_clicked()
@@ -481,8 +583,8 @@ void ChatPage::on_publicList_currentRowChanged(int currentRow)
             whileBlocking(ui.publicComboBoxVariant)->setCurrentIndex(0);
         }
     } else {
-        whileBlocking(ui.publicAuthor)->clear();
-        whileBlocking(ui.publicDescription)->clear();
+        ui.publicAuthor->clear();
+        ui.publicDescription->clear();
         whileBlocking(ui.publicComboBoxVariant)->clear();
         whileBlocking(ui.publicComboBoxVariant)->setDisabled(true);
     }
@@ -493,14 +595,14 @@ void ChatPage::on_publicList_currentRowChanged(int currentRow)
 void ChatPage::on_privateList_currentRowChanged(int currentRow)
 {
     if (currentRow != -1) {
-        ChatStyleInfo info = ui.privateStyle->itemData(ui.privateStyle->currentIndex(),Qt::UserRole).value<ChatStyleInfo>();
+        ChatStyleInfo info = ui.privateStyle->itemData(currentRow,Qt::UserRole).value<ChatStyleInfo>();
 
         QString author = info.authorName;
         if (info.authorEmail.isEmpty() == false) {
             author += " (" + info.authorEmail + ")";
         }
-        whileBlocking(ui.privateAuthor)->setText(author);
-        whileBlocking(ui.privateDescription)->setText(info.styleDescription);
+        ui.privateAuthor->setText(author);
+        ui.privateDescription->setText(info.styleDescription);
 
         QStringList variants;
         ChatStyle::getAvailableVariants(info.stylePath, variants);
@@ -516,8 +618,8 @@ void ChatPage::on_privateList_currentRowChanged(int currentRow)
             whileBlocking(ui.privateComboBoxVariant)->setCurrentIndex(0);
         }
     } else {
-        whileBlocking(ui.privateAuthor)->clear();
-        whileBlocking(ui.privateDescription)->clear();
+        ui.privateAuthor->clear();
+        ui.privateDescription->clear();
         whileBlocking(ui.privateComboBoxVariant)->clear();
         whileBlocking(ui.privateComboBoxVariant)->setDisabled(true);
     }
@@ -528,14 +630,14 @@ void ChatPage::on_privateList_currentRowChanged(int currentRow)
 void ChatPage::on_historyList_currentRowChanged(int currentRow)
 {
     if (currentRow != -1) {
-        ChatStyleInfo info = ui.historyStyle->itemData(ui.historyStyle->currentIndex(),Qt::UserRole).value<ChatStyleInfo>();
+        ChatStyleInfo info = ui.historyStyle->itemData(currentRow,Qt::UserRole).value<ChatStyleInfo>();
 
         QString author = info.authorName;
         if (info.authorEmail.isEmpty() == false) {
             author += " (" + info.authorEmail + ")";
         }
-        whileBlocking(ui.historyAuthor)->setText(author);
-        whileBlocking(ui.historyDescription)->setText(info.styleDescription);
+        ui.historyAuthor->setText(author);
+        ui.historyDescription->setText(info.styleDescription);
 
         QStringList variants;
         ChatStyle::getAvailableVariants(info.stylePath, variants);
@@ -551,8 +653,8 @@ void ChatPage::on_historyList_currentRowChanged(int currentRow)
             whileBlocking(ui.historyComboBoxVariant)->setCurrentIndex(0);
         }
     } else {
-        whileBlocking(ui.historyAuthor)->clear();
-        whileBlocking(ui.historyDescription)->clear();
+        ui.historyAuthor->clear();
+        ui.historyDescription->clear();
         whileBlocking(ui.historyComboBoxVariant)->clear();
         whileBlocking(ui.historyComboBoxVariant)->setDisabled(true);
     }
@@ -597,15 +699,15 @@ void ChatPage::distantChatComboBoxChanged(int i)
 {
 	switch(i)
 	{
-		default: 
-		case 0: rsMsgs->setDistantChatPermissionFlags(RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_NONE) ;           
-				  break ;
-				  
-		case 1:  rsMsgs->setDistantChatPermissionFlags(RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_NON_CONTACTS) ;
-				  break ;
+		default:
+		case 0: rsMsgs->setDistantChatPermissionFlags(RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_NONE) ;
+		break ;
 
-		case 2:  rsMsgs->setDistantChatPermissionFlags(RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_EVERYBODY) ;
-				  break ;
+		case 1: rsMsgs->setDistantChatPermissionFlags(RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_NON_CONTACTS) ;
+		break ;
+
+		case 2: rsMsgs->setDistantChatPermissionFlags(RS_DISTANT_CHAT_CONTACT_PERMISSION_FLAG_FILTER_EVERYBODY) ;
+		break ;
 	}
 
 }
