@@ -42,6 +42,7 @@
 #include <iostream>
 #include <stdlib.h>
 
+#include "gui/common/FilesDefs.h"
 #include <gui/common/rshtml.h>
 #include <gui/common/vmessagebox.h>
 #include <gui/gxs/GxsIdDetails.h>
@@ -138,17 +139,22 @@ static bool notifyRunningInstance()
     // that a new process had been started
     QLocalSocket localSocket;
     localSocket.connectToServer(QString(TARGET));
-
+#ifdef DEBUG
     std::cerr << "Rshare::Rshare waitForConnected to other instance." << std::endl;
+#endif
     if( localSocket.waitForConnected(100) )
     {
+#ifdef DEBUG
       std::cerr << "Rshare::Rshare Connection etablished. Waiting for disconnection." << std::endl;
+#endif
       localSocket.waitForDisconnected(1000);
       return true;
     }
     else
     {
+#ifdef DEBUG
         std::cerr << "Rshare::Rshare failed to connect to other instance." << std::endl;
+#endif
         return false;
     }
 }
@@ -173,17 +179,17 @@ Rshare::Rshare(QStringList args, int &argc, char **argv, const QString &dir)
     if(args.empty())
         sendArgsToRunningInstance = false;
     // if we find non-forwardable args, start a new instance
-    for(int i = 0; i < args.size(); ++i)
+    for(int iCurs = 0; iCurs < args.size(); ++iCurs)
     {
         const char* const* argit = forwardableArgs;
         bool found = false;
-        while(*argit && i < args.size())
+        while(*argit && iCurs < args.size())
         {
-            if(args.value(i) == "-"+QString(*argit) || args.value(i) == "--"+QString(*argit))
+            if(args.value(iCurs) == "-"+QString(*argit) || args.value(iCurs) == "--"+QString(*argit))
             {
                 found = true;
                 if(argNeedsValue(*argit))
-                    i++;
+                    iCurs++;
             }
             argit++;
         }
@@ -241,6 +247,11 @@ Rshare::Rshare(QStringList args, int &argc, char **argv, const QString &dir)
         localServer= new QLocalServer();
         QObject::connect(localServer, SIGNAL(newConnection()), this, SLOT(slotConnectionEstablished()));
         updateLocalServer();
+		// clear out any old arguments (race condition?)
+		QSharedMemory newArgs;
+		newArgs.setKey(QString(TARGET) + "_newArgs");
+		if(newArgs.attach(QSharedMemory::ReadWrite))
+			newArgs.detach();
     }
   }
 
@@ -253,7 +264,7 @@ Rshare::Rshare(QStringList args, int &argc, char **argv, const QString &dir)
 #ifndef __APPLE__
 
   /* set default window icon */
-  setWindowIcon(QIcon(":/icons/logo_128.png"));
+  setWindowIcon(FilesDefs::getIconFromQtResourcePath(":/icons/logo_128.png"));
 
 #endif
 
@@ -325,20 +336,27 @@ Rshare::~Rshare()
  */
 void Rshare::slotConnectionEstablished()
 {
-	QLocalSocket *socket = localServer->nextPendingConnection();
-	socket->close();
-	delete socket;
-
 	QSharedMemory newArgs;
 	newArgs.setKey(QString(TARGET) + "_newArgs");
 
+	QLocalSocket *socket = localServer->nextPendingConnection();
+
 	if (!newArgs.attach())
 	{
-		std::cerr << "(EE) Rshare::slotConnectionEstablished() Unable to attach to shared memory segment."
-		          << newArgs.errorString().toStdString() << std::endl;
+		/* this is not an error. It just means we were notified to check
+		   newArgs, but none had been set yet.
+		   TODO: implement separate ping/take messages
+		   std::cerr << "(EE) Rshare::slotConnectionEstablished() Unable to attach to shared memory segment."
+		   << newArgs.errorString().toStdString() << std::endl;
+		   */
+		socket->close();
+		delete socket;
 		return;
 	}
 
+	socket->close();
+	delete socket;
+
 	QBuffer buffer;
 	QDataStream in(&buffer);
 	QStringList args;
@@ -451,12 +469,12 @@ Rshare::showUsageMessageBox()
 
 /** Returns true if the specified argument expects a value. */
 bool
-Rshare::argNeedsValue(QString argName)
+Rshare::argNeedsValue(const QString &argName)
 {
 	return (
 	        argName == ARG_DATADIR  ||
 	        argName == ARG_LOGFILE  ||
-          argName == ARG_LOGLEVEL ||
+	        argName == ARG_LOGLEVEL ||
 	        argName == ARG_GUISTYLE ||
 	        argName == ARG_GUISTYLESHEET ||
 	        argName == ARG_LANGUAGE ||
@@ -473,25 +491,26 @@ Rshare::argNeedsValue(QString argName)
 void
 Rshare::parseArguments(QStringList args, bool firstRun)
 {
-	QString arg, value;
+	QString arg, argl, value;
 
 	/* Loop through all command-line args/values and put them in a map */
-	for (int i = 0; i < args.size(); ++i) {
+	for (int iCurs = 0; iCurs < args.size(); ++iCurs) {
 		/* Get the argument name and set a blank value */
-		arg = args.at(i);//.toLower(); Need Upper case for file name.
-		if (arg.toLower() == "empty") continue;
+		arg = args.at(iCurs);//.toLower(); Need Upper case for file name.
+		argl = arg.toLower();
+		if (argl == "empty") continue;
 		value = "";
 
 		/* Check if it starts with a - or -- */
 		if (arg.startsWith("-")) {
 			arg = arg.mid((arg.startsWith("--") ? 2 : 1));
 			/* Check if it takes a value and there is one on the command-line */
-			if (i < args.size()-1 && argNeedsValue(arg.toLower())) {
-				value = args.at(++i);
+			if (iCurs < args.size()-1 && argNeedsValue(arg)) {
+				value = args.at(++iCurs);
 			}
 		} else {
 			/* Check if links or files without arg */
-			if (arg.toLower().startsWith("retroshare://")) {
+			if (argl.startsWith("retroshare://")) {
 				value = arg;
 				arg = ARG_RSLINK_L;
 			} else {
@@ -505,8 +524,8 @@ Rshare::parseArguments(QStringList args, bool firstRun)
 		/* handle opmode that could be change while running.*/
 		QString omValue = QString(value).prepend(";").append(";").toLower();
 		QString omValues = QString(";full;noturtle;gaming;minimal;");
-		if ((arg == ARG_OPMODE_S || arg == ARG_OPMODE_L ) &&
-		    omValues.contains(omValue)) {
+		if ((arg == ARG_OPMODE_S || arg == ARG_OPMODE_L )
+		    && omValues.contains(omValue)) {
 			_opmode = value;
 		}
 
@@ -529,8 +548,8 @@ Rshare::validateArguments(QString &errmsg)
   /* Check for a writable log file */
   if (_args.contains(ARG_LOGFILE) && !_log.isOpen()) {
     errmsg = tr("Unable to open log file '%1': %2")
-                           .arg(_args.value(ARG_LOGFILE))
-                           .arg(_log.errorString());
+                           .arg(  _args.value(ARG_LOGFILE)
+                                , _log.errorString());
     return false;
   }
   /* Check for a valid log level */
@@ -675,8 +694,8 @@ void Rshare::resetLanguageAndStyle()
 
 // RetroShare:
 //   Default:
-//     :/qss/stylesheet/qss.default
-//     :/qss/stylesheet/qss.<locale>
+//     :/qss/stylesheet/default.qss
+//     :/qss/stylesheet/<locale>.qss
 //   Internal:
 //     :/qss/stylesheet/<name>.qss
 //   External:
@@ -686,8 +705,8 @@ void Rshare::resetLanguageAndStyle()
 //
 // Plugin:
 //   Default:
-//     :/qss/stylesheet/<plugin>/<plugin>_qss.default
-//     :/qss/stylesheet/<plugin>/<plugin>_qss.<locale>
+//     :/qss/stylesheet/<plugin>/<plugin>_default.qss
+//     :/qss/stylesheet/<plugin>/<plugin>_<locale>.qss
 //   Internal:
 //     :/qss/stylesheet/<plugin>/<plugin>_<name>.qss
 //   External:
@@ -706,8 +725,8 @@ void Rshare::loadStyleSheet(const QString &sheetName)
     /* Get stylesheet from plugins */
     if (rsPlugins) {
         int count = rsPlugins->nbPlugins();
-        for (int i = 0; i < count; ++i) {
-            RsPlugin* plugin = rsPlugins->plugin(i);
+        for (int iCurs = 0; iCurs < count; ++iCurs) {
+            RsPlugin* plugin = rsPlugins->plugin(iCurs);
             if (plugin) {
                 QString pluginStyleSheetName = QString::fromUtf8(plugin->qt_stylesheet().c_str());
                 if (!pluginStyleSheetName.isEmpty()) {
@@ -719,22 +738,22 @@ void Rshare::loadStyleSheet(const QString &sheetName)
 
     foreach (QString name, names) {
         /* load the default stylesheet */
-        QFile file(QString(":/qss/stylesheet/%1qss.default").arg(name));
+        QFile file(QString(":/qss/stylesheet/%1default.qss").arg(name));
         if (file.open(QFile::ReadOnly)) {
             styleSheet += QLatin1String(file.readAll()) + "\n";
             file.close();
         }
 
         /* load locale depended default stylesheet */
-        file.setFileName(QString(":/qss/stylesheet/%1qss.%2").arg(name, locale));
+        file.setFileName(QString(":/qss/stylesheet/%1%2.qss").arg(name, locale));
         if (file.open(QFile::ReadOnly)) {
             styleSheet += QLatin1String(file.readAll()) + "\n";
             file.close();
         }
 
-        if (!sheetName.isEmpty()) {
+        if (!sheetName.isEmpty() && (sheetName != ":default")) {
             /* load stylesheet */
-            if (sheetName.left(1) == ":") {
+            if (sheetName.at(0) == ':') {
                 /* internal stylesheet */
                 file.setFileName(QString(":/qss/stylesheet/%1%2.qss").arg(name, sheetName.mid(1)));
             } else {
@@ -755,6 +774,9 @@ void Rshare::loadStyleSheet(const QString &sheetName)
                     styleSheet += QLatin1String(file.readAll()) + "\n";
                     file.close();
                 }
+
+                /* replace %THISPATH% by file path so url can get relative files */
+                styleSheet = styleSheet.replace("url(%THISPATH%",QString("url(%1").arg(fileInfo.absolutePath()));
             }
         }
     }
@@ -770,7 +792,7 @@ void Rshare::getAvailableStyleSheets(QMap<QString, QString> &styleSheets)
 	foreach (fileInfo, fileInfoList) {
 		if (fileInfo.isFile()) {
 			QString name = fileInfo.baseName();
-			styleSheets.insert(QString("%1 (%2)").arg(name, tr("built-in")), ":" + name);
+			styleSheets.insert(QString(" %1 (%2)").arg(name, tr("built-in")), ":" + name);//Add space to name to get them up because QMap sort by Key.
 		}
 	}
 	fileInfoList = QDir(QString::fromUtf8(RsAccounts::ConfigDirectory().c_str()) + "/qss/").entryInfoList(QStringList("*.qss"));
@@ -864,7 +886,7 @@ Rshare::createDataDirectory(QString *errmsg)
 }
 
 /** Set Rshare's data directory - externally */
-bool Rshare::setConfigDirectory(QString dir)
+bool Rshare::setConfigDirectory(const QString& dir)
 {
   useConfigDir = true;
   configDir = dir;
@@ -930,30 +952,42 @@ bool Rshare::loadCertificate(const RsPeerId &accountId, bool autoLogin)
 	}
 
 	std::string lockFile;
-	int retVal = RsInit::LockAndLoadCertificates(autoLogin, lockFile);
-	switch (retVal) {
-		case 0:	break;
-		case 1:	QMessageBox::warning(	0,
-										QObject::tr("Multiple instances"),
-										QObject::tr("Another RetroShare using the same profile is "
-										"already running on your system. Please close "
-										"that instance first\n Lock file:\n") +
-										QString::fromUtf8(lockFile.c_str()));
-				return false;
-		case 2:	QMessageBox::critical(	0,
-										QObject::tr("Multiple instances"),
-										QObject::tr("An unexpected error occurred when Retroshare "
-										"tried to acquire the single instance lock\n Lock file:\n") +
-										QString::fromUtf8(lockFile.c_str()));
-                return false;
-        case 3:
-//		case 3: QMessageBox::critical(	0,
-//										QObject::tr("Login Failure"),
-//										QObject::tr("Maybe password is wrong") );
-				return false;
-		default: std::cerr << "Rshare::loadCertificate() unexpected switch value " << retVal << std::endl;
-				return false;
-	}
+    RsInit::LoadCertificateStatus retVal = RsInit::LockAndLoadCertificates(autoLogin, lockFile);
+
+    switch (retVal)
+    {
+    case RsInit::OK:
+        break;
+    case RsInit::ERR_ALREADY_RUNNING:	QMessageBox::warning(	nullptr,
+                                                            QObject::tr("Multiple instances"),
+                                                            QObject::tr("Another RetroShare using the same profile is "
+                                                                        "already running on your system. Please close "
+                                                                        "that instance first\n Lock file:\n") +
+                                                            QString::fromUtf8(lockFile.c_str()));
+        return false;
+    case RsInit::ERR_CANT_ACQUIRE_LOCK:	QMessageBox::critical(	nullptr,
+                                                            QObject::tr("Multiple instances"),
+                                                            QObject::tr("An unexpected error occurred when Retroshare "
+                                                                        "tried to acquire the single instance lock\n Lock file:\n") +
+                                                            QString::fromUtf8(lockFile.c_str()));
+        return false;
+    case RsInit::ERR_CERT_CRYPTO_IS_TOO_WEAK: QMessageBox::critical(	nullptr,
+                                                            QObject::tr("Old certificate"),
+                                                            QObject::tr("This node uses old certificate settings that are considered too "
+                                                                        "weak by your current OpenSSL library version. You need to create a new node "
+                                                                        "possibly using the same profile."));
+        return false;
+    case RsInit::ERR_CANNOT_CONFIGURE_TOR: QMessageBox::critical(	nullptr,
+                                                            QObject::tr("Tor error"),
+                                                            QObject::tr("Cannot run/configure Tor. Make sure it is installed on your system."));
+        return false;
+        //		case 3: QMessageBox::critical(	0,
+        //										QObject::tr("Login Failure"),
+        //										QObject::tr("Maybe password is wrong") );
+        return false;
+    default: std::cerr << "Rshare::loadCertificate() unexpected switch value " << retVal << std::endl;
+        return false;
+    }
 
 	return true;
 }
