@@ -31,17 +31,19 @@
 #include "gui/RetroShareLink.h"
 #include "util/ObjectPainter.h"
 #include "util/imageutil.h"
+
+#include "util/rsdebug.h"
 #include "util/rstime.h"
 
 #ifdef USE_CMARK
 //Include for CMark
-// This needs to be fixed: use system library if available, etc.
-#include <gui/../../../supportlibs/cmark/src/cmark.h>
-#include <gui/../../../supportlibs/cmark/src/node.h>
+#include <cmark.h>
 #endif
 
 #include <iostream>
 
+//#define DEBUG_SAVESPACE 1
+
 /**
  * The type of embedding we'd like to do
  */
@@ -279,7 +281,7 @@ void RsHtml::anchorStylesheetForImg(QDomDocument &/*doc*/, QDomElement &/*elemen
 		break;
 
 	case RetroShareLink::TYPE_CERTIFICATE:
-		styleSheet = "QPushButton{ border-image: url(:/images/btn_blue.png) ;border-width: 4;padding: 0px 6px;font-size: 12px;color: white;} QPushButton:hover{ border-image: url(:/images/btn_blue_hover.png) ;}";
+		styleSheet = "QPushButton{ border-image: url(:/images/btn_blue.png) ;border-width: 4;padding: 0px 6px;font-size: 12pt;color: white;} QPushButton:hover{ border-image: url(:/images/btn_blue_hover.png) ;}";
 		break;
 	}
 }
@@ -332,6 +334,29 @@ void RsHtml::replaceAnchorWithImg(QDomDocument &doc, QDomElement &element, QText
 	element.appendChild(img);
 }
 
+void RsHtml::filterEmbeddedImages(QDomDocument &doc, QDomElement &currentElement)
+{
+	QDomNodeList children = currentElement.childNodes();
+	for(uint index = 0; index < (uint)children.length(); index++) {
+		QDomNode node = children.item(index);
+		if(node.isElement()) {
+			QDomElement element = node.toElement();
+			if(element.tagName().toLower() == "img") {
+				if(element.hasAttribute("src")) {
+					QString src = element.attribute("src");
+					// Do not allow things in the image source, except these:
+					// :/          internal resource		needed for emotes
+					// data:image  base64 embedded image	needed for stickers
+					if(!src.startsWith(":/") && !src.startsWith("data:image", Qt::CaseInsensitive)) {
+						element.setAttribute("src", ":/images/imageblocked_24.png");
+					}
+				}
+			}
+			filterEmbeddedImages(doc, element);
+		}
+	}
+}
+
 int RsHtml::indexInWithValidation(QRegExp &rx, const QString &text, EmbedInHtml &embedInfos, int pos)
 {
 	int index = rx.indexIn(text, pos);
@@ -556,18 +581,24 @@ static QString saveSpace(const QString text)
 		if(cursChar==QLatin1Char('>'))         {
 			if(!echapChar && i>0) {outBrackets=true; firstOutBracket=true;}
  		} else if(cursChar==QLatin1Char('\t')) {
-			if(outBrackets && firstOutBracket && (keyName!="style")) savedSpaceText.replace(i, 1, "&nbsp;&nbsp;");
+			if(outBrackets && firstOutBracket && (keyName!="style")) { savedSpaceText.replace(i, 1, "&nbsp;&nbsp;"); i+= 11; }
 		} else if(cursChar==QLatin1Char(' '))  {
-			if(outBrackets && firstOutBracket && (keyName!="style")) savedSpaceText.replace(i, 1, "&nbsp;");
+			if(outBrackets && firstOutBracket && (keyName!="style")) { savedSpaceText.replace(i, 1, "&nbsp;"); i+= 5; }
 		} else if(cursChar==QChar(0xA0))  {
-			if(outBrackets && firstOutBracket && (keyName!="style")) savedSpaceText.replace(i, 1, "&nbsp;");
+			if(outBrackets && firstOutBracket && (keyName!="style")) { savedSpaceText.replace(i, 1, "&nbsp;"); i+= 5; }
 		} else if(cursChar==QLatin1Char('<'))  {
 			if(!echapChar) {outBrackets=false; getKeyName=true; keyName.clear();}
 		} else firstOutBracket=false;
 		echapChar=(cursChar==QLatin1Char('\\'));
 
 	}
-	
+#ifdef DEBUG_SAVESPACE
+	RsDbg() << __PRETTY_FUNCTION__ << "Text to save:" << std::endl
+	        << text.toStdString() << std::endl
+	        << "---------------------- Saved Text:" << std::endl
+	        << savedSpaceText.toStdString() << std::endl;
+#endif
+
 	return savedSpaceText;
 }
 
@@ -593,7 +624,7 @@ QString RsHtml::formatText(QTextDocument *textDocument, const QString &text, ulo
 		// Parse CommonMark
 		int options = CMARK_OPT_DEFAULT;
 		cmark_parser *parser = cmark_parser_new(options);
-		cmark_parser_feed(parser, formattedText.toStdString().c_str(),formattedText.length());
+		cmark_parser_feed(parser, formattedText.toStdString().c_str(),static_cast<size_t>(formattedText.length()));
 		cmark_node *document = cmark_parser_finish(parser);
 		cmark_parser_free(parser);
 		char *result;
@@ -601,7 +632,6 @@ QString RsHtml::formatText(QTextDocument *textDocument, const QString &text, ulo
 		// Get result as html
 		formattedText = QString::fromUtf8(result);
 		//Clean
-		cmark_node_mem(document)->free(result);
 		cmark_node_free(document);
 		//Get document formed HTML
 		textBrowser.setHtml(formattedText);
@@ -611,13 +641,17 @@ QString RsHtml::formatText(QTextDocument *textDocument, const QString &text, ulo
 
 	QString errorMsg; int errorLine; int errorColumn;
 
-  QDomDocument doc;
+	QDomDocument doc;
 	if (doc.setContent(formattedText, &errorMsg, &errorLine, &errorColumn) == false) {
 
-		// convert text with QTextBrowser
-		QTextBrowser textBrowser;
-		textBrowser.setText(text);
-		formattedText=textBrowser.toHtml();
+		// convert text with QTextDocument
+		QTextDocument textDoc;
+		if (Qt::mightBeRichText(formattedText))
+			textDoc.setHtml(formattedText);
+		else
+			textDoc.setPlainText(text);
+
+		formattedText=textDoc.toHtml();
 		formattedText.remove(0,formattedText.indexOf("<"));
 		formattedText=saveSpace(formattedText);
 		doc.setContent(formattedText, &errorMsg, &errorLine, &errorColumn);
@@ -625,6 +659,7 @@ QString RsHtml::formatText(QTextDocument *textDocument, const QString &text, ulo
 	}
 
 	QDomElement body = doc.documentElement();
+	filterEmbeddedImages(doc, body);	// This should be first, becuse it should not overwrite embedded custom smileys
 	if (flag & RSHTML_FORMATTEXT_EMBED_SMILEYS) {
 		embedHtml(textDocument, doc, body, defEmbedImg, flag);
 	}
@@ -1027,9 +1062,10 @@ static void styleCreate(QDomDocument& doc
 		it.next();
 		const QStringList& classUsingIt ( it.value()) ;
 		bool first = true;
+		QString classNames = "";
 		foreach(QString className, classUsingIt) {
 			if (!className.trimmed().isEmpty()) {
-				style += QString(first?".":",.") + className;// + " ";
+				classNames += QString(first?".":",.") + className;// + " ";
 				first = false;
 			}
 		}
@@ -1076,9 +1112,9 @@ static void styleCreate(QDomDocument& doc
 			}
 
 			//.S1 .S2 .S4 {font-family:'Sans';}
-			style += "{" + key + ":" + val + ";}";
+			style += classNames + "{" + key + ":" + val + ";}";
 		} else {
-			style += "{" + it.key() + ";}\n";
+			style += classNames + "{" + it.key() + ";}\n";
 		}
 	}
 
@@ -1180,7 +1216,7 @@ bool RsHtml::makeEmbeddedImage(const QString &fileName, QString &embeddedImage,
 		fprintf (stderr, "RsHtml::makeEmbeddedImage() - image \"%s\" can't be load\n", fileName.toLatin1().constData());
 		return false;
 	}
-	return RsHtml::makeEmbeddedImage(image, embeddedImage, maxPixels, maxBytes);
+    return RsHtml::makeEmbeddedImage(image, embeddedImage, maxPixels, maxBytes);
 }
 
 /** Converts image to embedded image HTML fragment **/
@@ -1188,7 +1224,7 @@ bool RsHtml::makeEmbeddedImage(const QImage &originalImage, QString &embeddedIma
 {
 	rstime::RsScopeTimer s("Embed image");
 	QImage opt;
-	return ImageUtil::optimizeSize(embeddedImage, originalImage, opt, maxPixels, maxBytes);
+    return ImageUtil::optimizeSizeHtml(embeddedImage, originalImage, opt, maxPixels, maxBytes);
 }
 
 QString RsHtml::plainText(const QString &text)
@@ -1218,8 +1254,10 @@ QString RsHtml::makeQuotedText(RSTextBrowser *browser)
 	}
 	QStringList sl = text.split(QRegExp("[\r\n]"),QString::SkipEmptyParts);
 	text = sl.join("\n> ");
+	text.replace("\n> >","\n>>"); // Don't add space for already quotted lines.
 	text.replace(QChar(-4)," ");//Char used when image on text.
-	return QString("> ") + text;
+	QString quote = (text.left(1) == ">") ? QString(">") : QString("> ");
+	return quote + text;
 }
 
 void RsHtml::insertSpoilerText(QTextCursor cursor)
